@@ -38,9 +38,11 @@
 #include "gretl_xml.h"
 #include "gretl_string_table.h"
 #include "dbread.h"
-#include "gretl_www.h"
 #include "uservar.h"
 #include "csvdata.h"
+#ifdef USE_CURL
+# include "gretl_www.h"
+#endif
 
 #ifdef WIN32
 # include <windows.h>
@@ -67,12 +69,12 @@ int data_status;
 char linebak[MAXLINE];      /* for storing comments */
 char *line_read;
 
-static int exec_line (ExecState *s, DATASET *dset, PRN *cmdprn);
+static int cli_exec_line (ExecState *s, DATASET *dset, PRN *cmdprn);
 static int push_input_file (FILE *fp);
 static FILE *pop_input_file (void);
-static int saved_object_action (const char *line, 
-				DATASET *dset, 
-				PRN *prn);
+static int cli_saved_object_action (const char *line, 
+				    DATASET *dset, 
+				    PRN *prn);
 
 static void usage (int err)
 {
@@ -340,10 +342,8 @@ static void handle_datafile (char *filearg, char *runfile,
     case GRETL_RATS_DB:
 	exit(EXIT_FAILURE);
 	break;
-    case GRETL_NATIVE_DATA:
-	err = gretl_get_data(datafile, dset, OPT_NONE, prn);
-	break;
     case GRETL_XML_DATA:
+    case GRETL_BINARY_DATA:
 	err = gretl_read_gdt(datafile, dset, OPT_NONE, prn);
 	break;
     case GRETL_CSV:
@@ -518,7 +518,7 @@ int main (int argc, char *argv[])
     }
 
 #ifdef WIN32
-    cli_read_registry(callname);
+    win32_cli_read_rc(callname);
 #else
     cli_read_rc();
 #endif /* WIN32 */
@@ -578,7 +578,10 @@ int main (int argc, char *argv[])
 	    set_gretl_echo(0);
 	} 
 	sprintf(line, "run %s\n", runfile);
-	err = exec_line(&state, dset, cmdprn);
+	err = cli_exec_line(&state, dset, cmdprn);
+	if (err && fb == NULL) {
+	    exit(EXIT_FAILURE);
+	}
     }
 
     *linecopy = '\0';
@@ -615,7 +618,7 @@ int main (int argc, char *argv[])
 
 	strcpy(linecopy, line);
 	tailstrip(linecopy);
-	err = exec_line(&state, dset, cmdprn);
+	err = cli_exec_line(&state, dset, cmdprn);
     } /* end of get commands loop */
 
     if (!err) {
@@ -628,7 +631,7 @@ int main (int argc, char *argv[])
     if (makepkg && !err) {
 	switch_ext(filearg, runfile, "gfn");
 	sprintf(line, "makepkg %s\n", filearg);
-	exec_line(&state, dset, cmdprn);
+	cli_exec_line(&state, dset, cmdprn);
     }
 
     /* leak check -- try explicitly freeing all memory allocated */
@@ -699,10 +702,15 @@ static int cli_try_http (const char *s, char *fname, int *http)
     s += strspn(s, " ");
 
     if (strncmp(s, "http://", 7) == 0) {
+#ifdef USE_CURL
 	err = retrieve_public_file(s, fname);
 	if (!err) {
 	    *http = 1;
-	} 
+	}
+#else
+	gretl_errmsg_set(_("Internet access not supported"));
+	err = E_DATA;
+#endif
     }
 
     return err;
@@ -735,7 +743,7 @@ static int cli_open_append (CMD *cmd, const char *line,
 	    return err;
 	}
     }
-    
+
     if (opt & OPT_W) {
 	ftype = GRETL_NATIVE_DB_WWW;
     } else if (opt & OPT_O) {
@@ -761,22 +769,22 @@ static int cli_open_append (CMD *cmd, const char *line,
 
     if (!dbdata && cmd->ci != APPEND) {
 	cli_clear_data(cmd, dset, model);
-    } 
+    }
 
     if (opt & OPT_Q) {
 	/* --quiet, but in case we hit any problems below... */
 	vprn = gretl_print_new(GRETL_PRINT_BUFFER, NULL);
-    } 
+    }
 
-    if (ftype == GRETL_CSV) {
+    if (ftype == GRETL_XML_DATA || ftype == GRETL_BINARY_DATA) {
+	err = gretl_read_gdt(newfile, dset, opt, vprn);
+    } else if (ftype == GRETL_CSV) {
 	err = import_csv(newfile, dset, opt, vprn);
     } else if (SPREADSHEET_IMPORT(ftype)) {
 	err = import_spreadsheet(newfile, ftype, cmd->list, cmd->parm2,
 				 dset, opt, vprn);
     } else if (OTHER_IMPORT(ftype)) {
 	err = import_other(newfile, ftype, dset, opt, vprn);
-    } else if (ftype == GRETL_XML_DATA) {
-	err = gretl_read_gdt(newfile, dset, opt, vprn);
     } else if (ftype == GRETL_ODBC) {
 	err = set_odbc_dsn(line, vprn);
     } else if (dbdata) {
@@ -859,15 +867,15 @@ static void maybe_save_session_output (const char *cmdfile)
 
 #define ENDRUN (NC + 1)
 
-/* exec_line: this is called to execute both interactive and script
-   commands.  Note that most commands get passed on to the libgretl
-   function gretl_cmd_exec(), but some commands that require special
-   action are dealt with here.
+/* cli_exec_line: this is called to execute both interactive and
+   script commands.  Note that most commands get passed on to the
+   libgretl function gretl_cmd_exec(), but some commands that require
+   special action are dealt with here.
 
    see also gui_exec_line() in gui2/library.c
 */
 
-static int exec_line (ExecState *s, DATASET *dset, PRN *cmdprn)
+static int cli_exec_line (ExecState *s, DATASET *dset, PRN *cmdprn)
 {
     char *line = s->line;
     CMD *cmd = s->cmd;
@@ -877,6 +885,10 @@ static int exec_line (ExecState *s, DATASET *dset, PRN *cmdprn)
     char runfile[MAXLEN];
     int err = 0;
 
+#if 0
+    fprintf(stderr, "cli_exec_line: '%s'\n", line);
+#endif
+
     if (gretl_compiling_function()) {
 	err = gretl_function_append_line(line);
 	if (err) {
@@ -894,7 +906,7 @@ static int exec_line (ExecState *s, DATASET *dset, PRN *cmdprn)
     if (!s->in_comment && !cmd->context && !gretl_if_state_false()) {
 	/* catch requests relating to saved objects, which are not
 	   really "commands" as such */
-	int action = saved_object_action(line, dset, prn);
+	int action = cli_saved_object_action(line, dset, prn);
 
 	if (action == OBJ_ACTION_INVALID) {
 	    return 1; /* action was faulty */
@@ -1135,7 +1147,7 @@ static int exec_line (ExecState *s, DATASET *dset, PRN *cmdprn)
     if (err) {
 	gretl_exec_state_uncomment(s);
     }
-    
+
     return err;
 }
 
