@@ -41,44 +41,11 @@
 # include "gretlwin32.h"
 #endif
 
-#define OPEN_DATA_ACTION(i)  (i == OPEN_DATA || \
-			      i == OPEN_GDT ||	\
-                              i == OPEN_CSV || \
-                              i == OPEN_OCTAVE || \
-                              i == OPEN_GNUMERIC || \
-	                      i == OPEN_XLS || \
-                              i == OPEN_WF1 || \
-                              i == OPEN_DTA || \
-	                      i == OPEN_SAV || \
-			      i == OPEN_SAS || \
-                              i == OPEN_JMULTI || \
-                              i == OPEN_ODS)
-
-#define APPEND_DATA_ACTION(i) (i == APPEND_DATA || \
-			       i == APPEND_GDT ||  \
-                               i == APPEND_CSV || \
-                               i == APPEND_OCTAVE || \
-                               i == APPEND_GNUMERIC || \
-                               i == APPEND_XLS || \
-                               i == APPEND_WF1 || \
-                               i == APPEND_DTA || \
-                               i == APPEND_SAV || \
-			       i == APPEND_SAS || \
-                               i == APPEND_JMULTI || \
-                               i == APPEND_ODS)
-
-#define EXPORT_ACTION(a,s) ((a == EXPORT_OCTAVE || \
-                             a == EXPORT_R || \
-                             a == EXPORT_CSV || \
-                             a == EXPORT_DAT || \
-                             a == EXPORT_JM) && s != FSEL_DATA_PRN)
-
-#define GDT_ACTION(i) (i == SAVE_DATA || \
-                       i == SAVE_DATA_AS || \
-                       i == SAVE_BOOT_DATA || \
-                       i == EXPORT_GDT || \
-                       i == OPEN_GDT || \
-                       i == APPEND_GDT)
+#define EXPORT_OTHER(a) (a == EXPORT_OCTAVE ||	\
+			 a == EXPORT_R ||	\
+			 a == EXPORT_CSV ||	\
+			 a == EXPORT_DAT ||	\
+			 a == EXPORT_JM)
 
 #define SET_DIR_ACTION(i) (i == SET_DIR || i == SET_WDIR || \
                            i == SET_FDIR || i == SET_DBDIR)
@@ -106,8 +73,6 @@ static struct extmap action_map[] = {
     { SAVE_GFN_SPEC,     ".spec" },
     { EXPORT_CSV,        ".csv" },
     { EXPORT_R,          ".R" },
-    { OPEN_OCTAVE,       ".m" },
-    { APPEND_OCTAVE,     ".m" },
     { EXPORT_OCTAVE,     ".m" },
     { EXPORT_DAT,        ".dat" },
     { SAVE_OUTPUT,       ".txt" },
@@ -116,24 +81,6 @@ static struct extmap action_map[] = {
     { SAVE_TEXT,         ".txt" },
     { OPEN_SCRIPT,       ".inp" },
     { OPEN_SESSION,      ".gretl" },
-    { OPEN_CSV,          ".csv" },
-    { APPEND_CSV,        ".csv" },
-    { OPEN_GNUMERIC,     ".gnumeric" },
-    { APPEND_GNUMERIC,   ".gnumeric" },
-    { OPEN_XLS,          ".xls" },
-    { APPEND_XLS,        ".xls" },
-    { OPEN_WF1,          ".wf1" },
-    { APPEND_WF1,        ".wf1" },
-    { OPEN_DTA,          ".dta" },
-    { APPEND_DTA,        ".dta" },
-    { OPEN_SAV,          ".sav" },
-    { APPEND_SAV,        ".sav" },
-    { OPEN_SAS,          ".xpt" },
-    { APPEND_SAS,        ".xpt" },
-    { OPEN_JMULTI,       ".dat" },
-    { APPEND_JMULTI,     ".dat" },
-    { OPEN_ODS,          ".ods" },
-    { APPEND_ODS,        ".ods" },
     { OPEN_RATS_DB,      ".rat" },
     { OPEN_PCGIVE_DB,    ".bn7" },
     { OPEN_GFN,          ".gfn" },
@@ -141,12 +88,20 @@ static struct extmap action_map[] = {
     { FILE_OP_MAX,       NULL }
 };
 
+static int gdtb_save;
+
 static const char *get_extension_for_action (int action, gpointer data)
 {
     const char *s = NULL;
 
-    if (GDT_ACTION(action)) {
+    if (gdtb_save) {
+	return ".gdtb";
+    }
+
+    if (action == EXPORT_GDT) {
 	return ".gdt";
+    } else if (action == EXPORT_GDTB) {
+	return ".gdtb";
     } else if (action == SAVE_GNUPLOT || action == SAVE_GRAPHIC) {
 	int ttype = gp_term_code(data, action);
 
@@ -220,10 +175,10 @@ static int post_process_savename (char *fname, int action, gpointer data)
 
 	if (ext != NULL && !has_suffix(fname, ext)) {
 	    /* We found a recommended extension, and it's not
-	       already present on fname; so we should probably
+	       already present on @fname; so we should probably
 	       append the extension -- unless perhaps we're
 	       looking at an action where there isn't exactly
-	       a canonical extension and fname already has some
+	       a canonical extension and @fname already has some
 	       sort of extension in place.
 	    */
 	    if (strcmp(ext, ".txt") && strcmp(ext, ".plt")) {
@@ -288,7 +243,6 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
     const gchar *cset;
     FILE *fp;
     gchar *buf;
-    gchar *trbuf;
 
     buf = textview_get_text(vwin->text);
     if (buf == NULL) {
@@ -303,15 +257,22 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
 	return;
     }
 
-    if (!g_get_charset(&cset)) {
-	/* UTF-8 minuses not wanted for locale */
-	strip_unicode_minus(buf);
-    }
+    if (action == SAVE_SCRIPT) {
+	/* don't mess with encoding */
+	system_print_buf(buf, fp);
+    } else {
+	gchar *trbuf;
 
-    trbuf = my_locale_from_utf8(buf);
-    if (trbuf != NULL) {
-	system_print_buf(trbuf, fp);
-	g_free(trbuf);
+	if (!g_get_charset(&cset)) {
+	    /* UTF-8 minuses not wanted for locale */
+	    strip_unicode_minus(buf);
+	}
+
+	trbuf = my_locale_from_utf8(buf);
+	if (trbuf != NULL) {
+	    system_print_buf(trbuf, fp);
+	    g_free(trbuf);
+	}
     }
 
     g_free(buf);
@@ -393,6 +354,13 @@ static void filesel_open_session (const char *fname)
     }
 }
 
+/* suggested_savename: pertains to the case where some data has
+   been imported from a file with name @fname and the user has
+   now chosen "Save data" (implicitly in native format). We'd
+   like to offer a suggestion for the name of the file to save.
+   A simple case would be, e.g. "foo.gdt" from imported "foo.xls".
+*/
+
 static char *suggested_savename (const char *fname)
 {
     const char *ss = strrchr(fname, SLASH);
@@ -519,10 +487,10 @@ file_selector_process_result (const char *in_fname, int action,
 	}
     } 
 
-    if (OPEN_DATA_ACTION(action)) {
+    if (action == OPEN_DATA) {
 	strcpy(tryfile, fname);
 	verify_open_data(NULL, action);
-    } else if (APPEND_DATA_ACTION(action)) {
+    } else if (action == APPEND_DATA) {
 	strcpy(tryfile, fname);
 	do_open_data(NULL, action);
     } else if (action == OPEN_SCRIPT) {
@@ -557,12 +525,14 @@ file_selector_process_result (const char *in_fname, int action,
 	return;
     }
 
-    if (EXPORT_ACTION(action, src) || action == EXPORT_GDT ||
+    if (EXPORT_OTHER(action) || 
+	action == EXPORT_GDT ||
+	action == EXPORT_GDTB ||
 	(action > END_SAVE_DATA && action < END_SAVE_OTHER)) {
 	/* saving CSV, graphs etc.; check overwrite */
 	quit = overwrite_stop(fname);
     } else if (action == SAVE_DATA_AS && strcmp(fname, datafile)) {
-	/* "saving as" over an existing gdt file */
+	/* check for "saving as" over an existing gdt file */
 	quit = overwrite_stop(fname);
     }
 
@@ -600,7 +570,9 @@ file_selector_process_result (const char *in_fname, int action,
 	err = do_save_markers(fname);
     } else if (action == SAVE_LABELS) {
 	err = do_save_labels(fname);
-    } else if (action == SET_PROG || action == SET_DIR) {
+    } else if (action == SET_PROG || 
+	       action == SET_DIR ||
+	       action == SET_OTHER) {
 	set_path_callback(data, fname);
     } else if (action == SET_WDIR) {
 	set_working_dir_callback(data, fname);
@@ -724,6 +696,22 @@ static void set_default_progs_path (GtkFileChooser *fsel)
 #endif
 }
 
+static void set_default_other_path (GtkFileChooser *fsel)
+{
+    char *home = getenv("HOME");
+
+    if (home != NULL) {
+#ifdef G_OS_WIN32
+	gchar *path = my_filename_to_utf8(home);
+
+	gtk_file_chooser_set_current_folder(fsel, path);
+	g_free(path);
+#else
+	gtk_file_chooser_set_current_folder(fsel, home);
+#endif
+    }
+}
+
 static void filesel_maybe_set_current_name (GtkFileChooser *filesel,
 					    int action, FselDataSrc src,
 					    gpointer data)
@@ -734,19 +722,23 @@ static void filesel_maybe_set_current_name (GtkFileChooser *filesel,
 	currname = suggested_savename(datafile);
 	gtk_file_chooser_set_current_name(filesel, currname);
 	g_free(currname);
-    } else if (EXPORT_ACTION(action, src) && *datafile != '\0') {
+    } else if (EXPORT_OTHER(action) && src != FSEL_DATA_PRN &&
+	       *datafile != '\0') {
+	/* formulate export name based on current datafile */
 	currname = suggested_exportname(datafile, action);
 	gtk_file_chooser_set_current_name(filesel, currname);
 	g_free(currname);
     } else if (action == SAVE_CMD_LOG) {
 	gtk_file_chooser_set_current_name(filesel, "session.inp");
-    } else if (action == SET_PROG || action == SET_DIR) {
+    } else if (action == SET_PROG || action == SET_DIR || action == SET_OTHER) {
 	currname = (gchar *) data;
 	if (currname != NULL && g_path_is_absolute(currname)) {
 	    gtk_file_chooser_set_filename(filesel, currname);
 	} else if (action == SET_PROG) {
 	    set_default_progs_path(filesel);
-	}	    
+	} else if (action == SET_OTHER) {
+	    set_default_other_path(filesel);
+	}
     } else if (action == SAVE_FUNCTIONS || 
 	       action == SAVE_FUNCTIONS_AS ||
 	       action == SAVE_GFN_SPEC) {
@@ -764,65 +756,63 @@ static void filesel_maybe_set_current_name (GtkFileChooser *filesel,
 
 static void filesel_add_filter (GtkWidget *filesel,
 				const char *desc, 
-				const char *pat,
-				int *maxlen)
+				const char *pat)
 {
     GtkFileFilter *filt = gtk_file_filter_new();
 
-    if (maxlen != NULL) {
-	int n = g_utf8_strlen(desc, -1);
-	
-	if (n > *maxlen) {
-	    *maxlen = n;
-	}
-    }
-
     gtk_file_filter_set_name(filt, _(desc));
     gtk_file_filter_add_pattern(filt, pat);
-
     maybe_upcase_filter_pattern(filt, pat);
 
     gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filt);
 }
 
+static void filesel_add_native_data_filter (GtkWidget *filesel)
+{
+    const char *desc = N_("Gretl datafiles (*.gdt, *.gdtb)");
+    GtkFileFilter *filt = gtk_file_filter_new();
+
+    gtk_file_filter_set_name(filt, _(desc));
+    gtk_file_filter_add_pattern(filt, "*.gdt");
+    gtk_file_filter_add_pattern(filt, "*.gdtb");
+
+    gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(filesel), filt);
+}
+
 static void filesel_set_filters (GtkWidget *filesel, int action,
-				 FselDataSrc src, gpointer data,
-				 int *maxlen)
+				 FselDataSrc src, gpointer data)
 {
     if (action == OPEN_DATA || action == APPEND_DATA) {
-	filesel_add_filter(filesel, N_("Gretl data files (*.gdt)"), "*.gdt", maxlen);
-	filesel_add_filter(filesel, N_("CSV files (*.csv)"), "*.csv", maxlen);
-	filesel_add_filter(filesel, N_("ASCII files (*.txt)"), "*.txt", maxlen);
-	filesel_add_filter(filesel, N_("Gnumeric files (*.gnumeric)"), "*.gnumeric", maxlen);
-	filesel_add_filter(filesel, N_("Open Document files (*.ods)"), "*.ods", maxlen);
-	filesel_add_filter(filesel, N_("Excel files (*.xls)"), "*.xls", maxlen);
-	filesel_add_filter(filesel, N_("Excel files (*.xlsx)"), "*.xlsx", maxlen);
-	filesel_add_filter(filesel, N_("Stata files (*.dta)"), "*.dta", maxlen);
-	filesel_add_filter(filesel, N_("Eviews files (*.wf1)"), "*.wf1", maxlen);
-	filesel_add_filter(filesel, N_("SPSS files (*.sav)"), "*.sav", maxlen);
-	filesel_add_filter(filesel, N_("SAS xport files (*.xpt)"), "*.xpt", maxlen);
-	filesel_add_filter(filesel, N_("Octave files (*.m)"), "*.m", maxlen);
-	filesel_add_filter(filesel, N_("JMulTi files (*.dat)"), "*.dat", maxlen);
-	filesel_add_filter(filesel, N_("all files (*.*)"), "*", maxlen);
-    } else if (action == OPEN_CSV || action == APPEND_CSV) {
-	filesel_add_filter(filesel, N_("CSV files (*.csv)"), "*.csv", maxlen);
-	filesel_add_filter(filesel, N_("ASCII files (*.txt)"), "*.txt", maxlen);
-	filesel_add_filter(filesel, N_("all files (*.*)"), "*", maxlen);
-    } else if (action == OPEN_XLS || action == APPEND_XLS) {
-	filesel_add_filter(filesel, N_("Excel files (*.xls)"), "*.xls", maxlen);
-	filesel_add_filter(filesel, N_("Excel files (*.xlsx)"), "*.xlsx", maxlen);
+	filesel_add_native_data_filter(filesel);
+	filesel_add_filter(filesel, N_("CSV files (*.csv)"), "*.csv");
+	filesel_add_filter(filesel, N_("ASCII files (*.txt)"), "*.txt");
+	filesel_add_filter(filesel, N_("Gnumeric files (*.gnumeric)"), "*.gnumeric");
+	filesel_add_filter(filesel, N_("Open Document files (*.ods)"), "*.ods");
+	filesel_add_filter(filesel, N_("Excel files (*.xls)"), "*.xls");
+	filesel_add_filter(filesel, N_("Excel files (*.xlsx)"), "*.xlsx");
+	filesel_add_filter(filesel, N_("Stata files (*.dta)"), "*.dta");
+	filesel_add_filter(filesel, N_("Eviews files (*.wf1)"), "*.wf1");
+	filesel_add_filter(filesel, N_("SPSS files (*.sav)"), "*.sav");
+	filesel_add_filter(filesel, N_("SAS xport files (*.xpt)"), "*.xpt");
+	filesel_add_filter(filesel, N_("Octave files (*.m)"), "*.m");
+	filesel_add_filter(filesel, N_("JMulTi files (*.dat)"), "*.dat");
+	filesel_add_filter(filesel, N_("all files (*.*)"), "*");
     } else if (action == OPEN_SCRIPT) {
-	filesel_add_filter(filesel, N_("gretl script files (*.inp)"), "*.inp", maxlen);
-	filesel_add_filter(filesel, N_("GNU R files (*.R)"), "*.R", maxlen);
-	filesel_add_filter(filesel, N_("gnuplot files (*.plt)"), "*.plt", maxlen);
-	filesel_add_filter(filesel, N_("GNU Octave files (*.m)"), "*.m", maxlen);
-	filesel_add_filter(filesel, N_("Python files (*.py)"), "*.py", maxlen);
+	filesel_add_filter(filesel, N_("gretl script files (*.inp)"), "*.inp");
+	filesel_add_filter(filesel, N_("GNU R files (*.R)"), "*.R");
+	filesel_add_filter(filesel, N_("gnuplot files (*.plt)"), "*.plt");
+	filesel_add_filter(filesel, N_("GNU Octave files (*.m)"), "*.m");
+	filesel_add_filter(filesel, N_("Python files (*.py)"), "*.py");
 	if (ox_support) {
-	    filesel_add_filter(filesel, N_("Ox files (*.ox)"), "*.ox", maxlen);
+	    filesel_add_filter(filesel, N_("Ox files (*.ox)"), "*.ox");
 	}
     } else if (action == OPEN_LABELS || action == OPEN_BARS) {
-	filesel_add_filter(filesel, N_("ASCII files (*.txt)"), "*.txt", maxlen);
-	filesel_add_filter(filesel, N_("all files (*.*)"), "*", maxlen);
+	filesel_add_filter(filesel, N_("ASCII files (*.txt)"), "*.txt");
+	filesel_add_filter(filesel, N_("all files (*.*)"), "*");
+    } else if (action == SAVE_DATA || action == SAVE_DATA_AS ||
+	       action == SAVE_BOOT_DATA) {
+	filesel_add_filter(filesel, N_("Gretl datafiles (*.gdt)"), "*.gdt");
+	filesel_add_filter(filesel, N_("Gretl binary datafiles (*.gdtb)"), "*.gdtb");
     } else {
 	GtkFileFilter *filter = get_file_filter(action, data);
 
@@ -838,29 +828,54 @@ static void remember_folder (GtkFileChooser *chooser, char *savedir)
     g_free(folder);
 }
 
-/* this is a hack to work around breakage in GTK 2.24.6 */
-
-static void resize_combo (GtkWidget *w, gpointer data)
+static void record_compress_level (GtkWidget *b, gpointer p)
 {
-    gint *ivals = data;
+    int level;
 
-    if (ivals[0]) {
-	return;
-    } else if (GTK_IS_COMBO_BOX(w)) {
-	gtk_widget_set_size_request(w, ivals[1] * 8, -1);
-	ivals[0] = 1;
-    } else if (GTK_IS_CONTAINER(w)) {
-	gtk_container_foreach(GTK_CONTAINER(w), resize_combo,
-			      data);
-    } 
+    level = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(b));
+    set_optval_int(STORE, OPT_Z, level);
 }
 
-static void fix_filter_combo_size (GtkWidget *filesel, int maxlen)
+static void add_compression_level_option (GtkWidget *filesel)
 {
-    GtkWidget *ca = gtk_dialog_get_content_area(GTK_DIALOG(filesel));
-    gint ivals[] = { 0, maxlen };
+    GtkWidget *hbox, *label, *spin;
+    guint64 dsetsize;
+    int deflt = 1;
 
-    gtk_container_foreach(GTK_CONTAINER(ca), resize_combo, ivals);
+    dsetsize = (dataset->v - 1) * dataset->n * 8;
+    if (dsetsize < 30 * 1024) {
+	/* default to no compression for small data */
+	deflt = 0;
+    }
+
+    hbox = gtk_hbox_new(FALSE, 5);
+    label = gtk_label_new(_("Compression level (0 = none)"));
+    spin = gtk_spin_button_new_with_range(0, 9, 1);
+
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), deflt);
+    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 0);
+    g_signal_connect(G_OBJECT(spin), "destroy", 
+		     G_CALLBACK(record_compress_level), NULL);
+
+    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
+    gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 5);
+    gtk_widget_show_all(hbox);
+    gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(filesel), 
+				      hbox);
+}
+
+static void check_native_save_filter (GtkWidget *filesel)
+{
+    GtkFileFilter *filter;
+    const gchar *filter_name;
+
+    /* we need to know if .gdt or .gdtb was chosen */
+	
+    filter = gtk_file_chooser_get_filter(GTK_FILE_CHOOSER(filesel));
+    filter_name = gtk_file_filter_get_name(filter);
+    if (strstr(filter_name, "gdtb") != NULL) {
+	gdtb_save = 1;
+    }
 }
 
 static void gtk_file_selector (int action, FselDataSrc src, 
@@ -872,22 +887,15 @@ static void gtk_file_selector (int action, FselDataSrc src,
     GtkFileChooserAction fsel_action;
     const gchar *okstr;
     int remember = get_keep_folder();
-    int max_filter_len = 0;
-    int *lenptr = NULL;
     gint response;
 
-    if (gtk_major_version == 2 &&
-	gtk_minor_version == 24 &&
-	gtk_micro_version == 6) {
-	/* broken GTK version, 2.24.6 */
-	lenptr = &max_filter_len;
-    }
+    gdtb_save = 0;
 
     if (SET_DIR_ACTION(action)) {
 	fsel_action = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
 	okstr = GTK_STOCK_OK;
 	remember = 0;
-    } else if (action == SET_PROG) {
+    } else if (action == SET_PROG || action == SET_OTHER) {
 	fsel_action = GTK_FILE_CHOOSER_ACTION_OPEN;
 	okstr = GTK_STOCK_OK;
 	remember = 0;
@@ -925,30 +933,40 @@ static void gtk_file_selector (int action, FselDataSrc src,
 					  okstr, GTK_RESPONSE_ACCEPT,
 					  NULL);
 
+    if (action == SAVE_DATA || 
+	action == SAVE_DATA_AS || 
+	action == EXPORT_GDT ||
+	action == EXPORT_GDTB) {
+	add_compression_level_option(filesel);
+    }
+
     gtk_dialog_set_default_response(GTK_DIALOG(filesel), GTK_RESPONSE_ACCEPT);
 
     if (SET_DIR_ACTION(action)) {
 	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(filesel), startdir);
     } else {
-	filesel_set_filters(filesel, action, src, data, lenptr);
+	if (action != SET_PROG && action != SET_OTHER) {
+	    filesel_set_filters(filesel, action, src, data);
+	}
 	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(filesel), startdir);
 	filesel_maybe_set_current_name(GTK_FILE_CHOOSER(filesel), action,
 				       src, data);
     }
 
-    if (max_filter_len > 0) {
-	fix_filter_combo_size(filesel, max_filter_len);
-    }
-
     response = gtk_dialog_run(GTK_DIALOG(filesel));
 
     if (response == GTK_RESPONSE_ACCEPT) {
 	gchar *fname;
-	
+
+	if (action == SAVE_DATA || action == SAVE_DATA_AS) {
+	    check_native_save_filter(filesel);
+	}
+
 	fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(filesel));
 	if (remember) {
 	    remember_folder(GTK_FILE_CHOOSER(filesel), savedir);
 	}
+
 	gtk_widget_destroy(filesel);
 	filesel = NULL;
 
