@@ -148,6 +148,7 @@ enum {
 static char *global_items[] = {
     N_("Save session"),
     N_("Arrange icons"),
+    N_("Add matrix..."),
     N_("Windows"),
     N_("Close window")
 };
@@ -265,6 +266,7 @@ static void make_short_label_string (char *targ, const char *src);
 static gui_obj *get_gui_obj_by_data (void *finddata);
 static int gui_user_var_callback (const char *name, GretlType type,
 				  int action);
+static void auto_view_session (void);
 
 static int session_graph_count;
 static int session_bundle_count;
@@ -714,7 +716,7 @@ void save_output_as_text_icon (windata_t *vwin)
 
 	session_add_icon(text, GRETL_OBJ_TEXT, ICON_ADD_SINGLE);
     } else if (autoicon_on()) {
-	view_session();
+	auto_view_session();
     }
 }
 
@@ -736,7 +738,7 @@ static int add_model_to_session (void *ptr, const char *name,
     } else if (iconlist != NULL) {
 	session_add_icon(mod, type, ICON_ADD_SINGLE);
     } else if (autoicon_on()) {
-	view_session();
+	auto_view_session();
     }
 
     return err;
@@ -777,7 +779,7 @@ real_add_graph_to_session (const char *fname, const char *grname,
     if (iconlist != NULL) {
 	session_add_icon(graph, type, ICON_ADD_SINGLE);
     } else if (autoicon_on()) {
-	view_session();
+	auto_view_session();
     }
 
     return ADD_OBJECT_OK;
@@ -1139,8 +1141,10 @@ void bundle_add_as_icon (GtkAction *action, gpointer p)
 	if (err) {
 	    gui_errmsg(err);
 	} else {
-	    if (autoicon_on() || show) {
+	    if (show) {
 		view_session();
+	    } else if (autoicon_on()) {
+		auto_view_session();
 	    }
 	    mark_session_changed();
 	    if (close_on_add(action)) {
@@ -1258,6 +1262,21 @@ static int set_session_dirname (const char *zdirname)
     return err;
 }
 
+static void maybe_absolutize_tryfile (void)
+{
+    if (!g_path_is_absolute(tryfile)) {
+	char *tmp, dirname[FILENAME_MAX];
+
+	*dirname = '\0';
+	tmp = getcwd(dirname, FILENAME_MAX - 1);
+	if (tmp != NULL) {
+	    tmp = g_strdup(tryfile);
+	    build_path(tryfile, dirname, tmp, NULL);
+	    g_free(tmp);
+	}
+    }	
+}
+
 /* note: the name of the file to be opened is in the global var
    'tryfile' */
 
@@ -1289,6 +1308,11 @@ gboolean do_open_session (void)
 
     fprintf(stderr, I_("\nReading session file %s\n"), tryfile);
 
+    /* we're about to change directory: if tryfile is not
+       an absolute path we'll lose track of it 
+    */
+    maybe_absolutize_tryfile();
+    
     gretl_chdir(gretl_dotdir());
     err = gretl_unzip_session_file(tryfile, &zdirname, &gerr);
 
@@ -1385,7 +1409,7 @@ gboolean do_open_session (void)
 	    err = dataset_resample(dataset, sinfo.resample_n, sinfo.seed); 
 	} else if (sinfo.mask != NULL) {
 	    err = restrict_sample_from_mask(sinfo.mask, dataset,
-					    OPT_NONE);
+					    OPT_NONE, NULL);
 	    if (!err) {
 		dataset->restriction = sinfo.restriction;
 		sinfo.restriction = NULL;
@@ -1455,18 +1479,12 @@ void verify_clear_data (void)
 static void remove_session_dir (void)
 {
     const char *dotdir = gretl_dotdir();
-
-#ifdef G_OS_WIN32
     char *fullpath = g_strdup_printf("%s%s", dotdir,
 				     session.dirname);
 
     gretl_chdir(dotdir);
-    win32_delete_dir(fullpath);
+    gretl_deltree(fullpath);
     g_free(fullpath);
-#else
-    gretl_chdir(dotdir);
-    gretl_deltree(session.dirname);
-#endif
 }
 
 void session_init (void)
@@ -1621,6 +1639,7 @@ static void session_clear_data (DATASET *pdinfo)
 
 void close_session (gretlopt opt)
 {
+    int preserve = (opt & OPT_P)? 1 : 0;
     int logcode = LOG_NULL;
     int iview = 0;
 
@@ -1651,12 +1670,12 @@ void close_session (gretlopt opt)
     session.show_notes = 0;
     commands_recorded = 0;
 
-    close_session_windows();
+    close_session_windows(opt);
     selector_cleanup();
     function_call_cleanup();
     edit_dialog_special_get_text(NULL);
 
-    if (opt & OPT_P) {
+    if (preserve) {
 	/* preserve non-dataset items */
 	libgretl_session_cleanup(SESSION_CLEAR_DATASET);
     } else {
@@ -1718,13 +1737,15 @@ static int real_save_session_dataset (const char *dname)
 
     if (!err) {
 	session_file_make_path(tmpname, dname);
-	write_err = gretl_write_gdt(tmpname, NULL, dataset, OPT_NONE, 1);
+	write_err = gretl_write_gdt(tmpname, NULL, dataset, 
+				    OPT_NONE, 1);
     }
 
     if (mask != NULL) {
 	/* reset the prior subsample */
 	if (!err) {
-	    err = restrict_sample_from_mask(mask, dataset, OPT_NONE);
+	    err = restrict_sample_from_mask(mask, dataset, 
+					    OPT_NONE, NULL);
 	}
 	free(mask);
     }
@@ -2879,9 +2900,9 @@ static int gui_user_var_callback (const char *name, GretlType type,
 		session_add_icon(u, otype, ICON_ADD_SINGLE);
 	    }
 	} else if (autoicon_on()) {
-	    view_session();
+	    auto_view_session();
 	}
-	if (waiting_for_output()) {
+	if (iconview != NULL && waiting_for_output()) {
 	    gtk_widget_set_sensitive(iconview, FALSE);
 	}
 	mark_session_changed();
@@ -2935,8 +2956,8 @@ static void add_user_var_icon (gpointer data, gpointer intp)
 
 static void add_all_icons (void) 
 {
+    int show_graph_page = check_for_program(latex);
     GList *list = NULL;
-    int show_graph_page = check_for_prog(latex);
     int i;
 
     active_object = NULL;
@@ -3187,6 +3208,8 @@ static void global_popup_callback (GtkWidget *widget, gpointer data)
 	}
     } else if (!strcmp(item, _("Arrange icons"))) {
 	rearrange_icons();
+    } else if (!strcmp(item, _("Add matrix..."))) {
+	gui_new_matrix(iconview);
     } else if (!strcmp(item, _("Windows"))) {
 	window_list_popup(widget, NULL, iconview);
     } else if (!strcmp(item, _("Close window"))) {
@@ -3782,6 +3805,25 @@ void view_session (void)
     gtk_widget_grab_focus(icon_table);
 }
 
+static int view_session_deferred;
+
+static void auto_view_session (void)
+{
+    if (waiting_for_output()) {
+	view_session_deferred = 1;
+    } else {
+	view_session();
+    }
+}
+
+void maybe_view_session (void)
+{
+    if (view_session_deferred) {
+	view_session_deferred = 0;
+	view_session();
+    }
+}
+
 static void make_short_label_string (char *targ, const char *src)
 {
     if (g_utf8_strlen(src, -1) > SHOWNAMELEN) {
