@@ -97,6 +97,18 @@ void cursor_to_top (windata_t *vwin)
     gtk_text_view_scroll_to_mark(view, mark, 0.0, FALSE, 0, 0);
 }
 
+void scroll_to_foot (windata_t *vwin)
+{
+    GtkTextView *view = GTK_TEXT_VIEW(vwin->text);
+    GtkTextBuffer *buf = gtk_text_view_get_buffer(view); 
+    GtkTextIter end;
+    GtkTextMark *mark;
+
+    gtk_text_buffer_get_end_iter(buf, &end);
+    mark = gtk_text_buffer_create_mark(buf, NULL, &end, FALSE);
+    gtk_text_view_scroll_to_mark(view, mark, 0.0, FALSE, 0, 0);
+}
+
 void cursor_to_mark (windata_t *vwin, GtkTextMark *mark)
 {
     GtkTextView *view = GTK_TEXT_VIEW(vwin->text);
@@ -984,6 +996,42 @@ void textview_append_text_colorized (GtkWidget *view, const char *buf, int trim)
     real_textview_add_colorized(view, buf, 1, trim);
 }
 
+void textview_delete_processing_message (GtkWidget *view)
+{
+    GtkTextBuffer *tbuf;
+    GtkTextMark *mark;
+    GtkTextIter i0, i1; 
+
+    g_return_if_fail(GTK_IS_TEXT_VIEW(view));
+
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
+    mark = gtk_text_buffer_get_mark(tbuf, "procmark");
+    if (mark != NULL) {
+	gtk_text_buffer_get_iter_at_mark(tbuf, &i0, mark);
+	gtk_text_buffer_get_end_iter(tbuf, &i1);
+	gtk_text_buffer_delete(tbuf, &i0, &i1);
+    }
+}
+
+void textview_add_processing_message (GtkWidget *view)
+{
+    const char *msg = N_("processing...\n");
+    GtkTextBuffer *tbuf;
+    GtkTextIter iter;
+    GtkTextMark *mark;
+
+    g_return_if_fail(GTK_IS_TEXT_VIEW(view));
+
+    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
+    gtk_text_buffer_get_end_iter(tbuf, &iter);
+    mark = gtk_text_buffer_create_mark(tbuf, "procmark", &iter, TRUE); 
+    gtk_text_buffer_insert_with_tags_by_name(tbuf, &iter,
+					     _(msg), -1,
+					     "redtext", NULL);
+    gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(view), mark, 0.0, 
+				 FALSE, 0, 0);
+}
+
 void textview_insert_file (windata_t *vwin, const char *fname)
 {
     FILE *fp;
