@@ -33,6 +33,7 @@
 #include "fileselect.h"
 #include "winstack.h"
 #include "tabwin.h"
+#include "fncall.h"
 #include "toolbar.h"
 
 #include "uservar.h"
@@ -55,6 +56,7 @@
 #include "../pixmaps/mini.split_v.xpm"
 #include "../pixmaps/mini.winlist.xpm"
 #include "../pixmaps/mini.spreadsheet.xpm"
+#include "../pixmaps/mini.bundle.xpm"
 
 /* for main-window toolbar */
 #include "../pixmaps/mini.calc.xpm"
@@ -102,8 +104,9 @@ enum {
     SPLIT_ITEM,
     EDITOR_ITEM,
     NOTES_ITEM,
-    STOP_ITEM,
-    NEW_ITEM
+    NEW_ITEM,
+    CLOSE_ITEM,
+    BUNDLE_ITEM
 } viewbar_flags;
 
 struct stock_maker {
@@ -133,6 +136,7 @@ void gretl_stock_icons_init (void)
 	{ mini_split_v_xpm, GRETL_STOCK_SPLIT_V },
 	{ mini_winlist_xpm, GRETL_STOCK_WINLIST },
 	{ mini_spreadsheet_xpm, GRETL_STOCK_SHEET },
+	{ mini_bundle_xpm, GRETL_STOCK_BUNDLE },
 	{ mini_db_xpm, GRETL_STOCK_DB},
 	{ mini_gretl_xpm, GRETL_STOCK_GRETL},
 	{ mini_table_xpm, GRETL_STOCK_TABLE},
@@ -558,23 +562,23 @@ static void coeffint_set_alpha (GtkWidget *w, windata_t *vwin)
     gtk_widget_show_all(dialog);
 }
 
-static void set_output_sticky (GtkWidget *w, windata_t *vwin)
+static void set_output_stickiness (GtkWidget *w, windata_t *vwin)
 {
     const char *opts[] = {
-	N_("Running a script replaces text"),
-	N_("Running a script adds to text")
+	N_("Always goes to a new window"),
+	N_("Replaces output in this window"),
+	N_("Adds to output in this window")
     };
+    const char *label = N_("New script output:");
     int resp, deflt;
 
-    deflt = (vwin->flags & VWIN_STICKY)? 1 : 0;
+    deflt = get_script_output_policy();
 
-    resp = radio_dialog(NULL, NULL, opts, 2, deflt, 0, 
+    resp = radio_dialog(NULL, label, opts, 3, deflt, 0, 
 			vwin_toplevel(vwin));
 
-    if (resp == 0) {
-	vwin->flags &= ~VWIN_STICKY;
-    } else if (resp == 1) {
-	vwin->flags |= VWIN_STICKY;
+    if (resp != GRETL_CANCEL) {
+	set_script_output_policy(resp, vwin);
     }
 }
 
@@ -582,11 +586,27 @@ static void toolbar_plot_callback (GtkWidget *w, windata_t *vwin)
 {
     if (vwin->role == VIEW_SERIES) {
 	series_view_graph(w, vwin);
+    } else if (vwin->role == VIEW_BUNDLE) {
+	exec_bundle_plot_function(vwin->data, NULL);
     } else {
 	do_nonparam_plot(vwin);
     }
 }
 
+static int bundle_plot_ok (windata_t *vwin)
+{
+    gretl_bundle *b = vwin->data;
+    gchar *pf = get_bundle_plot_function(b);
+    int ret = 0;
+
+    if (pf != NULL) {
+	ret = 1;
+	g_free(pf);
+    }
+
+    return ret;
+}
+
 static void activate_script_help (GtkWidget *widget, windata_t *vwin)
 {
     text_set_cursor(vwin->text, GDK_QUESTION_ARROW);
@@ -622,7 +642,7 @@ static int edit_script_popup_item (GretlToolItem *item)
 
 static void set_plot_icon (GretlToolItem *item, int role)
 {
-    if (role == LOESS || role == NADARWAT) {
+    if (role == LOESS || role == NADARWAT || role == VIEW_BUNDLE) {
 	item->icon = GRETL_STOCK_SCATTER;
     } else if (dataset_is_time_series(dataset)) {
 	item->icon = GRETL_STOCK_TS;
@@ -643,10 +663,10 @@ static GretlToolItem viewbar_items[] = {
     { N_("Open..."), GTK_STOCK_OPEN, G_CALLBACK(file_open_callback), OPEN_ITEM },
     { N_("Save"), GTK_STOCK_SAVE, G_CALLBACK(vwin_save_callback), SAVE_ITEM },
     { N_("Save as..."), GTK_STOCK_SAVE_AS, G_CALLBACK(save_as_callback), SAVE_AS_ITEM },
+    { N_("Save bundle content..."), GRETL_STOCK_BUNDLE, NULL, BUNDLE_ITEM },
     { N_("Print..."), GTK_STOCK_PRINT, G_CALLBACK(window_print_callback), 0 },
     { N_("Show/hide"), GRETL_STOCK_PIN, G_CALLBACK(session_notes_callback), NOTES_ITEM },
     { N_("Run"), GTK_STOCK_EXECUTE, G_CALLBACK(do_run_script), EXEC_ITEM },
-    { N_("Stop"), GTK_STOCK_STOP, G_CALLBACK(do_stop_script), STOP_ITEM },
     { N_("Cut"), GTK_STOCK_CUT, G_CALLBACK(vwin_cut_callback), EDIT_ITEM }, 
     { N_("Copy"), GTK_STOCK_COPY, G_CALLBACK(vwin_copy_callback), COPY_ITEM }, 
     { N_("Paste"), GTK_STOCK_PASTE, G_CALLBACK(text_paste), EDIT_ITEM },
@@ -667,7 +687,7 @@ static GretlToolItem viewbar_items[] = {
     { N_("Refresh"), GTK_STOCK_REFRESH, G_CALLBACK(toolbar_refresh), REFRESH_ITEM },
     { N_("Add to dataset..."), GTK_STOCK_ADD, G_CALLBACK(add_data_callback), ADD_DATA_ITEM },
     { N_("Add as matrix..."), GTK_STOCK_ADD, G_CALLBACK(add_matrix_callback), ADD_MATRIX_ITEM },
-    { N_("Stickiness..."), GRETL_STOCK_PIN, G_CALLBACK(set_output_sticky), STICKIFY_ITEM },
+    { N_("Stickiness..."), GRETL_STOCK_PIN, G_CALLBACK(set_output_stickiness), STICKIFY_ITEM },
     { N_("Toggle split pane"), GRETL_STOCK_SPLIT_H, G_CALLBACK(split_pane_callback), SPLIT_ITEM },
     { N_("Toggle split pane"), GRETL_STOCK_SPLIT_V, G_CALLBACK(split_pane_callback), SPLIT_ITEM },
     { N_("Help on command"), GTK_STOCK_HELP, G_CALLBACK(activate_script_help), CMD_HELP_ITEM },
@@ -675,7 +695,7 @@ static GretlToolItem viewbar_items[] = {
     { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(display_gnuplot_help), GP_HELP_ITEM },
     { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(display_x12a_help), X12A_HELP_ITEM },
     { N_("Windows"), GRETL_STOCK_WINLIST, GNULL, 0 },
-    { N_("Close"), GTK_STOCK_CLOSE, G_CALLBACK(delete_file_viewer), 0 }
+    { N_("Close"), GTK_STOCK_CLOSE, G_CALLBACK(delete_file_viewer), CLOSE_ITEM }
 };
 
 static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
@@ -684,10 +704,6 @@ static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
 		    r == VIEW_SCRIPT || \
 	            r == EDIT_PKG_SAMPLE)
 
-#define stop_ok(r) (r == EDIT_SCRIPT || \
-		    r == VIEW_SCRIPT || \
-	            r == EDIT_PKG_SAMPLE)
-
 #define open_ok(r) (vwin_editing_script(r))
 
 #define new_ok(r) (vwin_editing_script(r))
@@ -701,7 +717,8 @@ static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
 	               r != EDIT_NOTES && \
 	               r != EDIT_PKG_CODE && \
 		       r != EDIT_PKG_SAMPLE && \
-	               r != CONSOLE)
+		       r != CONSOLE && \
+		       r != VIEW_BUNDLE)
 
 #define help_ok(r) (r == LEVERAGE || \
 		    r == COINT2 || \
@@ -718,30 +735,29 @@ static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
 #define sort_ok(r) (r == VIEW_SERIES)
 
 #define plot_ok(r) (r == VIEW_SERIES || \
-		    r == LOESS || r == NADARWAT)
+		    r == LOESS || \
+		    r == NADARWAT)
 
 #define add_data_ok(r) (r == PCA || r == LEVERAGE || \
                         r == MAHAL || r == FCAST || \
 			r == LOESS || r == NADARWAT)
 
-#define split_ok(r) (r == SCRIPT_OUT)
+#define split_ok(r) (r == SCRIPT_OUT || FNCALL_OUT)
 
 /* Screen out unwanted menu items depending on the context; also
    adjust the callbacks associated with some items based on
    context.
 */
 
-static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin, 
-				    int latex_ok, int sortby_ok,
-				    int format_ok, int save_ok)
+static GCallback tool_item_get_callback (GretlToolItem *item, windata_t *vwin, 
+					 int latex_ok, int sortby_ok,
+					 int format_ok, int save_ok)
 {
     GCallback func = item->func;
     int f = item->flag;
     int r = vwin->role;
 
-    if (!stop_ok(r) && f == STOP_ITEM) {
-	return NULL;
-    } else if (!edit_ok(r) && f == EDIT_ITEM) {
+    if (!edit_ok(r) && f == EDIT_ITEM) {
 	return NULL;
     } else if (!open_ok(r) && f == OPEN_ITEM) {
 	return NULL;
@@ -766,7 +782,11 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
     } else if (!sortby_ok && f == SORT_BY_ITEM) {
 	return NULL;
     } else if (!plot_ok(r) && f == PLOT_ITEM) {
-	return NULL;
+	if (r == VIEW_BUNDLE && bundle_plot_ok(vwin)) {
+	    ; /* alright then */
+	} else {
+	    return NULL;
+	}
     } else if (!split_ok(r) && f == SPLIT_ITEM) {
 	return NULL;
     } else if (!format_ok && f == FORMAT_ITEM) {
@@ -792,6 +812,8 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
 	return NULL;
     } else if (r != EDIT_NOTES && f == NOTES_ITEM) {
 	return NULL;
+    } else if (r != VIEW_BUNDLE && f == BUNDLE_ITEM) {
+	return NULL;
     } else if (f == SAVE_AS_ITEM) {
 	if (!save_as_ok(r)) {
 	    return NULL;
@@ -803,6 +825,32 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
     return func;
 }
 
+static GtkWidget *tool_item_get_menu (GretlToolItem *item, windata_t *vwin)
+{
+    GtkWidget *menu = NULL;
+
+    if (vwin->role == VIEW_BUNDLE) {
+	if (item->flag == BUNDLE_ITEM) {
+	    menu = make_bundle_content_menu(vwin);
+	} else if (item->flag == PLOT_ITEM) {
+	    menu = make_bundle_plot_menu(vwin);
+	} else if (item->flag == SAVE_ITEM) {
+	    menu = make_bundle_save_menu(vwin);
+	    if (menu != NULL) {
+		item->tip = N_("Save...");
+	    }
+	}
+    }
+
+    if (menu != NULL) {
+	/* don't leak: record pointer to menu so it can
+	   be destroyed when the window is closed */
+	vwin_record_toolbar_popup(vwin, menu);
+    }
+
+    return menu;
+}
+
 GtkWidget *gretl_toolbar_new (void)
 {
     GtkWidget *tb = gtk_toolbar_new();
@@ -819,17 +867,30 @@ void gretl_tooltips_add (GtkWidget *w, const gchar *str)
     gtk_widget_set_tooltip_text(w, str);
 }
 
-void vwin_toolbar_insert_winlist (windata_t *vwin)
+static GtkToolItem *gretl_menu_button (const char *icon,
+				       const char *tip,
+				       GtkWidget **pw)
 {
     GtkWidget *img, *button = gtk_button_new();
     GtkToolItem *item = gtk_tool_item_new();
 
-    gtk_widget_set_tooltip_text(GTK_WIDGET(item), _("Windows"));
+    gtk_widget_set_tooltip_text(GTK_WIDGET(item), _(tip));
     gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-    img = gtk_image_new_from_stock(GRETL_STOCK_WINLIST, 
-				   GTK_ICON_SIZE_MENU);
+    img = gtk_image_new_from_stock(icon, GTK_ICON_SIZE_MENU);
     gtk_container_add(GTK_CONTAINER(button), img);
     gtk_container_add(GTK_CONTAINER(item), button);
+    *pw = button;
+
+    return item;
+}
+
+void vwin_toolbar_insert_winlist (windata_t *vwin)
+{
+    GtkWidget *button;
+    GtkToolItem *item;
+
+    item = gretl_menu_button(GRETL_STOCK_WINLIST, N_("Windows"),
+			     &button);
     g_signal_connect(G_OBJECT(button), "button-press-event", 
 		     G_CALLBACK(vwin_winlist_popup), vwin);
     gtk_toolbar_insert(GTK_TOOLBAR(vwin->mbar), item, -1);
@@ -851,20 +912,57 @@ GtkWidget *gretl_toolbar_insert (GtkWidget *tbar,
     return GTK_WIDGET(item);
 }
 
+static void button_menu_pos (GtkMenu *menu,
+			     gint *x,
+			     gint *y,
+			     gboolean *push_in,
+			     gpointer data)
+{
+    GtkWidget *button = data;
+    gint wx, wy, tx, ty;
+
+    gdk_window_get_origin(gtk_widget_get_window(button), &wx, &wy);
+    gtk_widget_translate_coordinates(button, gtk_widget_get_toplevel(button), 
+				     0, 0, &tx, &ty);
+    *x = wx + tx;
+    *y = wy + ty + 26;
+    *push_in = TRUE;
+}
+
+static void tool_item_popup (GtkWidget *button, GdkEvent *event, 
+			     GtkWidget *menu)
+{
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, 
+		   button_menu_pos, button,
+		   event->button.button, event->button.time);
+}
+
 static GtkWidget *vwin_toolbar_insert (GretlToolItem *tool,
 				       GCallback func,
+				       GtkWidget *menu,
 				       windata_t *vwin,
 				       gint pos)
 {
     GtkToolItem *item;
 
-    item = gtk_tool_button_new_from_stock(tool->icon);
-    if (tool->flag == NEW_ITEM && window_is_tab(vwin)) {
-	gtk_widget_set_tooltip_text(GTK_WIDGET(item), _("New tab"));
+    if (menu != NULL) {
+	/* make and insert a button that pops down a menu */
+	GtkWidget *button;
+	
+	item = gretl_menu_button(tool->icon, tool->tip, &button);
+	g_signal_connect(G_OBJECT(button), "button-press-event", 
+			 G_CALLBACK(tool_item_popup), menu);
     } else {
-	gtk_widget_set_tooltip_text(GTK_WIDGET(item), _(tool->tip));
+	/* make and insert a regular callback button */
+	item = gtk_tool_button_new_from_stock(tool->icon);
+	g_signal_connect(G_OBJECT(item), "clicked", func, vwin);
+	if (tool->flag == NEW_ITEM && window_is_tab(vwin)) {
+	    gtk_widget_set_tooltip_text(GTK_WIDGET(item), _("New tab"));
+	} else {
+	    gtk_widget_set_tooltip_text(GTK_WIDGET(item), _(tool->tip));
+	}
     }
-    g_signal_connect(G_OBJECT(item), "clicked", func, vwin);
+
     gtk_toolbar_insert(GTK_TOOLBAR(vwin->mbar), item, pos);
 
     return GTK_WIDGET(item);
@@ -878,21 +976,34 @@ static void viewbar_add_items (windata_t *vwin, ViewbarFlags flags)
     int save_ok = (flags & VIEWBAR_EDITABLE);
     GtkWidget *hpane = NULL, *vpane = NULL;
     GtkWidget *button;
+    GtkWidget *menu;
     GretlToolItem *item;
     GCallback func;
     int i;
  
     for (i=0; i<n_viewbar_items; i++) {
+	func = NULL;
+	menu = NULL;
+
 	item = &viewbar_items[i];
 
 	if (winlist_item(item)) {
+	    /* the window-list item is special */
 	    vwin_toolbar_insert_winlist(vwin);
 	    continue;
 	}
 
-	func = item_get_callback(item, vwin, latex_ok, sortby_ok,
-				 format_ok, save_ok);
-	if (func == NULL) {
+	/* Is there anything to hook up, in context? We
+	   try first for a menu to attach to the toolbar
+	   button; failing that we test for a "direct"
+	   callback function.
+	*/
+	menu = tool_item_get_menu(item, vwin);
+	if (menu == NULL && item->func != NULL) {
+	    func = tool_item_get_callback(item, vwin, latex_ok, sortby_ok,
+					  format_ok, save_ok);
+	}
+	if (func == NULL && menu == NULL) {
 	    continue;
 	}
 
@@ -900,7 +1011,7 @@ static void viewbar_add_items (windata_t *vwin, ViewbarFlags flags)
 	    set_plot_icon(item, vwin->role);
 	}
 
-	button = vwin_toolbar_insert(item, func, vwin, -1);
+	button = vwin_toolbar_insert(item, func, menu, vwin, -1);
 
 	if (func == (GCallback) split_pane_callback) {
 	    if (hpane == NULL) {
@@ -911,7 +1022,7 @@ static void viewbar_add_items (windata_t *vwin, ViewbarFlags flags)
 	}
 
 	if (item->flag == SAVE_ITEM) { 
-	    if (vwin->role != CONSOLE) {
+	    if (vwin->role != CONSOLE && vwin->role != VIEW_BUNDLE) {
 		/* nothing to save just yet */
 		g_object_set_data(G_OBJECT(vwin->mbar), "save_button", button); 
 		gtk_widget_set_sensitive(button, FALSE);
@@ -921,10 +1032,6 @@ static void viewbar_add_items (windata_t *vwin, ViewbarFlags flags)
 	    if (strstr(vwin->fname, "script_tmp")) {
 		gtk_widget_set_sensitive(button, FALSE);
 	    }
-	} else if (item->flag == STOP_ITEM) {
-	    g_object_set_data(G_OBJECT(vwin->mbar), "stop_button", button);
-	    /* nothing to stop yet */
-	    gtk_widget_set_sensitive(button, FALSE);
 	}
     }
 
@@ -990,7 +1097,7 @@ GtkWidget *build_text_popup (windata_t *vwin)
 		func = NULL;
 	    }
 	} else {
-	    func = item_get_callback(item, vwin, 0, 0, 0, 0);
+	    func = tool_item_get_callback(item, vwin, 0, 0, 0, 0);
 	}
 	if (func != G_CALLBACK(NULL)) {
 	    if (func == G_CALLBACK(text_paste)) {
@@ -1133,3 +1240,28 @@ void add_mainwin_toolbar (GtkWidget *vbox)
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 }
 
+/* add a temporary top-bar for use in a script output
+   window, while we're waiting for the output */
+
+void vwin_add_tmpbar (windata_t *vwin)
+{
+    GretlToolItem item = {
+	N_("Stop"), 
+	GTK_STOCK_STOP, 
+	G_CALLBACK(do_stop_script), 
+	0
+    };
+    GtkWidget *hbox, *tmp;
+
+    hbox = gtk_hbox_new(FALSE, 0);
+    g_object_set_data(G_OBJECT(vwin->main), "top-hbox", hbox);
+
+    tmp = gretl_toolbar_new();
+    gretl_toolbar_insert(tmp, &item, item.func, NULL, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 5);
+
+    gtk_box_pack_start(GTK_BOX(vwin->vbox), hbox, FALSE, FALSE, 0);
+    start_wait_for_output(hbox);
+    gtk_widget_show_all(hbox);
+}
+
