@@ -150,6 +150,9 @@ wald_test (const int *list, MODEL *pmod, double *chisq, double *F)
 	gretl_matrix_print(b, "Wald coeff vector");
 #endif
 	err = gretl_invert_symmetric_matrix(C);
+	if (err) {
+	    fprintf(stderr, "wald_test: couldn't invert C\n");
+	}
 #if WDEBUG
 	gretl_matrix_print(C, "VCV-inverse");
 #endif
@@ -339,7 +342,12 @@ static void print_compare (struct COMPARE *cmp,
 			   gretlopt opt,
 			   PRN *prn)
 {
-    if (!(opt & OPT_Q) && cmp->model_id >= 0) {
+    if (opt & OPT_Q) {
+	if (cmp->model_id >= 0) {
+	    pprintf(prn, _("Test on Model %d:"), cmp->model_id);
+	    pputc(prn, '\n');
+	}
+    } else if (cmp->model_id >= 0) {
 	if (opt & OPT_A) {
 	    /* --auto-omit: add vertical space */
 	    pputc(prn, '\n');
@@ -947,20 +955,21 @@ real_nonlinearity_test (MODEL *pmod, int *list,
 	}
 
 	pval = chisq_cdf_comp(df, trsq);
-
 	aux.aux = aux_code;
 
-	if (opt & OPT_Q) {
-	    nonlin_test_header(aux_code, prn);
-	} else {
-	    printmodel(&aux, dset, opt, prn);
-	    pputc(prn, '\n');
+	if (!(opt & OPT_I)) {
+	    /* OPT_I = --silent */
+	    if (opt & OPT_Q) {
+		nonlin_test_header(aux_code, prn);
+	    } else {
+		printmodel(&aux, dset, opt, prn);
+		pputc(prn, '\n');
+	    }
+	    pprintf(prn, "  %s: TR^2 = %g,\n  ", _("Test statistic"), trsq);
+	    pprintf(prn, "%s = P(%s(%d) > %g) = %g\n\n", 
+		    _("with p-value"), _("Chi-square"), df, trsq, pval);
 	}
 
-	pprintf(prn, "  %s: TR^2 = %g,\n  ", _("Test statistic"), trsq);
-	pprintf(prn, "%s = P(%s(%d) > %g) = %g\n\n", 
-		_("with p-value"), _("Chi-square"), df, trsq, pval);
-
 	if (opt & OPT_S) {
 	    ModelTest *test;
 
@@ -990,7 +999,8 @@ real_nonlinearity_test (MODEL *pmod, int *list,
  * @pmod: pointer to original model.
  * @dset: dataset struct.
  * @aux: AUX_SQ for squares or AUX_LOG for logs
- * @opt: if contains OPT_S, save test results to model.
+ * @opt: if contains OPT_S, save test results to model; if
+ * contains OPT_I, run silently.
  * @prn: gretl printing struct.
  *
  * Run an auxiliary regression to test @pmod for nonlinearity,
@@ -1009,7 +1019,7 @@ int nonlinearity_test (MODEL *pmod, DATASET *dset, ModelAuxCode aux,
     const int orig_nvar = dset->v; 
     int err = 0;
 
-    if (!command_ok_for_model(ADD, 0, pmod->ci)) {
+    if (!command_ok_for_model(ADD, 0, pmod)) {
 	return E_NOTIMP;
     }
 
@@ -1018,7 +1028,7 @@ int nonlinearity_test (MODEL *pmod, DATASET *dset, ModelAuxCode aux,
     }
 
     /* check for changes in original list members */
-    err = list_members_replaced(pmod->list, dset, pmod->ID);
+    err = list_members_replaced(pmod, dset);
     if (err) {
 	return err;
     }
@@ -1145,7 +1155,7 @@ int add_test_full (MODEL *orig, MODEL *pmod, const int *addvars,
 	return E_DATA;
     }
 
-    if (!command_ok_for_model(ADD, opt, orig->ci)) {
+    if (!command_ok_for_model(ADD, opt, orig)) {
 	return E_NOTIMP;
     }
 
@@ -1159,7 +1169,7 @@ int add_test_full (MODEL *orig, MODEL *pmod, const int *addvars,
     }
 
     /* check for changes in original list members */
-    err = list_members_replaced(orig->list, dset, orig->ID);
+    err = list_members_replaced(orig, dset);
     if (err) {
 	return err;
     }
@@ -1507,7 +1517,7 @@ static int omit_test_precheck (MODEL *pmod, gretlopt opt)
 
     if (pmod == NULL || pmod->list == NULL) {
 	err = E_DATA;
-    } else if (!command_ok_for_model(OMIT, 0, pmod->ci)) {
+    } else if (!command_ok_for_model(OMIT, 0, pmod)) {
 	err = E_NOTIMP;
     } else if (omit_options_inconsistent(pmod, opt)) {
 	err = E_BADOPT;
@@ -1551,7 +1561,7 @@ int omit_test_full (MODEL *orig, MODEL *pmod, const int *omitvars,
     }
 
     /* check that vars to omit have not been redefined */
-    if ((err = list_members_replaced(orig->list, dset, orig->ID))) {
+    if ((err = list_members_replaced(orig, dset))) {
 	return err;
     }
 
@@ -1663,7 +1673,7 @@ double get_DW_pvalue_for_model (const MODEL *pmod, DATASET *dset,
 	*err = E_BADSTAT;
     } else {
 	/* check that relevant vars have not been redefined */
-	*err = list_members_replaced(pmod->list, dset, pmod->ID);
+	*err = list_members_replaced(pmod, dset);
     }
 
     if (!*err) {
@@ -1687,9 +1697,6 @@ double get_DW_pvalue_for_model (const MODEL *pmod, DATASET *dset,
 
     if (!*err) {
 	pv = gretl_model_get_double(&dwmod, "dw_pval");
-	if (na(pv)) {
-	    *err = E_DATA;
-	}
     }
 
     /* put back into dset what was there on input */
@@ -1970,7 +1977,7 @@ static int ivreg_autocorr_test (MODEL *pmod, int order,
     int v = dset->v;
     int *addlist = NULL;
     int *testlist = NULL;
-    double x, pval = 1.0;
+    double x = 0, pval = 1;
     MODEL aux;
     int i, t;
     int err = 0;
@@ -2147,7 +2154,7 @@ static int lb_autocorr_test (MODEL *pmod, int order,
  * @order: lag order for test.
  * @dset: dataset struct.
  * @opt: if flags include OPT_S, save test results to model;
- * if OPT_Q, be less verbose.
+ * if OPT_Q, be less verbose; if OPT_I, be silent.
  * @prn: gretl printing struct.
  *
  * Tests the given model for autocorrelation of order equal to
@@ -2275,7 +2282,9 @@ int autocorr_test (MODEL *pmod, int order, DATASET *dset,
 	LMF = ((RSSx - RSSxe) / RSSxe) * dfd / order;
 	pval = snedecor_cdf_comp(order, dfd, LMF);
 
-	if (pmod->aux != AUX_VAR) {
+	if (pmod->aux == AUX_VAR || (opt & OPT_I)) {
+	    ; /* don't print anything here */
+	} else {
 	    if (opt & OPT_Q) {
 		bg_test_header(order, prn, 0);
 	    } else {
@@ -2465,11 +2474,10 @@ static double get_QLR_pval (double test, int df, int k1, int k2,
 			    MODEL *pmod, PRN *prn)
 {
     double (*qlr_asy_pvalue) (double, int, double);
-    void *handle;
     double pi_1, pi_2, lam0;
     double pval;
 
-    qlr_asy_pvalue = get_plugin_function("qlr_asy_pvalue", &handle);
+    qlr_asy_pvalue = get_plugin_function("qlr_asy_pvalue");
 
     if (qlr_asy_pvalue == NULL) {
 	fputs(I_("Couldn't load plugin function\n"), stderr);
@@ -2482,8 +2490,6 @@ static double get_QLR_pval (double test, int df, int k1, int k2,
 
     pval = (*qlr_asy_pvalue) (test, df, lam0);
 
-    close_plugin(handle);
-
     if (!na(pval)) {
 	pprintf(prn, _("Asymptotic p-value = %.4g for "
 		       "chi-square(%d) = %g"),
@@ -3252,7 +3258,7 @@ int comfac_test (MODEL *pmod, DATASET *dset,
     }
 
     /* check for changes in original list members */
-    err = list_members_replaced(pmod->list, dset, pmod->ID);
+    err = list_members_replaced(pmod, dset);
     if (err) {
 	return err;
     }
@@ -3339,20 +3345,22 @@ int comfac_test (MODEL *pmod, DATASET *dset,
 	double Ftest = ((SSRr - SSRu)/dfn) / (SSRu/dfd);
 	double pval = snedecor_cdf_comp(dfn, dfd, Ftest);
 
-	if (!(opt & OPT_Q)) {
-	    cmod.aux = AUX_COMFAC;
-	    printmodel(&cmod, dset, OPT_S, prn);
+	if (!(opt & OPT_I)) {
+	    if (!(opt & OPT_Q)) {
+		cmod.aux = AUX_COMFAC;
+		printmodel(&cmod, dset, OPT_S, prn);
+		pputc(prn, '\n');
+	    }
+
+	    pputs(prn, _("Test of common factor restriction"));
+	    pputs(prn, "\n\n");
+
+	    pprintf(prn, "  %s: %s(%d, %d) = %g, ", _("Test statistic"), 
+		    "F", dfn, dfd, Ftest);
+	    pprintf(prn, _("with p-value = %g\n"), pval);
 	    pputc(prn, '\n');
 	}
 
-	pputs(prn, _("Test of common factor restriction"));
-	pputs(prn, "\n\n");
-
-	pprintf(prn, "  %s: %s(%d, %d) = %g, ", _("Test statistic"), 
-		"F", dfn, dfd, Ftest);
-	pprintf(prn, _("with p-value = %g\n"), pval);
-	pputc(prn, '\n');
-
 	if (opt & OPT_S) {
 	    ModelTest *test;
 
@@ -3538,7 +3546,6 @@ int add_leverage_values_to_dataset (DATASET *dset, gretl_matrix *m,
 int leverage_test (MODEL *pmod, DATASET *dset, 
 		   gretlopt opt, PRN *prn)
 {
-    void *handle;
     gretl_matrix *(*model_leverage) (const MODEL *, const DATASET *, 
 				     gretlopt, PRN *, int *);
     gretl_matrix *m;
@@ -3548,7 +3555,7 @@ int leverage_test (MODEL *pmod, DATASET *dset,
 	return E_OLSONLY;
     }
 
-    model_leverage = get_plugin_function("model_leverage", &handle);
+    model_leverage = get_plugin_function("model_leverage");
     if (model_leverage == NULL) {
 	return 1;
     }
@@ -3564,8 +3571,6 @@ int leverage_test (MODEL *pmod, DATASET *dset,
 
     gretl_matrix_free(m);
 
-    close_plugin(handle);
-
     return err;
 }
 
@@ -3583,20 +3588,14 @@ int leverage_test (MODEL *pmod, DATASET *dset,
 
 int vif_test (MODEL *pmod, DATASET *dset, PRN *prn)
 {
-    void *handle;
     int (*print_vifs) (MODEL *, DATASET *, PRN *);
-    int err;
 
     gretl_error_clear();
 
-    print_vifs = get_plugin_function("print_vifs", &handle);
+    print_vifs = get_plugin_function("print_vifs");
     if (print_vifs == NULL) {
 	return 1;
     }
 
-    err = (*print_vifs)(pmod, dset, prn);
-
-    close_plugin(handle);
-
-    return err;
+    return (*print_vifs)(pmod, dset, prn);
 }
