@@ -39,10 +39,13 @@ struct VARINFO_ {
     int transform;
     int lag;
     char compact_method;
+    gint64 mtime;
     char stack_level; /* FIXME should be int? */
     series_table *st;
 };
 
+static int pad_daily_data (DATASET *dset, int pd, PRN *prn);
+
 static int dataset_changed;
 
 /**
@@ -299,6 +302,7 @@ static void gretl_varinfo_init (VARINFO *vinfo)
     vinfo->transform = 0;
     vinfo->lag = 0;
     vinfo->compact_method = COMPACT_NONE;
+    vinfo->mtime = 0;
     vinfo->stack_level = gretl_function_depth();
     vinfo->st = NULL;
 }
@@ -573,7 +577,8 @@ int allocate_Z (DATASET *dset, gretlopt opt)
 /**
  * start_new_Z:
  * @dset: pointer to dataset.
- * @opt: if includes OPT_R we're sub-sampling from a full data set.
+ * @opt: if includes OPT_R we're sub-sampling from a full data set;
+ * if includes OPT_P, do not null out dset->S and markers.
  *
  * Initializes the data array within @dset (adding the constant in
  * position 0).
@@ -600,8 +605,11 @@ int start_new_Z (DATASET *dset, gretlopt opt)
 	return E_ALLOC;
     }
 
-    dset->S = NULL;
-    dset->markers = NO_MARKERS;
+    if (!(opt & OPT_P)) {
+	dset->S = NULL;
+	dset->markers = NO_MARKERS;
+    }
+
     dset->descrip = NULL;
     dset->submask = NULL;
     dset->restriction = NULL;
@@ -2936,6 +2944,8 @@ int dataset_op_from_string (const char *s)
 	op = DS_RENUMBER;
     } else if (!strcmp(s, "insobs")) {
 	op = DS_INSOBS;
+    } else if (!strcmp(s, "pad-daily")) {
+	op = DS_PAD_DAILY;
     }
 
     return op;
@@ -2954,6 +2964,11 @@ static int dataset_int_param (const char **ps, int op,
 	return 0;
     }
 
+    if (op == DS_PAD_DAILY && !dated_daily_data(dset)) {
+	*err = E_PDWRONG;
+	return 0;
+    }
+
     *test = '\0';
     sscanf(s, "%31s", test);
     *ps += strlen(test);
@@ -2988,6 +3003,11 @@ static int dataset_int_param (const char **ps, int op,
 	    *err = E_PDWRONG;
 	    gretl_errmsg_set("This conversion is not supported");
 	}
+    } else if (op == DS_PAD_DAILY) {
+	if (k < 5 || k > 7 || k < dset->pd) {
+	    *err = E_PDWRONG;
+	    gretl_errmsg_set("This conversion is not supported");
+	}
     }
 
     return k;
@@ -3186,7 +3206,8 @@ int renumber_series_with_checks (const char *s, int fixmax,
    dataset resample          500
    dataset clear 
    dataset renumber   orig   2  
-   dataset insobs     13
+   dataset insobs            13
+   dataset pad-daily         7
 
 */
 
@@ -3236,7 +3257,8 @@ int modify_dataset (DATASET *dset, int op, const int *list,
     }
 
     if (op == DS_ADDOBS || op == DS_INSOBS || 
-	op == DS_COMPACT || op == DS_RESAMPLE) {
+	op == DS_COMPACT || op == DS_RESAMPLE ||
+	op == DS_PAD_DAILY) {
 	k = dataset_int_param(&s, op, dset, &err);
 	if (err) {
 	    return err;
@@ -3264,6 +3286,8 @@ int modify_dataset (DATASET *dset, int op, const int *list,
 	    interp = 1;
 	}
 	err = expand_data_set(dset, k, interp);
+    } else if (op == DS_PAD_DAILY) {
+	err = pad_daily_data(dset, k, prn);
     } else if (op == DS_TRANSPOSE) {
 	err = transpose_data(dset);
     } else if (op == DS_SORTBY) {
@@ -3757,6 +3781,22 @@ int series_get_stack_level (const DATASET *dset, int i)
     }
 }
 
+void series_set_mtime (DATASET *dset, int i)
+{
+    if (i > 0 && i < dset->v) {
+	dset->varinfo[i]->mtime = gretl_monotonic_time();
+    }
+}
+
+gint64 series_get_mtime (const DATASET *dset, int i)
+{
+    if (i > 0 && i < dset->v) {
+	return dset->varinfo[i]->mtime;
+    } else {
+	return 0;
+    }
+}
+
 void series_set_label (DATASET *dset, int i, 
 		       const char *s)
 {
@@ -3827,8 +3867,10 @@ void series_decrement_stack_level (DATASET *dset, int i)
 
 void series_attach_string_table (DATASET *dset, int i, void *ptr)
 {
-    series_set_discrete(dset, i, 1);
-    dset->varinfo[i]->st = ptr;
+    if (dset != NULL && i > 0 && i < dset->v) {
+	series_set_discrete(dset, i, 1);
+	dset->varinfo[i]->st = ptr;
+    }
 }
 
 void series_destroy_string_table (DATASET *dset, int i)
@@ -3864,7 +3906,7 @@ int is_string_valued (const DATASET *dset, int i)
  * @i: index number of series.
  *
  * Returns: the string table attched to series @i or NULL if
- * there is no such table.
+ * there is no such table. 
  */
 
 series_table *series_get_string_table (const DATASET *dset, int i)
@@ -3996,10 +4038,10 @@ double series_decode_string (const DATASET *dset, int i, const char *s)
  * you need to modify them.
  */
 
-const char **series_get_string_vals (const DATASET *dset, int i,
+char **series_get_string_vals (const DATASET *dset, int i,
 				     int *n_strs)
 {
-    const char **strs = NULL;
+    char **strs = NULL;
     int n = 0;
 
     if (i > 0 && i < dset->v && dset->varinfo[i]->st != NULL) {
@@ -4038,6 +4080,112 @@ int steal_string_table (DATASET *l_dset, int lvar,
     return 0;
 }
 
+static void maybe_adjust_label (DATASET *dset, int v, 
+				char **S, int ns)
+{
+    int i, len = 3 * ns; /* "=" + ", " */
+
+    if (len < MAXLABEL) {
+	for (i=0; i<ns; i++) {
+	    len += strlen(S[i]) + 1 + floor(log10(1.0 + i));
+	}
+    }
+
+    if (len < MAXLABEL) {
+	char *s = dset->varinfo[v]->label;
+	char bit[8];
+
+	*s = '\0';
+	for (i=0; i<ns; i++) {
+	    sprintf(bit, "%d=", i+1);
+	    strcat(s, bit);
+	    strcat(s, S[i]);
+	    if (i < ns - 1) {
+		strcat(s, ", ");
+	    }
+	}
+    }
+}
+
+/* here we're trying to set strings values on a series from
+   scratch */
+
+int series_set_string_vals (DATASET *dset, int i,
+			    char **S, int ns)
+{
+    gretl_matrix *vals = NULL;
+    int err = 0;
+
+    if (S == NULL || dset == NULL || i < 1 || i >= dset->n) {
+	return E_DATA;
+    }
+
+    /* get sorted vector of unique values */
+    vals = gretl_matrix_values(dset->Z[i], dset->n, OPT_S, &err);
+
+    if (!err) {
+	int i, nvals = gretl_vector_get_length(vals);
+
+	if (ns != nvals) {
+	    /* we want exactly one string per (non-missing) value */
+	    gretl_errmsg_sprintf("Wrong number of strings, %d: should be %d",
+				 ns, nvals);
+	    err = E_DATA;
+	} else {
+	    /* the values should be successive 1-based integers */
+	    double x0 = gretl_vector_get(vals, 0);
+	    double x1 = gretl_vector_get(vals, nvals - 1);
+
+	    if (x0 != 1.0 || x1 != nvals) {
+		gretl_errmsg_set("The series values must be successive integers");
+		err = E_DATA;
+	    }
+	    for (i=1; i<nvals && !err; i++) {
+		x1 = gretl_vector_get(vals, i);
+		if (x1 != x0 + 1.0) {
+		    gretl_errmsg_set("The series values must be successive integers");
+		    err = E_DATA;
+		}
+		x0 = x1;
+	    }
+	    /* and the strings should all be UTF8 */
+	    for (i=0; i<ns && !err; i++) {
+		if (!g_utf8_validate(S[i], -1, NULL)) {
+		    gretl_errmsg_sprintf("String %d is not valid UTF-8", i+1);
+		    err = E_DATA;
+		}
+	    }
+	}
+    }
+
+    if (!err) {
+	char **Scpy = strings_array_dup(S, ns);
+	series_table *st = NULL;
+
+	if (Scpy == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    st = series_table_new(Scpy, ns);
+	    if (st == NULL) {
+		err = E_ALLOC;
+		strings_array_free(Scpy, ns);
+	    }
+	}
+	if (!err) {
+	    if (dset->varinfo[i]->st != NULL) {
+		series_table_destroy(dset->varinfo[i]->st);
+	    }
+	    series_set_discrete(dset, i, 1);
+	    dset->varinfo[i]->st = st;
+	    maybe_adjust_label(dset, i, S, ns);
+	}
+    }
+
+    gretl_matrix_free(vals);
+
+    return err;
+}
+
 int set_panel_groups_name (DATASET *dset, const char *vname)
 {
     if (dset->pangrps != NULL) {
@@ -4049,9 +4197,11 @@ int set_panel_groups_name (DATASET *dset, const char *vname)
     return (dset->pangrps == NULL)? E_ALLOC : 0;
 }
 
-char const **get_panel_group_names (DATASET *dset)
+/* don't modify the returned value */
+
+char **get_panel_group_names (DATASET *dset)
 {
-    char const **S = NULL;
+    char **S = NULL;
 
     if (dataset_is_panel(dset) && dset->pangrps != NULL) {
 	int n, v = current_series_index(dset, dset->pangrps);
@@ -4077,7 +4227,7 @@ int panel_group_names_ok (const DATASET *dset)
 	int n, v = current_series_index(dset, dset->pangrps);
 
 	if (v > 0 && v < dset->v) {
-	    char const **S = series_get_string_vals(dset, v, &n);
+	    char **S = series_get_string_vals(dset, v, &n);
 
 	    if (S != NULL && n == dset->n / dset->pd) {
 		ret = 1;
@@ -4094,7 +4244,7 @@ const char *panel_group_names_varname (const DATASET *dset)
 	int n, v = current_series_index(dset, dset->pangrps);
 
 	if (v > 0 && v < dset->v) {
-	    char const **S = series_get_string_vals(dset, v, &n);
+	    char **S = series_get_string_vals(dset, v, &n);
 
 	    if (S != NULL && n == dset->n / dset->pd) {
 		return dset->pangrps;
@@ -4113,3 +4263,98 @@ int is_panel_group_names_series (const DATASET *dset, int v)
 	return 0;
     }
 }
+
+static int effective_daily_skip (int delta, int wd, int pd)
+{
+    int k, skip = delta - 1;
+
+    if (pd < 7) {
+	skip = 0;
+	for (k=1; k<delta; k++) {
+	    wd = (wd == 0)? 6 : wd - 1;
+	    if (pd == 6) {
+		skip += (wd != 0);
+	    } else {
+		skip += (wd != 0 && wd != 6);
+	    }
+	}
+    }
+
+    return skip;
+}
+
+/* If we get here we've already checked that @dset is dated daily
+   data, and that @pd is a valid daily periodicity greater than or
+   equal to the current dset->pd.
+*/
+
+static int pad_daily_data (DATASET *dset, int pd, PRN *prn)
+{
+    DATASET *bigset = NULL;
+    char datestr[OBSLEN];
+    long ed, ed0, edbak = 0;
+    int wd, skip, totskip = 0;
+    int t, err = 0;
+
+    for (t=0; t<dset->n; t++) {
+	ntodate(datestr, t, dset);
+	if (t == 0) {
+	    ed0 = edbak = get_epoch_day(datestr);
+	} else {
+	    wd = weekday_from_date(datestr);
+	    ed = get_epoch_day(datestr);
+	    skip = effective_daily_skip(ed - edbak, wd, pd);
+	    totskip += skip;
+	    edbak = ed;
+	}
+    }
+
+    if (totskip == 0) {
+	pprintf(prn, "Dataset is already complete for %d-day calendar", pd);
+	return 0;
+    }
+
+    bigset = create_new_dataset(dset->v, dset->n + totskip, NO_MARKERS);
+
+    if (bigset == NULL) {
+	err = E_ALLOC;
+    } else {
+	int i, s = 0;
+
+	edbak = ed0;
+
+	for (t=0; t<dset->n; t++) {
+	    if (t > 0) {
+		ntodate(datestr, t, dset);
+		wd = weekday_from_date(datestr);
+		ed = get_epoch_day(datestr);
+		s += 1 + effective_daily_skip(ed - edbak, wd, pd);
+		edbak = ed;
+	    }
+	    for (i=1; i<dset->v; i++) {
+		bigset->Z[i][s] = dset->Z[i][t];
+	    }
+	}
+
+	bigset->varname = dset->varname;
+	bigset->varinfo = dset->varinfo;
+	bigset->descrip = dset->descrip;
+
+	bigset->pd = pd;
+	bigset->structure = TIME_SERIES;
+	bigset->sd0 = (double) ed0;
+	ntodate(bigset->stobs, 0, bigset);
+	ntodate(bigset->endobs, bigset->n - 1, bigset);
+
+	dset->varname = NULL;
+	dset->varinfo = NULL;
+	dset->descrip = NULL;
+	dataset_destroy_obs_markers(dset);
+	free_Z(dset);
+	clear_datainfo(dset, CLEAR_SUBSAMPLE); 
+
+	*dset = *bigset;
+    }
+
+    return err;
+}
