@@ -23,6 +23,7 @@
 #include "gretl_func.h"
 #include "uservar.h"
 #include "gretl_string_table.h"
+#include "gretl_bundle.h"
 
 #include <glib.h>
 
@@ -50,6 +51,11 @@ struct str_table consts[] = {
     { CONST_INF,   "$inf" },
     { CONST_WIN32, "WIN32" },
     { CONST_EPS,   "$macheps" },
+    { CONST_HAVE_MPI, "$havempi" },
+    { CONST_MPI_RANK, "$mpirank" },
+    { CONST_MPI_SIZE, "$mpisize" },
+    { CONST_N_PROC,   "$nproc" },
+    { CONST_SYSINFO,  "$sysinfo" },
     { 0,        NULL }
 };
 
@@ -60,6 +66,8 @@ struct str_table dummies[] = {
     { 0,        NULL }
 };
 
+/* dvars: dataset-related accessors */
+
 struct str_table dvars[] = {
     { R_NOBS,      "$nobs" },
     { R_NVARS,     "$nvars" },
@@ -88,6 +96,8 @@ struct str_table dvars[] = {
     { 0,           NULL },
 };
 
+/* mvars: model-related accessors */
+
 struct str_table mvars[] = {
     { M_ESS,     "$ess" },
     { M_T,       "$T" },
@@ -297,7 +307,7 @@ struct str_table funcs[] = {
     { F_ISLIST,   "islist" },
     { F_ISSTRING, "isstring" },
     { F_ISNULL,   "isnull" },
-    { F_LISTLEN,  "nelem" },
+    { F_NELEM,    "nelem" },
     { F_PDF,      "pdf" },
     { F_CDF,      "cdf" },
     { F_INVCDF,   "invcdf" },
@@ -316,6 +326,8 @@ struct str_table funcs[] = {
     { F_MRLS,     "mrls" },
     { F_MREAD,    "mread" },
     { F_MWRITE,   "mwrite" },
+    { F_BREAD,    "bread" },
+    { F_BWRITE,   "bwrite" },
     { F_MCSEL,    "selifc" },
     { F_MRSEL,    "selifr" },
     { F_POLROOTS, "polroots" },
@@ -404,7 +416,24 @@ struct str_table funcs[] = {
     { F_ATOF,     "atof" },
     { F_FIXNAME,  "fixname" },
     { F_ISOCONV,  "isoconv" },
-    { 0,          NULL }
+    { F_SUBSTR,   "substr" },
+    { F_MPI_SEND, "mpisend" },
+    { F_MPI_RECV, "mpirecv" },
+    { F_BCAST,    "mpibcast" },
+    { F_REDUCE,   "mpireduce" },
+    { F_ALLREDUCE, "mpiallred" },
+    { F_SCATTER,   "mpiscatter" },
+    { F_EASTER,    "easterday" },
+    { F_GENSERIES, "genseries" },
+    { F_CURL,      "curl" },
+    { F_JSONGET,   "jsonget" },
+    { F_NLINES,    "nlines" },
+    { F_KPSSCRIT,  "kpsscrit" },
+    { F_ARRAY,     "array" },
+    { F_STRVALS,   "strvals" },
+    { F_STRINGIFY, "stringify" },
+    { F_PRINTARRAY, "printarray" },
+    { 0,           NULL }
 };
 
 struct str_table func_alias[] = {
@@ -414,7 +443,7 @@ struct str_table func_alias[] = {
     { F_LOG,       "logs" },
     { F_OBSLABEL,  "date" },
     { F_BACKTICK,  "$" },
-    { 0,          NULL }
+    { 0,           NULL }
 };
 
 int const_lookup (const char *s)
@@ -907,12 +936,23 @@ int is_gretl_accessor (const char *s)
 
 static void look_up_dollar_word (const char *s, parser *p)
 {
+    char *bstr;
+
     if ((p->idnum = dvar_lookup(s)) > 0) {
 	p->sym = DVAR;
     } else if ((p->idnum = const_lookup(s)) > 0) {
-	p->sym = CON;
+	if (p->idnum == CONST_SYSINFO) {
+	    p->sym = BUNDLE;
+	    p->idstr = gretl_strdup("$sysinfo");
+	    p->uval = get_sysinfo_bundle(&p->err);
+	} else {
+	    p->sym = CON;
+	}
     } else if ((p->idnum = mvar_lookup(s)) > 0) {
 	p->sym = MVAR;
+    } else if ((bstr = get_built_in_string_by_name(s+1))) {
+	p->sym = STR;
+	p->idstr = gretl_strdup(bstr);
     } else {
 	undefined_symbol_error(s, p);
     }
@@ -980,10 +1020,9 @@ static void look_up_word (const char *s, parser *p)
 		p->sym = DUM;
 	    } else {
 		GretlType vtype = 0;
-		char *bstr;
 
 		if ((p->idnum = current_series_index(p->dset, s)) >= 0) {
-		    p->sym = UVEC;
+		    p->sym = USERIES;
 		    p->idstr = gretl_strdup(s);
 		} else if (!strcmp(s, "time")) {
 		    p->sym = DUM;
@@ -995,6 +1034,8 @@ static void look_up_word (const char *s, parser *p)
 			p->sym = UMAT;
 		    } else if (vtype == GRETL_TYPE_BUNDLE) {
 			p->sym = BUNDLE;
+		    } else if (vtype == GRETL_TYPE_ARRAY) {
+			p->sym = ARRAY;
 		    } else if (vtype == GRETL_TYPE_STRING) {
 			p->sym = USTR;
 		    } else if (vtype == GRETL_TYPE_LIST) {
@@ -1003,10 +1044,6 @@ static void look_up_word (const char *s, parser *p)
 			p->sym = USTR;
 		    }
 		    p->idstr = gretl_strdup(s);
-		} else if ((bstr = get_built_in_string_by_name(s))) {
-		    /* FIXME should use $-accessors? */
-		    p->sym = STR;
-		    p->idstr = gretl_strdup(bstr);
 		} else if (gretl_get_object_by_name(s)) {
 		    p->sym = UOBJ;
 		    p->idstr = gretl_strdup(s);
@@ -1075,13 +1112,15 @@ static void word_check_next_char (parser *p)
     if (p->ch) fprintf(stderr, "word_check_next_char: ch = '%c'\n", p->ch);
     else fprintf(stderr, "word_check_next_char: ch = NUL\n");
 #endif
+    p->upsym = 0;
 
     if (p->ch == '(') {
 	/* series (lag) or function */
-	if (p->sym == UVEC) {
-	    if (p->idnum == p->lh.v) {
+	if (p->sym == USERIES) {
+	    if (p->idnum > 0 && p->idnum == p->lh.v) {
 		p->flags |= P_AUTOREG;
 	    }
+	    p->upsym = USERIES;
 	    p->sym = LAG;
 	} else if (p->sym == MVAR && model_data_matrix(p->idnum)) {
 	    /* old-style "$coeff(x1)" etc. */
@@ -1095,6 +1134,7 @@ static void word_check_next_char (parser *p)
 	    p->err = E_PARSE;
 	} 
     } else if (p->ch == '[') {
+	p->upsym = p->sym;
 	if (p->sym == UMAT) {
 	    /* slice of user matrix */
 	    p->sym = MSL;
@@ -1102,21 +1142,24 @@ static void word_check_next_char (parser *p)
 		   could_be_matrix(p->idnum)) {
 	    /* slice of $ matrix */
 	    p->sym = DMSL;
-	} else if (p->sym == UVEC) {
+	} else if (p->sym == USERIES) {
 	    /* observation from series */
 	    p->sym = OBS;
 	} else if (p->sym == DVAR && dollar_series(p->idnum)) {
 	    /* observation from "dollar" series */
-	    p->sym = DOBS;
+	    p->sym = OBS;
 	} else if (p->sym == ULIST) {
 	    /* element of list */
-	    p->sym = LISTELEM;
+	    p->sym = ELEMENT;
 	} else if (p->sym == MVAR && model_data_list(p->idnum)) {
 	    /* element of accessor list */
-	    p->sym = MLISTELEM;
+	    p->sym = ELEMENT;
+	} else if (p->sym == ARRAY) {
+	    /* element of array */
+	    p->sym = ELEMENT;
 	} else if (p->sym == BUNDLE) {
-	    /* object from bundle */
-	    p->sym = BOBJ;
+	    /* member from bundle */
+	    p->sym = BMEMB;
 	} else {
 	    p->err = E_PARSE;
 	} 
@@ -1608,7 +1651,7 @@ const char *getsymb (int t, const parser *p)
     }
 
     /* yes, well */
-    if (t == OBS || t == DOBS) {
+    if (t == OBS) {
 	return "OBS";
     } else if (t == MSL) {
 	return "MSL";
@@ -1616,8 +1659,8 @@ const char *getsymb (int t, const parser *p)
 	return "DMSL";
     } else if (t == DMSTR) {
 	return "DMSTR";
-    } else if (t == MSL2) {
-	return "MSL2";
+    } else if (t == MSLRAW) {
+	return "MSLRAW";
     } else if (t == MSPEC) {
 	return "MSPEC";
     } else if (t == SUBSL) {
@@ -1636,14 +1679,12 @@ const char *getsymb (int t, const parser *p)
 	return "EMPTY";
     } else if (t == LISTVAR) {
 	return "LISTVAR";
-    } else if (t == BOBJ) {
-	return "BOBJ";
     } else if (t == BMEMB) {
 	return "BMEMB";
-    } else if (t == LISTELEM) {
-	return "LISTELEM";
-    } else if (t == VEC) {
-	return "VEC";
+    } else if (t == ELEMENT) {
+	return "ELEMENT";
+    } else if (t == SERIES) {
+	return "SERIES";
     } else if (t == MAT) {
 	return "MAT";
     } else if (t == EROOT) {
@@ -1655,12 +1696,14 @@ const char *getsymb (int t, const parser *p)
     }
 
     if (p != NULL) {
-	if (t == UVEC) {
+	if (t == USERIES) {
 	    return p->dset->varname[p->idnum];
 	} else if (t == UNUM) {
 	    return p->idstr;
 	} else if (t == BUNDLE) {
 	    return p->idstr;
+	} else if (t == ARRAY) {
+	    return p->idstr;
 	} else if (t == UMAT || t == UOBJ) {
 	    return p->idstr;
 	} else if (t == CON) {
@@ -1677,12 +1720,14 @@ const char *getsymb (int t, const parser *p)
 	    return p->idstr;
 	}
     } else {
-	if (t == UVEC) {
-	    return "UVEC";
+	if (t == USERIES) {
+	    return "USERIES";
 	} else if (t == UNUM) {
 	    return "UNUM";
 	} else if (t == BUNDLE) {
 	    return "BUNDLE";
+	} else if (t == ARRAY) {
+	    return "ARRAY";
 	} else if (t == UMAT) {
 	    return "UMAT";
 	} else if (t == UOBJ) {
@@ -1805,5 +1850,3 @@ const char *getsymb (int t, const parser *p)
 
     return "unknown";
 }
-
-
