@@ -35,7 +35,7 @@ static void write_scalar_message (const parser *p, PRN *prn)
 {
     double x = gretl_scalar_get_value(p->lh.name, NULL);
 
-    if (p->flags & P_LHSCAL) {
+    if (p->lh.t == NUM) {
 	pprintf(prn, _("Replaced scalar %s"), p->lh.name);
     } else {
 	pprintf(prn, _("Generated scalar %s"), p->lh.name);
@@ -62,7 +62,7 @@ static void gen_write_message (const parser *p, int oldv, PRN *prn)
 	} else {
 	    write_scalar_message(p, prn);
 	}
-    } else if (p->targ == VEC) {
+    } else if (p->targ == SERIES) {
 	if (p->lh.v < oldv) {
 	    pprintf(prn, _("Replaced series %s (ID %d)"),
 		    p->lh.name, p->lh.v);
@@ -71,22 +71,22 @@ static void gen_write_message (const parser *p, int oldv, PRN *prn)
 		    p->lh.name, p->lh.v);
 	}
     } else if (p->targ == MAT) {
-	if ((p->flags & P_LHMAT) && p->lh.substr != NULL && 
+	if (p->lh.t == MAT && p->lh.substr != NULL && 
 	    *p->lh.substr != '\0') {
 	    pprintf(prn, _("Modified matrix %s"), p->lh.name);
-	} else if (p->flags & P_LHMAT) {
+	} else if (p->lh.t == MAT) {
 	    pprintf(prn, _("Replaced matrix %s"), p->lh.name);
 	} else {
 	    pprintf(prn, _("Generated matrix %s"), p->lh.name);
 	}
     } else if (p->targ == LIST) {
-	if (p->flags & P_LHLIST) {
+	if (p->lh.t == LIST) {
 	    pprintf(prn, _("Replaced list %s"), p->lh.name);
 	} else {
 	    pprintf(prn, _("Generated list %s"), p->lh.name);
 	}
     } else if (p->targ == STR) {
-	if (p->flags & P_LHSTR) {
+	if (p->lh.t == STR) {
 	    pprintf(prn, _("Replaced string %s"), p->lh.name);
 	} else {
 	    pprintf(prn, _("Generated string %s"), p->lh.name);
@@ -143,9 +143,8 @@ static void gen_write_label (parser *p, int oldv)
 {
     char tmp[MAXLABEL];
     const char *src = "";
-    size_t len = 0;
 
-    if (p->targ != VEC) {
+    if (p->targ != SERIES) {
 	/* this is relevant only for series */
 	return;
     }
@@ -158,15 +157,8 @@ static void gen_write_label (parser *p, int oldv)
 
     maybe_record_lag_info(p);
 
-    *tmp = '\0';
-
-    if (p->lh.v < oldv && p->targ == VEC) {
-	int m = get_model_count();
-
-	if (m > 0) {
-	    sprintf(tmp, _("Replaced after model %d: "), m);
-	    len = strlen(tmp);
-	}
+    if (p->lh.v < oldv && p->targ == SERIES) {
+	series_set_mtime(p->dset, p->lh.v);
     }
 
     if (*p->lh.label != '\0' && (p->flags & P_UFRET)) {
@@ -177,8 +169,10 @@ static void gen_write_label (parser *p, int oldv)
 	src = p->rhs;
     }
 
-    if (strlen(src) > MAXLABEL - 1 - len) {
-	strncat(tmp, src, MAXLABEL - 4 - len);
+    *tmp = '\0';
+
+    if (strlen(src) > MAXLABEL - 1) {
+	strncat(tmp, src, MAXLABEL - 4);
 	strcat(tmp, "...");
     } else {
 	strncat(tmp, src, MAXLABEL - 1);
@@ -586,7 +580,7 @@ static int gen_special (const char *s, const char *line,
 	strcpy(p->lh.name, s);
 	p->lh.v = series_index(dset, s);
 	p->dset = dset;
-	p->targ = VEC;
+	p->targ = SERIES;
 	p->flags = 0;
 	p->err = 0;
 	p->prn = prn;
@@ -667,7 +661,6 @@ static int is_gen_special (const char *s, char *spec, const char **rem)
 #define gen_verbose(f) (!(f & P_PRINT) && \
                         !(f & P_DISCARD) && \
                         !(f & P_PRIVATE) && \
-                        !(f & P_UFUN) && \
                         !(f & P_QUIET) && \
                         !(f & P_DECL))
 
@@ -677,14 +670,17 @@ int generate (const char *line, DATASET *dset, gretlopt opt,
     char vname[VNAMELEN];
     const char *subline = NULL;
     int oldv, flags = 0;
+    int targtype = UNK;
     parser p;
 
     if (opt & OPT_P) {
+	/* internal use of generate() */
 	flags |= P_PRIVATE;
     }
 
-    if (opt & OPT_U) {
-	flags |= P_UFUN;
+    if (opt & OPT_O) {
+	/* no assignment */
+	targtype = EMPTY;
     }
 
     if (opt & OPT_Q) {
@@ -703,16 +699,15 @@ int generate (const char *line, DATASET *dset, gretlopt opt,
 	return dataset_stack_variables(vname, subline, dset, prn);
     }
 
-    realgen(line, &p, dset, prn, flags);
+    realgen(line, &p, dset, prn, flags, targtype);
 
-    if (!(opt & OPT_U)) {
+    if (!p.err && targtype != EMPTY) {
 	gen_save_or_print(&p, prn);
-    }
-
-    if (!p.err && gen_verbose(p.flags)) {
-	gen_write_label(&p, oldv);
-	if (!(opt & OPT_Q)) {
-	    gen_write_message(&p, oldv, prn);
+	if (!p.err && gen_verbose(p.flags)) {
+	    gen_write_label(&p, oldv);
+	    if (!(opt & OPT_Q)) {
+		gen_write_message(&p, oldv, prn);
+	    }
 	}
     }
 
@@ -739,7 +734,7 @@ double generate_scalar (const char *s, DATASET *dset, int *err)
     parser p;
     double x = NADBL;
 
-    *err = realgen(s, &p, dset, NULL, P_SCALAR | P_PRIVATE);
+    *err = realgen(s, &p, dset, NULL, P_PRIVATE, NUM);
 
     if (!*err) {
 	if (p.ret->t == MAT) {
@@ -774,12 +769,12 @@ double *generate_series (const char *s, DATASET *dset, PRN *prn,
     parser p;
     double *x = NULL;
 
-    *err = realgen(s, &p, dset, prn, P_SERIES | P_PRIVATE);
+    *err = realgen(s, &p, dset, prn, P_PRIVATE, SERIES);
 
     if (!*err) {
 	NODE *n = p.ret;
 
-	if (n->t == VEC) {
+	if (n->t == SERIES) {
 	    if (n->flags & TMP_NODE) {
 		/* steal the generated series */
 		x = n->v.xvec;
@@ -807,7 +802,7 @@ gretl_matrix *generate_matrix (const char *s, DATASET *dset,
     gretl_matrix *m = NULL;
     parser p;
 
-    *err = realgen(s, &p, dset, NULL, P_MATRIX | P_PRIVATE);
+    *err = realgen(s, &p, dset, NULL, P_PRIVATE, MAT);
 
     if (!*err) {
 	NODE *n = p.ret;
@@ -853,11 +848,19 @@ char *generate_string (const char *s, DATASET *dset, int *err)
     parser p;
     char *ret = NULL;
 
-    *err = realgen(s, &p, dset, NULL, P_STRING | P_PRIVATE);
+    *err = realgen(s, &p, dset, NULL, P_PRIVATE, STR);
 
     if (!*err) {
-	if (p.ret->t == STR) {
-	    ret = gretl_strdup(p.ret->v.str);
+	NODE *n = p.ret;
+
+	if (n->t == STR) {
+	    if (n->flags & TMP_NODE) {
+		/* steal the generated string */
+		ret = n->v.str;
+		n->v.str = NULL;
+	    } else {
+		ret = gretl_strdup(n->v.str);
+	    }
 	} else {
 	    *err = E_TYPES;
 	}
@@ -877,7 +880,7 @@ int *generate_list (const char *s, DATASET *dset, int *err)
     parser p;
     int *ret = NULL;
 
-    *err = realgen(s, &p, dset, NULL, P_LIST | P_PRIVATE);
+    *err = realgen(s, &p, dset, NULL, P_PRIVATE, LIST);
 
     if (!*err) {
 	NODE *n = p.ret;
@@ -890,7 +893,7 @@ int *generate_list (const char *s, DATASET *dset, int *err)
 	    } else {
 		ret = gretl_list_copy(nlist);
 	    }
-	} else if (n->t == VEC && n->vnum >= 0) {
+	} else if (n->t == SERIES && n->vnum >= 0) {
 	    ret = gretl_list_new(1);
 	    if (ret != NULL) {
 		ret[1] = n->vnum;
@@ -917,6 +920,7 @@ parser *genr_compile (const char *s, DATASET *dset,
 {
     parser *p = malloc(sizeof *p);
     int flags = P_COMPILE;
+    int targtype = UNK;
 
 #if GDEBUG
     fprintf(stderr, "\n*** genr_compile: s = '%s'\n", s);
@@ -931,17 +935,16 @@ parser *genr_compile (const char *s, DATASET *dset,
 	flags |= P_PRIVATE;
     }
 
-    if (opt & OPT_U) {
-	flags |= P_UFUN;
+    if (opt & OPT_O) {
+	targtype = EMPTY;
+    } else if (opt & OPT_S) {
+	targtype = NUM;
     }
 
-    if (opt & OPT_S) {
-	flags |= P_SCALAR;
-    }
-
-    *err = realgen(s, p, dset, NULL, flags);
+    *err = realgen(s, p, dset, NULL, flags, targtype);
 
     if (*err) {
+	p->flags = 0;
 	gen_cleanup(p);
 	free(p);
 	p = NULL;
@@ -963,9 +966,9 @@ int execute_genr (parser *p, DATASET *dset, PRN *prn)
 	    (void *) p, p->lh.name, (void *) dset->Z, (void *) prn);
 #endif
 
-    realgen(NULL, p, dset, prn, P_EXEC);
+    realgen(NULL, p, dset, prn, P_EXEC, UNK);
 
-    if (!p->err && !(p->flags & P_UFUN)) {
+    if (!p->err && p->targ != EMPTY) {
 	gen_save_or_print(p, prn);
     } 
 
@@ -982,8 +985,8 @@ double evaluate_if_cond (parser *p, DATASET *dset, int *err)
 {
     double x = NADBL;
 
-    *err = realgen(NULL, p, dset, NULL, 
-		   P_EXEC | P_SCALAR | P_PRIVATE);
+    *err = realgen(NULL, p, dset, NULL,  P_EXEC | P_PRIVATE, 
+		   NUM);
 
     if (!*err) {
 	if (p->ret->t == MAT) {
@@ -992,7 +995,7 @@ double evaluate_if_cond (parser *p, DATASET *dset, int *err)
 	    if (gretl_matrix_is_scalar(m)) {
 		x = p->ret->v.m->val[0];
 	    } else if (!gretl_is_null_matrix(m)) {
-		fprintf(stderr, "generate_scalar: got %d x %d matrix\n",
+		fprintf(stderr, "evaluate_if_cond: got %d x %d matrix\n",
 			m->rows, m->cols);
 		*err = E_TYPES;
 	    }
@@ -1019,11 +1022,11 @@ void destroy_genr (parser *p)
 #endif
 
     if (p != NULL) {
+	if (p->flags & P_AUXDONE) {
+	    /* avoid double-freeing */
+	    p->lh.mspec = NULL;
+	}
 	p->flags = 0;
-#if PRESERVE_AUX_NODES
-	/* free this stuff only when finished */
-	parser_free_aux_nodes(p);
-#endif
 	gen_cleanup(p);
 	free(p);
     }
@@ -1036,7 +1039,7 @@ int genr_get_output_type (const parser *p)
     if (!p->err) {
 	if (p->targ == NUM) {
 	    t = GRETL_TYPE_DOUBLE;
-	} else if (p->targ == VEC) {
+	} else if (p->targ == SERIES) {
 	    t = GRETL_TYPE_SERIES;
 	} else if (p->targ == MAT) {
 	    t = GRETL_TYPE_MATRIX;
@@ -1076,7 +1079,7 @@ int genr_is_print (const parser *p)
 
 int genr_is_autoregressive (const parser *p)
 {
-    return (p->flags & P_AUTOREG);
+    return (autoreg(p));
 }
 
 void genr_set_na_check (parser *p)
