@@ -23,6 +23,7 @@
 #include "uservar.h"
 #include "gretl_func.h"
 #include "gretl_bundle.h"
+#include "gretl_array.h"
 
 #define GENDEBUG 0
 
@@ -84,22 +85,21 @@ enum {
 	      CON,	  /* named constant */
 	      DUM,	  /* dummy variable */
               UNUM,	  /* user variable, named scalar */
-              UVEC,       /* user variable, named series */
+              USERIES,    /* user variable, named series */
 	      UMAT,	  /* user variable, named matrix */
 	      ULIST,      /* user variable, named list */
 	      UOBJ,	  /* user-defined object (e.g. model) */
   /* 60 */    UNUM_P,     /* user scalar++ */
 	      UNUM_M,     /* user scalar-- */
 	      NUM,	  /* scalar, evaluated */
-	      VEC,	  /* series, evaluated */
+	      SERIES,	  /* series, evaluated */
 	      MAT,	  /* matrix, evaluated */
 	      OBS,	  /* observation from a series */
-	      DOBS,       /* observation from "dollar" series */
               MSL,	  /* matrix plus subspec */
               DMSL,	  /* "dollar" matrix plus subspec */
 	      DMSTR,	  /* "dollar" matrix plus string subspec */
-  /* 70 */    MSL2,	  /* unevaluated matrix subspec */
-	      MSPEC,	  /* evaluated matrix subspec */
+              MSLRAW,	  /* unevaluated matrix subspec */
+  /* 70 */    MSPEC,	  /* evaluated matrix subspec */
 	      SUBSL,	  /* row or column component of MSPEC */
 	      MDEF,	  /* explicit matrix definition {...} */
               LAG,        /* variable plus lag length */	  
@@ -108,27 +108,25 @@ enum {
               OVAR,	  /* object variable: variable "under" an object */
               LIST,	  /* list, evaluated */
 	      LISTVAR,    /* variable in list, dot syntax */
-  /* 80 */    LISTELEM,   /* list member, [...] syntax) */
-	      MLISTELEM,  /* accessor list member, [...] syntax) */
-	      STR,	  /* string */
+              ELEMENT,    /* element of list, array [...] syntax) */
+  /* 80 */    STR,	  /* string */
 	      BUNDLE,     /* gretl bundle (hash table) */
-              BOBJ,       /* object inside a bundle */
-	      BMEMB,      /* object in bundle (dot notation) */
+	      BMEMB,      /* member of bundle */
+	      ARRAY,      /* generic array object */
 	      FARGS,	  /* set of n function arguments */
               WLIST,      /* wildcard list spec */
-              EMPTY,      /* "null" */
-	      ABSENT,
-  /* 90 */    DTYPE_MAX,  /* SEPARATOR: end of "bare" types */
+              EMPTY,      /* "null" or empty arg slot */
+              DTYPE_MAX,  /* SEPARATOR: end of "bare" types */
 	      EROOT,	  /* dummy root for (...) expression */
 	      UFUN,	  /* user-defined function */
-	      RFUN,       /* GNU R function */
+  /* 90 */    RFUN,       /* GNU R function */
 	      USTR,       /* string variable */
 	      IVEC,       /* array of ints, not a varlist */
               INC,        /* increment */
               DEC,        /* decrement */
-	      QUERY,      /* ternary "?" expression */
-	      UNDEF,      /* undefined (in "query" context only) */
-  /* 100 */   EOT,	  /* end of transmission */
+              QUERY,      /* ternary "?" expression */
+              UNDEF,      /* undefined (in "query" context only) */
+              EOT,	  /* end of transmission */
 	      UNK 
 };
 
@@ -225,7 +223,7 @@ enum {
     F_ISLIST,	 
     F_ISSTRING,
     F_ISNULL,
-    F_LISTLEN,
+    F_NELEM,
     F_PDF,
     F_PVAL,
     F_CDF,
@@ -276,6 +274,12 @@ enum {
     F_TYPEOF,
     F_ATOF,
     F_FIXNAME,
+    F_MPI_RECV,
+    F_EASTER,
+    F_CURL,
+    F_NLINES,
+    F_ARRAY,
+    F_STRVALS,
     F1_MAX,	  /* SEPARATOR: end of single-arg functions */
     F_COR,
     F_COV,
@@ -339,10 +343,19 @@ enum {
     F_PSD,
     F_RANDINT,
     F_MREAD,
+    F_BREAD,
     F_GETLINE,
     F_ISODATE,
     F_READFILE,
     F_PRINTF,
+    F_MPI_SEND,
+    F_BCAST,
+    F_ALLREDUCE,
+    F_GENSERIES,
+    F_JSONGET,
+    F_KPSSCRIT,
+    F_STRINGIFY,
+    F_PRINTARRAY,
     F2_MAX,	  /* SEPARATOR: end of two-arg functions */
     F_LLAG,
     F_PRINCOMP,
@@ -374,10 +387,14 @@ enum {
     F_SIMANN,
     F_HALTON,
     F_MWRITE,
+    F_BWRITE,
     F_AGGRBY,
     F_IWISHART,
     F_SSCANF,
     F_SPRINTF,
+    F_SUBSTR,
+    F_REDUCE,
+    F_SCATTER,
     F3_MAX,       /* SEPARATOR: end of three-arg functions */
     F_BKFILT,
     F_MOLS,
@@ -401,7 +418,12 @@ enum {
     CONST_NA,
     CONST_INF,
     CONST_WIN32,
-    CONST_EPS
+    CONST_EPS,
+    CONST_HAVE_MPI,
+    CONST_MPI_RANK,
+    CONST_MPI_SIZE,
+    CONST_N_PROC,
+    CONST_SYSINFO
 };
 
 enum {
@@ -437,14 +459,16 @@ enum {
                              s == F_NRMAX || s == F_DESEAS || \
 			     s == F_AGGRBY || s == F_INBUNDLE || \
 			     s == F_SSCANF || s == F_PRINTF || \
-			     s == F_SPRINTF)
+			     s == F_SPRINTF || s == F_ALLREDUCE || \
+			     s == F_GENSERIES)
 
 /* functions taking string arg in middle position */
-#define string_mid_func(s) (s == F_BFGSMAX || s == F_NRMAX || s == F_SIMANN)
+#define string_mid_func(s) (s == F_REDUCE || s == F_SCATTER)
 
-/* functions taking one or more "fncall" string arguments */
-#define fncall_func(s) (s == F_BFGSMAX || s == F_FDJAC || \
-			s == F_NRMAX || s == F_SIMANN)
+/* functions taking one or more "fncall" (string) arguments */
+#define fncall_func(s) (s == F_BFGSMAX || s == F_NRMAX || \
+			s == F_FDJAC || s == F_SIMANN || \
+			s == F_GENSERIES)
 
 #define unary_op(s)  (s >= 1 && s < U_MAX)
 #define binary_op(s) (s > U_MAX && s < OP_MAX)
@@ -453,9 +477,8 @@ enum {
 #define evalb3(s) (func3_symb(s))
 
 #define evalb2(s) (binary_op(s) || func2_symb(s) || s == MSL || \
-                   s == MSL2 || s == SUBSL || s == LAG || \
-                   s == OBS || s == BOBJ || s == LISTELEM || \
-                   s == BMEMB || s == DOBS)
+                   s == MSLRAW || s == SUBSL || s == LAG || \
+                   s == OBS || s == BMEMB || s == ELEMENT)
 
 #define b1sym(s) (unary_op(s) || func1_symb(s) || funcn_symb(s) || \
                   s == G_LPR || s == EROOT)
@@ -522,13 +545,16 @@ union val {
     gretl_matrix *m;
     matrix_subspec *mspec;
     gretl_bundle *b;
+    gretl_array *a;
+    void *ptr;
 };
 
 enum {
     AUX_NODE = 1 << 0, /* auxiliary: free on exit */
     TMP_NODE = 1 << 1, /* temporary: free content on exit */
     PTR_NODE = 1 << 2, /* node is compatible with P_LHPTR */
-    SVL_NODE = 1 << 3  /* holds string-valued series */
+    SVL_NODE = 1 << 3, /* holds string-valued series */
+    CPY_NODE = 1 << 4  /* node derives from lhs_copy_node() */
 };
 
 struct node {
@@ -545,42 +571,35 @@ enum {
     P_AUTOREG = 1 <<  2, /* expression is autoregressive */
     P_DECL    = 1 <<  3, /* command is actually a declaration */
     P_PRINT   = 1 <<  4, /* command just prints an existing var */
-    P_SCALAR  = 1 <<  5, /* return a scalar result (only) */ 
-    P_SERIES  = 1 <<  6, /* return a series result (only) */
-    P_MATRIX  = 1 <<  7, /* return a matrix result (only) */
-    P_STRING  = 1 <<  8, /* return a string result (only) */
-    P_LIST    = 1 <<  9, /* return a list result (only) */
-    P_PRIVATE = 1 << 10, /* generating a private or internal var */
-    P_COMPILE = 1 << 11, /* just compiling tree, not evaluating */
-    P_EXEC    = 1 << 12, /* evaluating pre-built tree */ 
-    P_SLICE   = 1 << 13, /* compute matrix slice specification */
-    P_UFUN    = 1 << 14, /* user function call with no assignment */
-    P_NATEST  = 1 << 15, /* testing for NAs in expression */
-    P_UFRET   = 1 << 16, /* returning value generated by user function */
-    P_LHSCAL  = 1 << 17, /* there was a pre-existing LHS scalar */
-    P_LHLIST  = 1 << 18, /* there was a pre-existing LHS list */
-    P_LHSTR   = 1 << 19, /* there was a pre-existing LHS string */
-    P_LHMAT   = 1 << 20, /* there was a pre-existing LHS matrix */
-    P_LHBUN   = 1 << 21, /* there was a pre-existing LHS bundle */
-    P_QUIET   = 1 << 22, /* don't print any messages or labels */
-    P_GETSTR  = 1 << 23, /* state: flag acceptance of plain strings */
-    P_SLAVE   = 1 << 24, /* running as "slave" of NLS/MLE/GMM */
-    P_LHPTR   = 1 << 25, /* left-hand side: pointer type wanted */
-    P_MMASK   = 1 << 26, /* genr result is masked matrix */
-    P_SLICING = 1 << 27, /* calculating matrix slice (temporary) */
-    P_LAGPRSE = 1 << 28  /* parsing lag spec (temporary) */
+    P_PRIVATE = 1 <<  5, /* generating a "private" or internal var */
+    P_COMPILE = 1 <<  6, /* just compiling tree, not evaluating */
+    P_EXEC    = 1 <<  7, /* evaluating pre-built tree */ 
+    P_NATEST  = 1 <<  8, /* testing for NAs in expression */
+    P_UFRET   = 1 <<  9, /* returning value generated by user function */
+    P_QUIET   = 1 << 10, /* don't print any messages or labels */
+    P_GETSTR  = 1 << 11, /* state: flag acceptance of plain strings */
+    P_SLAVE   = 1 << 12, /* running as "slave" of NLS/MLE/GMM */
+    P_LHPTR   = 1 << 13, /* left-hand side: pointer type wanted */
+    P_MMASK   = 1 << 14, /* genr result is masked matrix */
+    P_SLICING = 1 << 15, /* calculating object slice (temporary) */
+    P_LAGPRSE = 1 << 16, /* parsing lag spec (temporary) */
+    P_SAVEAUX = 1 << 17, /* indicates try aux node compilation */
+    P_AUXDONE = 1 << 18, /* indicates aux nodes compiled */
+    P_DELTAN  = 1 << 19, /* flag for change in series length */
+    P_LHBKVAR = 1 << 20  /* LHS bundle key is string variable */  
 };
 
 struct lhinfo {
-    int t;                 /* type of result */
+    int t;                 /* type of pre-existing LHS variable, if any */
     char name[VNAMELEN];   /* name of LHS variable */   
-    char label[MAXLABEL];  /* descriptive string for var */
-    int v;                 /* ID number for variable */
+    char label[MAXLABEL];  /* descriptive string for series */
+    int v;                 /* ID number of pre-existing LHS series */
     int obs;               /* specific obs number in series */
     gretl_matrix *m0;      /* original LHS matrix (or NULL) */
-    gretl_matrix *m1;      /* computed LHS matrix */
-    char *substr;          /* obs or matrix selection string */
+    gretl_matrix *m1;      /* computed LHS matrix (or NULL) */
+    char *substr;          /* obs or matrix/array selection string */
     matrix_subspec *mspec; /* evaluated submatrix spec */
+    GretlType atype;       /* type of LHS array, if any */
 };
 
 typedef struct parser_ parser;
@@ -595,15 +614,18 @@ struct parser_ {
     int targ;          /* target type */
     int op;            /* assignment operator (possibly inflected) */
     struct lhinfo lh;  /* left-hand side info */
-    parser *subp;      /* left-hand side matrix subslice tree */
+    parser *subp;      /* left-hand side subslice tree */
     NODE *tree;        /* parsed syntax tree */
     NODE *ret;         /* evaluated result node */
     NODE **aux;        /* auxiliary nodes used in evaluation */
     int n_aux;         /* the number of the above */
     int aux_i;         /* the current ID of the above */
     /* below: parser state variables */
+    int callcount;
+    int dset_n;
     int obs;
     int sym;
+    int upsym;
     int ch;
     double xval;
     int idnum;
@@ -614,7 +636,6 @@ struct parser_ {
 
 #define starting(p) (p->flags & P_START)
 #define autoreg(p) (p->flags & P_AUTOREG)
-#define simple_ufun_call(p) (p->flags & P_UFUN)
 
 int parser_getc (parser *p);
 void parser_ungetc (parser *p);
@@ -628,14 +649,14 @@ NODE *expr (parser *s);
 NODE *newdbl (double x);
 NODE *newempty (void);
 NODE *obs_node (parser *p);
-NODE *msl_node_direct (parser *p);
+NODE *slice_node_direct (parser *p);
 void context_error (int c, parser *p);
 void undefined_symbol_error (const char *s, parser *p);
 const char *getsymb (int t, const parser *p);
 void set_parsing_query (int s);
 
 int realgen (const char *s, parser *p, DATASET *dset, 
-	     PRN *prn, int flags);
+	     PRN *prn, int flags, int targtype);
 void gen_save_or_print (parser *p, PRN *prn);
 void gen_cleanup (parser *p);
 void parser_free_aux_nodes (parser *p);
