@@ -79,8 +79,8 @@ static NODE *newref (parser *p, int t)
     NODE *n = new_node(t);
 
     if (n != NULL) {
-	if (t == UVEC) {
-	    n->t = VEC;
+	if (t == USERIES) {
+	    n->t = SERIES;
 	    n->vnum = p->idnum;
 	    n->v.xvec = p->dset->Z[n->vnum];
 	    n->vname = p->idstr;
@@ -102,6 +102,9 @@ static NODE *newref (parser *p, int t)
 	} else if (t == BUNDLE) {
 	    n->vname = p->idstr;
 	    n->v.b = p->uval;
+	} else if (t == ARRAY) {
+	    n->vname = p->idstr;
+	    n->v.a = p->uval;
 	} else if (t == USTR) {
 	    n->t = STR;
 	    n->vname = p->idstr;
@@ -281,23 +284,23 @@ static NODE *base (parser *p, NODE *up)
 	lex(p);
 	break;
     case UNUM: 
-    case UVEC:
+    case USERIES:
     case UMAT:
     case UOBJ:
     case CON: 
     case DVAR:
     case MVAR:
     case OBS:
-    case DOBS:
     case ULIST:
     case WLIST:
     case BUNDLE:
+    case ARRAY:
     case USTR:
     case UNUM_P:
     case UNUM_M:
     case UNDEF:
-	if (p->sym == OBS || p->sym == DOBS) {
-	    fprintf(stderr, "*** HERE 1: sym OBS or DOBS\n");
+	if (p->sym == OBS) {
+	    fprintf(stderr, "*** HERE 1: sym OBS\n");
 	}
 	t = newref(p, p->sym);
 	lex(p);
@@ -328,9 +331,9 @@ static NODE *base (parser *p, NODE *up)
 	if (up == NULL) {
 	    goto deferr;
 	}
-	if (up->t == OBS || up->t == DOBS) {
+	if (up->t == OBS) {
 	    t = obs_node(p);
-	} else if (up->t == LISTELEM) {
+	} else if (up->t == ELEMENT) {
 	    lex(p);
 	    t = expr(p);
 	}
@@ -408,7 +411,7 @@ static NODE *listvar_node (parser *p)
 	    free(p->idstr);
 	    p->idstr = NULL;
 	    p->idnum = v;
-	    ret = newref(p, UVEC);
+	    ret = newref(p, USERIES);
 	    if (ret == NULL) {
 		p->err = E_ALLOC;
 	    } else {
@@ -444,7 +447,7 @@ static void unwrap_string_arg (parser *p)
 #define varargs_func(f) (f == F_PRINTF || f == F_SPRINTF || \
 			 f == F_SSCANF)
 
-/* Grab a string argument.  Note: we have a mechanism in genlex.c for
+/* Grab a string argument. Note: we have a mechanism in genlex.c for
    retrieving arguments that take the form of quoted string literals
    or names of string variables.  The special use of this function is
    to grab a literal string without requiring the user to wrap it in
@@ -467,11 +470,10 @@ static void unwrap_string_arg (parser *p)
 static NODE *get_final_string_arg (parser *p, NODE *t, int sym, 
 				   int eat_last)
 {
-    char p0 = (sym == BOBJ)? '[' : '(';
-    char p1 = (sym == BOBJ)? ']' : ')';
+    char p0 = '(', p1 = ')';
     const char *src = NULL;
     int n, wrapped = 0;
-    int strsym = STR;
+    int strvar = 0;
 
     while (p->ch == ' ') {
 	parser_getc(p);
@@ -562,18 +564,12 @@ static NODE *get_final_string_arg (parser *p, NODE *t, int sym,
 	    unwrap_string_arg(p);
 	} else if (sym != F_ISSTRING && sym != F_ISNULL) {
 	    /* not quoted: give priority to string variables
-	       unless we need the names of string variables 
+	       unless we need the _names_ of string variables 
 	       rather then their content
 	    */
-	    char *ustr = get_string_by_name(p->idstr);
-
-	    if (ustr != NULL) {
-		p->uval = gretl_strdup(ustr);
-		if (p->uval == NULL) {
-		    p->err = E_ALLOC;
-		} else {
-		    strsym = USTR;
-		}
+	    p->uval = get_string_by_name(p->idstr);
+	    if (p->uval != NULL) {
+		strvar = 1;
 	    }
 	}
     }
@@ -584,32 +580,37 @@ static NODE *get_final_string_arg (parser *p, NODE *t, int sym,
 
     if (p->err) {
 	return NULL;
-    } else if (strsym == USTR) {
+    } else if (strvar) {
 	return newref(p, USTR);
     } else {
 	return newstr(p->idstr);
     }
 }
 
-/* get_middle_string_arg() is used to retrieve a string that
-   defines a function call, e.g. in BFGSmax(). In this 
-   context we expect a literal string, but not wrapped in
-   quotes. 
+enum {
+    RIGHT_STR  = 1 << 0,
+    MID_STR    = 1 << 1,
+    MID_FNCALL = 1 << 2
+};
+
+/* get_middle_string_arg() is mostly used to retrieve a string
+   that defines a function call, e.g. in BFGSmax(). It can also
+   be used to trap a non-quoted string that functions as an
+   "enum" value, as in the "vcat" flag for mpireduce().
+   In the former case @opt will be MID_FNCALL, in the latter
+   MID_STR.
 */
 
-static NODE *get_middle_string_arg (parser *p)
+static NODE *get_middle_string_arg (parser *p, int opt)
 {
-    const char *src = NULL;
-    const char *s;
-    int gotparen = 0, close = 0;
-    int i, quoted, paren = 0;
+    int close = 0;
 
     while (p->ch == ' ') {
 	parser_getc(p);
     }
 
     if (p->ch == '"') {
-	/* special: arg is wrapped in quotes */
+	/* handle the case where the argument is quoted */
 	close = parser_char_index(p, '"');
 	if (close < 0) {
 	    unmatched_symbol_error('"', p);
@@ -617,117 +618,178 @@ static NODE *get_middle_string_arg (parser *p)
 	}
 	p->idstr = gretl_strndup(p->point, close);
 	close++;
-	goto post_process;
-    }
+    } else if (opt == MID_STR) {
+	/* handle the "enum"-type unquoted case */
+	int i1 = parser_char_index(p, ',');
+	int i2 = parser_char_index(p, ')');
 
-    /* find length of string to bare comma (unquoted, not in parens) or
-       matched right paren */
-
-    quoted = 0;
-    s = p->point;
-    i = 0;
-
-    while (*s) {
-	if (*s == '"') {
-	    quoted = !quoted;
+	if (i2 < 0) {
+	    unmatched_symbol_error('(', p);
+	    return NULL;
 	} 
-	if (!quoted) {
-	    if (*s == '(') {
-		gotparen++;
-		paren++;
-	    } else if (*s == ')') {
-		paren--;
-	    }
-	    if (paren == 0) {
-		if (gotparen) {
-		    /* include right paren */
-		    close = i + 1;
-		    break;
-		} else if (*s == ',') {
-		    /* leave comma */
-		    close = i;
-		    break;
-		} 
+	close = (i1 > 0 && i1 < i2)? i1 : i2;
+	p->idstr = gretl_strndup(p->point - 1, close + 1);
+    } else {
+	/* handle the MID_FNCALL case: the terminator of
+	   the argument will be a bare comma (unquoted, not 
+	   in parentheses) or a right paren which matches
+	   the paren that opens the argument list of
+	   the function call.
+	*/
+	const char *s = p->point;
+	int gotparen = 0;
+	int quoted = 0;
+	int paren = 0;
+	int i = 0;
+
+	while (*s) {
+	    if (*s == '"') {
+		quoted = !quoted;
 	    } 
+	    if (!quoted) {
+		if (*s == '(') {
+		    gotparen++;
+		    paren++;
+		} else if (*s == ')') {
+		    paren--;
+		}
+		if (paren == 0) {
+		    if (gotparen) {
+			/* include right paren */
+			close = i + 1;
+			break;
+		    } else if (*s == ',') {
+			/* leave comma */
+			close = i;
+			break;
+		    }
+		} 
+	    }
+	    s++;
+	    i++;
+	}
+
+	if (paren > 0) {
+	    unmatched_symbol_error('(', p);
+	} else if (paren < 0) {
+	    unmatched_symbol_error(')', p);
+	} else if (quoted) {
+	    unmatched_symbol_error('"', p);
+	} else {
+	    p->idstr = gretl_strndup(p->point - 1, close + 1);
 	}
-	s++;
-	i++;
     }
 
-    if (paren > 0) {
-	unmatched_symbol_error('(', p);
-    } else if (paren < 0) {
-	unmatched_symbol_error(')', p);
-    } else if (quoted) {
-	unmatched_symbol_error('"', p);
-    }
-
-    if (p->err) {
-	return NULL;
-    }
-
-    src = p->point - 1;
-    p->idstr = gretl_strndup(src, close + 1);
-
- post_process:
-
     if (p->idstr == NULL) {
-	p->err = E_ALLOC;
-	return NULL;
-    }  
-
-    tailstrip(p->idstr);
-    parser_advance(p, close);
-    lex(p);
-
+	if (!p->err) {
+	    p->err = E_ALLOC;
+	}
+    } else {
+	tailstrip(p->idstr);
+	parser_advance(p, close);
+	lex(p);
 #if SDEBUG
-    fprintf(stderr, "get_middle_string_arg: '%s'\n", p->idstr);
+	fprintf(stderr, "get_middle_string_arg: '%s'\n", p->idstr);
 #endif
+    }
 
-    return (p->err)? NULL : newstr(p->idstr);
+    return (p->idstr == NULL)? NULL : newstr(p->idstr);
 }
 
 static NODE *get_bundle_member_name (parser *p)
 {
-    int i, n = gretl_namechar_spn(p->point);
+    NODE *ret = NULL;
+    const char *s = p->point;
+    int n_extra = 0;
+    int strvar = 0;
+    int i, n = 0;
 
-    if (*(p->point + n) == '[') {
-	/* support "[...]" spec following member name */
-	const char *s = p->point + (++n);
-	int br = 1;
+#if SDEBUG
+    fprintf(stderr, "get_bundle_member_name: p->ch='%c'\n", p->ch);
+#endif
 
-	while (*s && br > 0) {
-	    if (*s == '[') br++;
-	    else if (*s == ']') br--;
-	    n++;
+    if (p->ch == '.') {
+	/* using bundle dot notation */
+	n = gretl_namechar_spn(s);
+	if (*(s + n) == '[') {
+	    /* support "[...]" spec following member name */
+	    const char *q = s + (++n);
+	    int br = 1;
+
+	    while (*q && br > 0) {
+		if (*q == '[') br++;
+		else if (*q == ']') br--;
+		n++;
+		q++;
+	    }
+
+	    if (br != 0) {
+		unmatched_symbol_error('[', p);
+	    }
+	}
+    } else if (p->ch == '[') {
+	/* using bracketed key notation */
+	if (*s == '"') {
+	    /* key is quoted string literal */
+	    parser_getc(p);
 	    s++;
+	    if (strchr(s, '"') == NULL) {
+		p->err = E_PARSE;
+	    } else {
+		n = strcspn(s, "\"");
+		n_extra = 2;
+		s += n + 1;
+	    }
+	} else {
+	    /* try for a string variable */
+	    n = gretl_namechar_spn(s);
+	    if (n == 0 || n >= VNAMELEN) {
+		p->err = E_PARSE;
+	    } else {
+		strvar = 1;
+		n_extra = 1;
+		s += n;
+	    }
 	}
-
-	if (br != 0) {
+	if (!p->err && *s != ']') {
 	    unmatched_symbol_error('[', p);
-	    return NULL;
+	}
+    }
+	
+    if (!p->err) {
+	p->idstr = gretl_strndup(p->point, n);
+	if (p->idstr == NULL) {
+	    p->err = E_ALLOC;
+	} else {
+	    n += n_extra;
+	    for (i=0; i<=n; i++) {
+		parser_getc(p);
+	    }
+	    lex(p);
 	}
     }
 
-    p->idstr = gretl_strndup(p->point, n);
-    if (p->idstr == NULL) {
-	p->err = E_ALLOC;
-	return NULL;
+#if SDEBUG
+    fprintf(stderr, " %s = '%s'\n", strvar ? "key-vname" : "key", p->idstr);
+#endif
+    
+    if (!p->err) {
+	if (strvar) {
+	    p->uval = get_string_by_name(p->idstr);
+	    if (p->uval == NULL) {
+		gretl_errmsg_sprintf(_("%s: not a string variable"), p->idstr);
+		p->err = E_DATA;
+	    } else {
+		ret = newref(p, USTR);
+	    }
+	} else {
+	    ret = newstr(p->idstr);
+	}
     }
 
-    for (i=0; i<=n; i++) {
-	parser_getc(p);
-    }
-    lex(p);
-
-    return newstr(p->idstr);
+    return ret;
 }
 
-enum {
-    RIGHT_STR = 1 << 0,
-    MID_STR   = 1 << 1
-};
-
 static void get_matrix_def (NODE *t, parser *p, int *sub)
 {
     NODE *n;
@@ -784,8 +846,8 @@ static void get_matrix_def (NODE *t, parser *p, int *sub)
     }
 }
 
-#define set_matrix_slice_on(p) (p->flags |= P_SLICING)
-#define set_matrix_slice_off(p) (p->flags &= ~P_SLICING)
+#define set_slice_on(p) (p->flags |= P_SLICING)
+#define set_slice_off(p) (p->flags &= ~P_SLICING)
 
 #define set_lag_parse_on(p) (p->flags |= P_LAGPRSE)
 #define set_lag_parse_off(p) (p->flags &= ~P_LAGPRSE)
@@ -798,7 +860,7 @@ static void get_slice_parts (NODE *t, parser *p)
     fprintf(stderr, "get_slice_parts, p->sym = %d\n", p->sym);
 #endif  
 
-    set_matrix_slice_on(p);
+    set_slice_on(p);
 
     if (p->sym == G_LBR) {
 	lex(p);
@@ -821,10 +883,9 @@ static void get_slice_parts (NODE *t, parser *p)
 	}
 	if (p->sym == G_RBR) {
 	    /* no comma, no second arg string: may be OK */
-	    t->v.b2.r = newempty();
-	    t->v.b2.r->t = ABSENT;
+	    t->v.b2.r = NULL;
 	    lex(p);
-	    set_matrix_slice_off(p);
+	    set_slice_off(p);
 	    return;
 	}
 	if (p->sym == P_COM) {
@@ -862,7 +923,7 @@ static void get_slice_parts (NODE *t, parser *p)
 	expected_symbol_error(cexp, p);
     }
 
-    set_matrix_slice_off(p);
+    set_slice_off(p);
 }
 
 static void attach_child (NODE *parent, NODE *child, int k, int i,
@@ -909,7 +970,7 @@ static void pad_parent (NODE *parent, int k, int i, parser *p)
     }
 }
 
-#define next_arg_is_string(i,k,o) ((i>0 && i<k-1 && (o & MID_STR)) || \
+#define next_arg_is_string(i,k,o) ((i>0 && i<k-1 && (o & (MID_STR|MID_FNCALL))) || \
                                    (i==k-1 && (o & RIGHT_STR)))
 
 /* Get up to k comma-separated arguments (possibly optional).
@@ -947,8 +1008,8 @@ static void get_args (NODE *t, parser *p, int f, int k, int opt, int *next)
 	}
 
 	/* get the next argument */
-	if (i > 0 && i < k - 1 && (opt & MID_STR)) {
-	    child = get_middle_string_arg(p);
+	if (i > 0 && i < k - 1 && (opt & (MID_STR|MID_FNCALL))) {
+	    child = get_middle_string_arg(p, opt);
 	} else if (i == k - 1 && (opt & RIGHT_STR)) {
 	    child = get_final_string_arg(p, t, f, 0);
 	} else {
@@ -1042,7 +1103,9 @@ static NODE *powterm (parser *p)
 
     if (string_mid_func(sym)) {
 	opt |= MID_STR;
-    }    
+    } else if (fncall_func(sym) && !func2_symb(sym)) {
+	opt |= MID_FNCALL;
+    }
 
     if (unary_op(sym)) {
 	if (p->ch == 0) {
@@ -1103,13 +1166,13 @@ static NODE *powterm (parser *p)
 		next = G_LBR;
 	    }
 	}
-    } else if (sym == LAG || sym == OBS || sym == DOBS) {
+    } else if (sym == LAG || sym == OBS) {
 	if (sym == LAG) {
 	    set_lag_parse_on(p);
 	}
 	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
-	    t->v.b2.l = newref(p, sym == DOBS ? DVAR : UVEC);
+	    t->v.b2.l = newref(p, p->upsym);
 	    lex(p);
 	    t->v.b2.r = base(p, t);
 	}
@@ -1124,7 +1187,7 @@ static NODE *powterm (parser *p)
 	    } else {
 		t->v.b2.l = newref(p, MVAR);
 	    }
-	    t->v.b2.r = newb2(MSL2, NULL, NULL);
+	    t->v.b2.r = newb2(MSLRAW, NULL, NULL);
 	    if (t->v.b2.r != NULL) {
 		lex(p);
 		get_slice_parts(t->v.b2.r, p);
@@ -1137,23 +1200,12 @@ static NODE *powterm (parser *p)
 	    lex(p);
 	    t->v.b2.r = get_final_string_arg(p, t, sym, 1);
 	}
-    } else if (sym == LISTELEM || sym == MLISTELEM) {
-	t = newb2(LISTELEM, NULL, NULL);
-	if (t != NULL) {
-	    if (sym == LISTELEM) {
-		t->v.b2.l = newref(p, ULIST);
-	    } else {
-		t->v.b2.l = newref(p, MVAR);
-	    }
-	    lex(p);
-	    t->v.b2.r = base(p, t);
-	}
-    } else if (sym == BOBJ) {
+    } else if (sym == ELEMENT) {
 	t = newb2(sym, NULL, NULL);
 	if (t != NULL) {
-	    t->v.b2.l = newref(p, BUNDLE);
+	    t->v.b2.l = newref(p, p->upsym);
 	    lex(p);
-	    t->v.b2.r = get_final_string_arg(p, t, sym, 1);
+	    t->v.b2.r = base(p, t);
 	}
     } else if (sym == BMEMB) {
 	t = newb2(sym, NULL, NULL);
@@ -1199,7 +1251,7 @@ static NODE *powterm (parser *p)
 	    if (sub) {
 		t = newb2(MSL, t, NULL);
 		if (t != NULL) {
-		    t->v.b2.r = newb2(MSL2, NULL, NULL);
+		    t->v.b2.r = newb2(MSLRAW, NULL, NULL);
 		    if (t->v.b2.r != NULL) {
 			lex(p);
 			get_slice_parts(t->v.b2.r, p);
@@ -1236,7 +1288,7 @@ static NODE *powterm (parser *p)
 	/* support func(args)[slice] */
 	t = newb2(MSL, t, NULL);
 	if (t != NULL) {
-	    t->v.b2.r = newb2(MSL2, NULL, NULL);
+	    t->v.b2.r = newb2(MSLRAW, NULL, NULL);
 	    if (t->v.b2.r != NULL) {
 		get_slice_parts(t->v.b2.r, p);
 	    }
@@ -1557,14 +1609,14 @@ NODE *expr (parser *p)
     return t;
 }
 
-/* for use when we need to evaluate a sub-matrix specification
-   on the left-hand side of a genr formula */
+/* for use when we need to evaluate a sub-matrix or sub-array
+   specification on the left-hand side of a genr formula */
 
-NODE *msl_node_direct (parser *p)
+NODE *slice_node_direct (parser *p)
 {
     NODE *t;
 
-    t = newb2(MSL2, NULL, NULL);
+    t = newb2(MSLRAW, NULL, NULL);
     if (t != NULL) {
 	lex(p);
 	get_slice_parts(t, p);
