@@ -963,7 +963,7 @@ static int gmm_update_e (nlspec *s)
     for (j=0; j<s->oc->e->cols && !err; j++) {
 	v = s->oc->ecols[j].v;
 	if (v > 0) {
-#if GMM_DEBUG
+#if GMM_DEBUG > 1
 	    fprintf(stderr, "gmm_update_e: updating col %d from var %d\n",
 		    j, v);
 #endif
@@ -973,7 +973,7 @@ static int gmm_update_e (nlspec *s)
 		gretl_matrix_set(s->oc->e, t, j, etj);
 	    }
 	} else {
-#if GMM_DEBUG
+#if GMM_DEBUG > 1
 	    fprintf(stderr, "gmm_update_e: updating col %d from matrix %s\n",
 		    j, s->oc->ecols[j].mname);
 #endif
@@ -998,41 +998,15 @@ static int gmm_update_e (nlspec *s)
 
 static int gmm_multiply_ocs (nlspec *s)
 {
-    int err = 0;
+    int err;
 
-    if (s->oc->S == NULL) {
-	/* simple case: no selection mechanism needed */
-	err = gretl_matrix_columnwise_product(s->oc->e,
-					      s->oc->Z,
-					      s->oc->tmp);
-	if (err) {
-	    fprintf(stderr, "gmm_multiply_ocs: err = %d from "
-		    "gretl_matrix_columnwise_product\n", err);
-	}
-    } else {
-	/* select the wanted i, j pairs */
-	int i, j, k, m, t, p = 0;
-	double x, y;
-
-	m = s->oc->e->cols;
-	k = s->oc->Z->cols;
-
-	for (i=0; i<m; i++) {
-	    for (j=0; j<k; j++) {
-		if (gretl_matrix_get(s->oc->S, i, j) != 0) {
-#if GMM_DEBUG
-		    fprintf(stderr, "O.C. %d: multiplying col %d of e "
-			    "into col %d of Z\n", p, i, j);
-#endif
-		    for (t=0; t<s->nobs; t++) {
-			x = gretl_matrix_get(s->oc->e, t, i);
-			y = gretl_matrix_get(s->oc->Z, t, j);
-			gretl_matrix_set(s->oc->tmp, t, p, x * y);
-		    }
-		    p++;
-		}
-	    }
-	}
+    err = gretl_matrix_columnwise_product(s->oc->e,
+					  s->oc->Z,
+					  s->oc->S,
+					  s->oc->tmp);
+    if (err) {
+	fprintf(stderr, "gmm_multiply_ocs: err = %d from "
+		"gretl_matrix_columnwise_product\n", err);
     }
 
 #if GMM_DEBUG > 1
@@ -1045,63 +1019,64 @@ static int gmm_multiply_ocs (nlspec *s)
     return err;
 }
 
-#define CRIT_DEBUG 0
+static double gmm_criterion (nlspec *s)
+{
+    double crit = 0.0;
+    gretl_matrix *sum = s->oc->sum;
+    gretl_matrix *W = s->oc->W;
+    int k = s->oc->noc;
+    int i, j, err;
+
+    err = gmm_multiply_ocs(s);
+    if (err) {
+	return NADBL;
+    }
+
+    /* compute column sums */
+    for (i=0; i<k; i++) {
+	sum->val[i] = 0.0;
+	for (j=0; j<s->nobs; j++) {
+	    sum->val[i] += gretl_matrix_get(s->oc->tmp, j, i);
+	}
+    }
+
+    crit = gretl_scalar_qform(sum, W, &err);
+    if (!err) {
+	crit = -crit;
+    }
+
+    return crit;
+}
 
 /* calculate the value of the GMM criterion given the current
    parameter values */
 
-static double get_gmm_crit (const double *b, void *p)
+static double get_gmm_crit (const double *b, void *data)
 {
-    nlspec *s = (nlspec *) p;
-    int i, k, t;
-    int err = 0;
+    nlspec *s = (nlspec *) data;
+    int err;
 
     update_coeff_values(b, s);
 
-    if ((err = nl_calculate_fvec(s))) {
-#if CRIT_DEBUG
+    err = nl_calculate_fvec(s);
+    if (err) {
 	fprintf(stderr, "get_gmm_crit: err on nl_calculate_fvec = %d\n", err);
-#endif
 	return NADBL;
     }
 
-    if ((err = gmm_update_e(s))) {
-#if CRIT_DEBUG
+    err = gmm_update_e(s);
+    if (err) {
 	fprintf(stderr, "get_gmm_crit: err on gmm_update_e = %d\n", err);
-#endif
 	return NADBL;
     }
 
-    if ((err = gmm_multiply_ocs(s))) {
-#if CRIT_DEBUG
-	fprintf(stderr, "get_gmm_crit: err on gmm_multiply_ocs = %d\n", err);
-#endif
-	return NADBL;
-    }
-    
-    k = s->oc->noc;
-
-    for (i=0; i<k; i++) {
-	s->oc->sum->val[i] = 0.0;
-	for (t=0; t<s->nobs; t++) {
-	    s->oc->sum->val[i] += gretl_matrix_get(s->oc->tmp, t, i);
-	}
-    }
-
-    s->crit = gretl_scalar_qform(s->oc->sum, s->oc->W, &err);
-    if (!err) {
-	s->crit = -s->crit;
-    }
+    s->crit = gmm_criterion(s);
 
 #if GMM_DEBUG > 2
     gretl_matrix_print(s->oc->sum, "GMM: s->oc->sum");
     fprintf(stderr, "GMM: s->crit = %g\n", s->crit);
 #endif
 
-    if (err) {
-	s->crit = NADBL;
-    }
-	
     return s->crit;
 }
 
@@ -1200,29 +1175,11 @@ static int HAC_prewhiten (gretl_matrix *E, gretl_matrix *A)
 #endif
 
     if (!err) {
-	gretl_matrix *U = NULL;
-	gretl_matrix *S = NULL;
-	gretl_matrix *V = NULL;
-	gretl_matrix *tmp = NULL;
-	int amod = 0;
-
-	err = gretl_matrix_SVD(A, &U, &S, &V);
+	err = maybe_limit_VAR_coeffs(A, NULL, NULL, NULL);
 	if (err) {
 	    goto bailout;
 	}
 
-	for (i=0; i<k; i++) {
-	    if (S->val[i] > 0.97) {
-		S->val[i] = 0.97;
-		amod = 1;
-	    }
-	}
-
-	if (amod) {
-	    tmp = gretl_matrix_dot_op(U, S, '*', &err);
-	    gretl_matrix_multiply(tmp, V, A);
-	}
-
 #if GMM_DEBUG
 	/* (possibly) modified VAR coefficients */
 	gretl_matrix_print(A, "A~");
@@ -1235,7 +1192,7 @@ static int HAC_prewhiten (gretl_matrix *E, gretl_matrix *A)
 
 	/* Now "whiten" E using A~ */
 	for (t=1; t<T; t++) {
-	    /* re-use 'b' for fitted values */
+	    /* re-use @b for fitted values */
 	    gretl_matrix_multiply(A, e, b);
 	    for (i=0; i<k; i++) {
 		/* retrieve current value */
@@ -1246,11 +1203,6 @@ static int HAC_prewhiten (gretl_matrix *E, gretl_matrix *A)
 		gretl_matrix_set(E, t, i, eti - b->val[i]);
 	    }
 	}
-
-	gretl_matrix_free(U);
-	gretl_matrix_free(S);
-	gretl_matrix_free(V);
-	gretl_matrix_free(tmp);
     }
 
  bailout:
@@ -1307,7 +1259,7 @@ static int gmm_HAC (gretl_matrix *E, gretl_matrix *V, hac_info *hinfo)
     }
 
     if (data_based_hac_bandwidth()) {
-	err = newey_west_bandwidth(E, hinfo->kern, &hinfo->h,
+	err = newey_west_bandwidth(E, NULL, hinfo->kern, &hinfo->h,
 				   &hinfo->bt);
 	if (err) {
 	    return err;
@@ -1434,7 +1386,7 @@ int gmm_add_vcv (MODEL *pmod, nlspec *s)
 	    f[i] *= Tfac;
 	}
 
-	fdjac2_(gmm_jacobian_calc, m, n, s->coeff, f, 
+	fdjac2_(gmm_jacobian_calc, m, n, 0, s->coeff, f, 
 		J->val, m, &iflag, 0.0, wa4, s);
 
 	if (iflag != 0) {
@@ -1595,6 +1547,8 @@ static void gmm_print_oc (nlspec *s, PRN *prn)
     gretl_matrix_free(V);
 }
 
+#define NORM_DEBUG 0
+
 static double gmm_log_10 (double x)
 {
     double y;
@@ -1618,7 +1572,7 @@ static double gmm_log_10 (double x)
    keeping computer arithmetic in bounds.
 */
 
-static int maybe_prescale_weights (nlspec *s)
+static int gmm_normalize_wts_1 (nlspec *s)
 {
     double *coeff;
     double crit;
@@ -1633,14 +1587,13 @@ static int maybe_prescale_weights (nlspec *s)
     if (crit > 0 && !na(crit)) {
 	double m, lc = gmm_log_10(crit);
 
-#if 0
+#if NORM_DEBUG
 	fprintf(stderr, "maybe_preadjust_weights: crit=%g, lc=%g\n", 
 		crit, lc);
 	gretl_matrix_print(s->oc->sum, "s->oc->sum");
 #endif
 
-	if (!na(lc) && (lc > 5 || lc < -5)) {
-	    /* FIXME ? */
+	if (!na(lc) && (lc > 4 || lc < -4)) { /* was 5 */
 	    if (lc > 0) {
 		m = floor(lc/2);
 	    } else {
@@ -1658,6 +1611,77 @@ static int maybe_prescale_weights (nlspec *s)
     return 0;
 }
 
+/* the idea here is to rescale rows and columns of the initial
+   weights matrix so that 1-step becomes independent of the 
+   order of magnitude of the individual orthogonality conditions
+*/
+
+static int gmm_normalize_wts_2 (nlspec *s)
+{
+    double *coeff;
+    double inicrit;
+    int err = 0;
+
+    /* don't mess with what might be a user-specified
+       initialization of s->coeff */
+    coeff = copyvec(s->coeff, s->ncoeff);
+    if (coeff == NULL) {
+	return E_ALLOC;
+    }
+
+    inicrit = -1 * get_gmm_crit(coeff, s);
+
+    if (inicrit > 0 && !na(inicrit)) {
+	int k = s->oc->noc;
+	int n = s->oc->tmp->rows;
+	gretl_vector *qvec;
+
+	qvec = gretl_column_vector_alloc(k);
+
+	if (qvec == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    double *q = qvec->val;
+	    double x, sq = 0;
+	    double scale = 0;
+	    int i, j;
+#if NORM_DEBUG
+	    gretl_matrix_print(s->oc->W, "Weights (before)");
+#endif
+	    for (j=0; j<k; j++) {
+		q[j] = 0;
+		for (i=0; i<n; i++) {
+		    x = gretl_matrix_get(s->oc->tmp, i, j);
+		    q[j] += x*x;
+		}
+		sq += q[j];
+		scale += gretl_matrix_get(s->oc->W, j, j); 
+	    }
+
+	    scale /= n * sq;
+	    fprintf(stderr, "gmm_normalize: scale = %g\n", scale);
+
+	    for (i=0; i<k; i++) {
+		for (j=0; j<k; j++) {
+		    x = gretl_matrix_get(s->oc->W, i, j);
+		    x /= scale * sqrt(q[i]*q[j]) * n;
+		    gretl_matrix_set(s->oc->W, i, j, x);
+		}
+	    }
+
+#if NORM_DEBUG
+	    gretl_matrix_print(qvec, "q");
+	    gretl_matrix_print(s->oc->W, "Weights (after)");
+#endif
+	    gretl_matrix_free(qvec);
+	}
+    }
+
+    free(coeff);
+
+    return err;
+}
+
 /* BFGS driver, for the case of GMM estimation.  We may have to handle
    both "inner" (BFGS) and "outer" iterations here: the "outer"
    iterations (if applicable) are re-runs of BFGS using an updated
@@ -1666,6 +1690,7 @@ static int maybe_prescale_weights (nlspec *s)
 
 int gmm_calculate (nlspec *s, PRN *prn)
 {
+    static int normalize = 0;
     int full_fncount = 0;
     int full_grcount = 0;
     double itol = 1.0e-12, icrit = 1;
@@ -1677,6 +1702,14 @@ int gmm_calculate (nlspec *s, PRN *prn)
     int converged = 0;
     int err = 0;
 
+    if (normalize == 0) {
+	if (getenv("GMM_NORMALIZE_2") != NULL) {
+	    normalize = 2;
+	} else {
+	    normalize = 1;
+	}
+    }
+
     if (s->opt & OPT_I) {
 	/* iterate */
 	oldcoeff = copyvec(s->coeff, s->ncoeff);
@@ -1690,9 +1723,13 @@ int gmm_calculate (nlspec *s, PRN *prn)
 	outer_max = 2;
     }
 
-    /* experimental, 2010-04-22 */
-    if (!s->oc->userwts) {
-	maybe_prescale_weights(s);
+    /* experimental, 2013-12-30 */
+    if (normalize == 1) {
+	if (!s->oc->userwts) {
+	    gmm_normalize_wts_1(s);
+	}
+    } else if (normalize == 2) {
+	err = gmm_normalize_wts_2(s);
     }
 
     while (!err && outer_iters < outer_max && !converged) {
@@ -1701,7 +1738,6 @@ int gmm_calculate (nlspec *s, PRN *prn)
 	fprintf(stderr, "GMM calling BFGS: outer_iters = %d\n",
 		outer_iters);
 #endif
-
 	s->crit = 0.0;
 
 	err = BFGS_max(s->coeff, s->ncoeff, maxit, s->tol, 
@@ -1709,6 +1745,10 @@ int gmm_calculate (nlspec *s, PRN *prn)
 		       get_gmm_crit, C_GMM, NULL, s,
 		       NULL, iopt, s->prn);
 
+#if GMM_DEBUG
+	fprintf(stderr, "GMM BFGS: err = %d\n", err);
+#endif
+
 	/* don't keep displaying certain things */
 	iopt |= OPT_Q;
 
@@ -1749,14 +1789,12 @@ int gmm_calculate (nlspec *s, PRN *prn)
 #endif
     }
 
-#if 1
-    if (!err && !(s->opt & (OPT_T | OPT_I))) {
-	/* we automatically scaled the identity matrix for weighting
+    if (!err && normalize == 1 && s->oc->scale_wt != 1) {
+	/* we auto-scaled the identity matrix for weighting
 	   purposes: undo the scaling here
 	*/
 	s->crit /= s->oc->scale_wt;
     }
-#endif
 
     if (oldcoeff != NULL) {
 	free(oldcoeff);
