@@ -261,8 +261,8 @@ static void hess_b_adjust_ij (double *c, const double *b, double *h, int n,
    Extrapolation.  It is derived from code in the gnu R package
    "numDeriv" by Paul Gilbert, which was in turn derived from code
    by Xinqiao Liu.  Turned into C and modified for gretl by
-   Allin Cottrell, June 2006.  On successful completion, returns
-   the negative inverse of the Hessian.
+   Allin Cottrell, June 2006.  On successful completion, writes
+   the negative inverse of the Hessian into @H.
 */
 
 static int numerical_hessian (const double *b, gretl_matrix *H,
@@ -539,7 +539,6 @@ static int richardson_gradient (double *b, double *g, int n,
     double bi0, f1, f2;
     int r = RSTEPS;
     int i, k, m;
-    int err = 0;
 
     for (i=0; i<n; i++) {
 	bi0 = b[i];
@@ -559,7 +558,7 @@ static int richardson_gradient (double *b, double *g, int n,
 	b[i] = bi0;
 	p4m = 4.0;
 	for (m=0; m<r-1; m++) {
-	    for (k=0; k<r-m; k++) {
+	    for (k=0; k<r-m-1; k++) {
 		df[k] = (df[k+1] * p4m - df[k]) / (p4m - 1.0);
 	    }
 	    p4m *= 4.0;
@@ -567,7 +566,7 @@ static int richardson_gradient (double *b, double *g, int n,
 	g[i] = df[0];
     }
 
-    return err;
+    return 0;
 }
 
 /* trigger for switch to Richardson gradient */
@@ -631,7 +630,7 @@ int numeric_gradient (double *b, double *g, int n,
     return err;
 }
 
-#define stepfrac	0.2
+#define STEPFRAC	0.2
 #define acctol		1.0e-7 /* alt: 0.0001 or 1.0e-7 (?) */
 #define reltest		10.0
 
@@ -654,12 +653,13 @@ static int broken_gradient (double *g, int n)
    If "set initvals" has been used, replace whatever initial values
    might have been in place with those given by the user (the customer
    is always right).  In addition, respect user settings for the
-   maximum number of iterations and the convergence tolerance.
+   maximum number of iterations, the convergence tolerance and
+   so on.
 */
 
 static void optim_get_user_values (double *b, int n, int *maxit,
 				   double *reltol, double *gradmax,
-				   gretlopt opt, PRN *prn)
+				   int *quad, gretlopt opt, PRN *prn)
 {
     const gretl_matrix *uinit;
     int uilen, umaxit;
@@ -700,9 +700,7 @@ static void optim_get_user_values (double *b, int n, int *maxit,
 	return;
     }
 
-    /* then check for a setting of the maximum number
-       of iterations */
-
+    /* check for a setting of the maximum number of iterations */
     umaxit = libset_get_int(BFGS_MAXITER);
     if (umaxit >= 0) {
 	*maxit = umaxit;
@@ -710,8 +708,7 @@ static void optim_get_user_values (double *b, int n, int *maxit,
 	*maxit = BFGS_MAXITER_DEFAULT;
     }
 
-    /* and then the convergence tolerance */
-
+    /* convergence tolerance */
     utol = libset_get_user_tolerance(BFGS_TOLER);
     if (!na(utol)) {
 	/* the user has actually set a value */
@@ -724,9 +721,15 @@ static void optim_get_user_values (double *b, int n, int *maxit,
 	*reltol = libset_get_double(BFGS_TOLER);
     }
 
-    /* and the maximum acceptable gradient norm */
-
+    /* maximum acceptable gradient norm */
     *gradmax = libset_get_double(BFGS_MAXGRAD);
+
+    /* step-length algorithm (BFGS only at present) */
+    if (quad != NULL) {
+	if (libset_get_int(OPTIM_STEPLEN) == STEPLEN_QUAD) {
+	    *quad = 1;
+	}
+    }
 }
 
 #define bfgs_print_iter(v,s,i) (v && (s == 1 || i % s == 0))
@@ -762,26 +765,185 @@ static int copy_initial_hessian (gretl_matrix *A, double **H,
     return 0;
 }
 
+/* returns number of coefficient that have actually changed */
+
+static int coeff_at_end (double *b, const double *X, const double *t, 
+			 int n, double length)
+{
+    int i, ndelta = n;
+
+    for (i=0; i<n; i++) {
+	b[i] = X[i] + length * t[i];
+	if (coeff_unchanged(b[i], X[i])) {
+	    ndelta--;
+	}
+    }
+
+    return ndelta;
+}
+
+static double quad_slen (int n, int *pndelta, double *b, 
+			 const double *X, const double *t, 
+			 double *pf, BFGS_CRIT_FUNC cfunc, void *data, 
+			 double g0, double f0, int *pfcount)
+{
+    double d, f1 = *pf;
+    double steplen = 1.0, endpoint = 1.0;
+    int ndelta, crit_ok = 0, fcount = 0, f1_done = 0;
+    double safelen = 1.0e-12;
+    double incredible = -1.0e12;
+
+    /* Below: iterate so long as (a) we haven't achieved an acceptable
+       value of the criterion and (b) there is still some prospect
+       of doing so.
+    */    
+
+    do {
+	crit_ok = 0;
+	ndelta = coeff_at_end(b, X, t, n, endpoint);
+
+	if (ndelta > 0) {
+	    if (!f1_done) {
+		f1 = cfunc(b, data);
+		fcount++;
+	    }
+	    d = -g0 * endpoint * acctol;
+
+	    /* find the optimal steplength by quadratic interpolation; 
+	       inspired by Kelley (1999), "Iterative Methods for Optimization", 
+	       especially section 3.2.1. 
+	    */
+	    if (xna(f1)) {
+		/* function goes into NA zone, presumably outside the 
+		   admissible parameter space; hence, try a much smaller 
+		   step. FIXME execution can come back here indefinitely.
+		*/
+		endpoint *= STEPFRAC;
+#if BFGS_DEBUG
+		fprintf(stderr, "quad_slen: f1 is NA; trimming\n");
+#endif
+	    } else if ((f1 - f0) < incredible) {
+		/* Same as above, with the exception that the objective
+		   function technically computes, but returns a fishy value.
+		*/
+		endpoint *= STEPFRAC;
+#if BFGS_DEBUG
+		fprintf(stderr, "opt_slen: %g is incredible; trimming\n", 
+			f1 - f0);
+#endif
+	    } else if (f1 < f0 + d) {
+		/* function computes, but goes down: try quadratic approx */
+		steplen = 0.5 * endpoint * g0 / (f0 - f1 + g0);
+#if BFGS_DEBUG
+		fprintf(stderr, "quad_slen, interpolate: f0 = %g, f1 = %g, g0 = %g\n", 
+			f0, f1, g0);
+		fprintf(stderr, "quad_slen, interpolate: endpoint = %g, "
+			"steplen = %g\n", endpoint, steplen);
+#endif
+		    
+		if (steplen < safelen) {
+		    /* We have a ludicrously small steplength here,
+		       most likely because the endpoint is too far out.
+		       Let's trim it down and retry. 
+		    */
+		    endpoint *= STEPFRAC;
+		} else {
+		    ndelta = coeff_at_end(b, X, t, n, steplen);
+		    f1 = cfunc(b, data);
+		    fcount++;		    
+#if BFGS_DEBUG
+		    fprintf(stderr, "quad_slen, interpolate: %g is safe\n", steplen); 
+#endif
+		    crit_ok = !na(f1) && (f1 >= f0 + d);
+		    /* if the function still goes down (or berserk), let's 
+		       trim the endpoint one more time and retry */
+#if BFGS_DEBUG
+		    fprintf(stderr, "quad_slen, interpolate: crit_ok = %d"
+			    " (f1 = %g)\n", crit_ok, f1);
+#endif
+		    if (!crit_ok) {
+			endpoint = steplen;
+			f1_done = 1;
+		    } else {
+			f1_done = 0;
+		    }
+		}
+	    } else {
+		crit_ok = 1;
+		steplen = endpoint;
+	    }
+	}
+    } while (ndelta > 0 && !crit_ok);
+
+#if BFGS_DEBUG
+    fprintf(stderr, "opt_slen: steplen = %g\n", steplen);
+#endif
+    *pndelta = ndelta;
+    *pfcount += fcount;
+    *pf = f1;
+
+    return steplen;
+}
+
+static double simple_slen (int n, int *pndelta, double *b, double *X, double *t, 
+			   double *pf, BFGS_CRIT_FUNC cfunc, void *data, 
+			   double g0, double f0, int *pfcount)
+{
+    double d, f1 = *pf, steplen = 1.0;
+    int i, crit_ok = 0, fcount = 0;
+    int ndelta = *pndelta;
+
+    /* Below: iterate so long as (a) we haven't achieved an acceptable
+       value of the criterion and (b) there is still some prospect
+       of doing so.
+    */    
+
+    do {
+	ndelta = n;
+	crit_ok = 0;
+	for (i=0; i<n; i++) {
+	    b[i] = X[i] + steplen * t[i];
+	    if (coeff_unchanged(b[i], X[i])) {
+		ndelta--;
+	    }
+	}
+	if (ndelta > 0) {
+	    f1 = cfunc(b, data);
+	    d = -g0 * steplen * acctol;
+	    fcount++;
+	    crit_ok = !na(f1) && (f1 >= f0 + d);
+	    if (!crit_ok) {
+		/* calculated criterion no good: try smaller step */
+		steplen *= STEPFRAC;
+	    }
+	}
+    } while (ndelta != 0 && !crit_ok);
+
+    *pndelta = ndelta;
+    *pfcount += fcount;
+    *pf = f1;
+
+    return steplen;
+}
+
 static int BFGS_orig (double *b, int n, int maxit, double reltol,
 		      int *fncount, int *grcount, BFGS_CRIT_FUNC cfunc, 
 		      int crittype, BFGS_GRAD_FUNC gradfunc, void *data, 
 		      gretl_matrix *A0, gretlopt opt, PRN *prn)
 {
-    int crit_ok, done;
+    int verbskip, verbose = (opt & OPT_V);
     double *wspace = NULL;
     double **H = NULL;
     double *g, *t, *X, *c;
-    int verbskip, verbose = (opt & OPT_V);
     int fcount, gcount, ndelta = 0;
-    int show_activity = 0;
+    int quad = 0, show_activity = 0;
     double sumgrad, gradmax, gradnorm = 0.0;
-    double fmax, f, f0, d = 0.0;
-    double s, steplen = 0.0;
+    double fmax, f, f0, s, steplen = 0.0;
     double D1, D2;
-    int i, j, ilast, iter;
+    int i, j, ilast, iter, done;
     int err = 0;
 
-    optim_get_user_values(b, n, &maxit, &reltol, &gradmax, opt, prn);
+    optim_get_user_values(b, n, &maxit, &reltol, &gradmax, &quad, opt, prn);
 
     if (gradfunc == NULL) {
 	gradfunc = numeric_gradient;
@@ -882,37 +1044,24 @@ static int BFGS_orig (double *b, int n, int maxit, double reltol,
 	fprintf(stderr, "\niter %d: sumgrad = %g\n", iter, sumgrad);
 #endif
 
+#if BFGS_DEBUG
+	fprintf(stderr, "H = \n");
+	for (i=0; i<n; i++) {
+	    for (j=0; j<=i; j++) {
+		fprintf(stderr, "%15.6f", H[i][j]);
+	    }
+	    fputc('\n', stderr);
+	}
+#endif
 	if (sumgrad > 0.0) { 
 	    /* heading in the right direction */
-	    steplen = 1.0;
-	    crit_ok = 0;
-	    do {
-		/* loop so long as (a) we haven't achieved an
-		   acceptable value of the criterion and (b) there is
-		   still some prospect of doing so */
-		ndelta = n;
-		for (i=0; i<n; i++) {
-		    b[i] = X[i] + steplen * t[i];
-		    if (coeff_unchanged(b[i], X[i])) {
-			ndelta--;
-		    }
-		}
-		if (ndelta > 0) {
-		    f = cfunc(b, data);
-		    d = -sumgrad * steplen * acctol;
-		    fcount++;
-		    crit_ok = !na(f) && (f >= fmax + d);
-#if BFGS_DEBUG
-		    fprintf(stderr, "crit_ok: f=%.10g, fmax=%.10g, d=%g, steplen=%g, ok=%d\n",
-			    f, fmax, d, steplen, crit_ok);
-#endif
-		    if (!crit_ok) {
-			/* calculated criterion no good: try smaller step */
-			steplen *= stepfrac;
-		    }
-		}
-	    } while (ndelta != 0 && !crit_ok);
-
+	    if (quad) {
+		steplen = quad_slen(n, &ndelta, b, X, t, &f, cfunc, data, 
+				    sumgrad, fmax, &fcount);
+	    } else {
+		steplen = simple_slen(n, &ndelta, b, X, t, &f, cfunc, data, 
+				      sumgrad, fmax, &fcount);
+	    }		
 	    done = fabs(fmax - f) <= reltol * (fabs(fmax) + reltol);
 
 #if BFGS_DEBUG
@@ -934,6 +1083,13 @@ static int BFGS_orig (double *b, int n, int maxit, double reltol,
 #endif
 		fmax = f;
 		gradfunc(b, g, n, cfunc, data);
+#if BFGS_DEBUG
+		fprintf(stderr, "new gradient:\n");
+		for (i=0; i<n; i++) {
+		    fprintf(stderr, "%15.6f", g[i]);
+		}
+		fputc('\n', stderr);
+#endif
 		gcount++;
 		iter++;
 		D1 = 0.0;
@@ -981,8 +1137,8 @@ static int BFGS_orig (double *b, int n, int maxit, double reltol,
 	} else {
 	    /* heading in the wrong direction */
 	    if (ilast == gcount) {
-		/* we just reset: don't reset again; set ndelta = 0 so
-		   that we exit the main loop
+		/* we just did a reset, so don't reset again; instead set 
+		   ndelta = 0 so that we exit the main loop
 		*/
 		ndelta = 0;
 		if (gcount == 1) {
@@ -1014,20 +1170,25 @@ static int BFGS_orig (double *b, int n, int maxit, double reltol,
     if (iter >= maxit) {
 	fprintf(stderr, _("stopped after %d iterations\n"), iter);
 	err = E_NOCONV;
-    } else if (gradnorm > gradmax || gradnorm > GRAD_TOLER) {
-	if (gradnorm > gradmax) {
-	    err = E_NOCONV;
-	} else {
-	    gretl_warnmsg_sprintf(_("norm of gradient = %g"), gradnorm);
-	    set_gretl_warning(W_GRADIENT);
-	}
-    } else if (fmax < f0) {
-	/* FIXME this should never happen */
-	fprintf(stderr, "failed to match initial value of objective function:\n"
-		" f0=%.18g, fmax=%.18g\n", f0, fmax);
+    } else if (gradnorm > gradmax) {
 	err = E_NOCONV;
+    } else if (fmax < f0) {
+	/* allow a small sloppiness factor here? */
+	double rdiff;
+
+	rdiff = (f0 == 0.0)? -fmax : fabs((f0 - fmax) / f0);
+	if (rdiff > 1.0e-12) {
+	    fprintf(stderr, "failed to match initial value of objective function:\n"
+		    " f0=%.18g, fmax=%.18g\n", f0, fmax);
+	    err = E_NOCONV;
+	}
     } 
 
+    if (!err && gradnorm > GRAD_TOLER) {
+	gretl_warnmsg_sprintf(_("norm of gradient = %g"), gradnorm);
+	set_gretl_warning(W_GRADIENT);
+    }
+
  skipcalc:
 
     *fncount = fcount;
@@ -1088,7 +1249,7 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
 
     *fncount = *grcount = 0;    
 
-    optim_get_user_values(b, n, &maxit, &reltol, &gradmax, opt, prn);
+    optim_get_user_values(b, n, &maxit, &reltol, &gradmax, NULL, opt, prn);
 
     /*
       m: the number of corrections used in the limited memory matrix.
@@ -1492,7 +1653,7 @@ static int user_gen_setup (umax *u,
 	/* process gradient formula */
 	err = optimizer_get_matrix_name(gradcall, u->gmname);
 	if (!err) {
-	    u->gg = genr_compile(gradcall, dset, OPT_P | OPT_U, &err);
+	    u->gg = genr_compile(gradcall, dset, OPT_P | OPT_O, &err);
 	    if (!err) {
 		err = execute_genr(u->gg, dset, u->prn);
 	    } 
@@ -1503,7 +1664,7 @@ static int user_gen_setup (umax *u,
 	/* process Hessian formula */
 	err = optimizer_get_matrix_name(hesscall, u->hmname);
 	if (!err) {
-	    u->gh = genr_compile(hesscall, dset, OPT_P | OPT_U, &err);
+	    u->gh = genr_compile(hesscall, dset, OPT_P | OPT_O, &err);
 	    if (!err) {
 		err = execute_genr(u->gg, dset, u->prn);
 	    } 
@@ -1762,6 +1923,7 @@ gretl_matrix *fdjac (gretl_matrix *theta, const char *fncall,
     gretl_matrix *J = NULL;
     int m, n;
     int iflag = 0;
+    int quality = 0;
     double *wa = NULL;
     double *fvec = NULL;
     int i;
@@ -1812,7 +1974,9 @@ gretl_matrix *fdjac (gretl_matrix *theta, const char *fncall,
 	fvec[i] = u->fm_out->val[i];
     }
 
-    fdjac2_(user_calc_fvec, m, n, theta->val, fvec, J->val, 
+    quality = libset_get_int(FDJAC_QUAL);
+
+    fdjac2_(user_calc_fvec, m, n, quality, theta->val, fvec, J->val, 
 	    m, &iflag, 0.0, wa, u);
 
  bailout:
@@ -2106,7 +2270,7 @@ int newton_raphson_max (double *b, int n, int maxit,
     }
 
     /* needs some work */
-    optim_get_user_values(b, n, NULL, NULL, NULL, opt, prn);
+    optim_get_user_values(b, n, NULL, NULL, NULL, NULL, opt, prn);
 
     b1 = b0 + n;
     copy_to(b1, b, n);
