@@ -19,8 +19,13 @@
 
 #include "libgretl.h"
 #include "libset.h"
+#include "gretl_func.h"
 #include "gretl_foreign.h"
 
+#ifdef HAVE_MPI
+# include "gretl_mpi.h"
+#endif
+
 #include <glib.h>
 
 #ifdef USE_RLIB
@@ -51,10 +56,13 @@ static gchar *gretl_Rprofile;
 static gchar *gretl_Rsrc;
 static gchar *gretl_Rout;
 static gchar *gretl_Rmsg;
-static gchar *gretl_Oxprog;
+static gchar *gretl_ox_prog;
 static gchar *gretl_octave_prog;
 static gchar *gretl_stata_prog;
 static gchar *gretl_python_prog;
+#ifdef HAVE_MPI
+static gchar *gretl_mpi_prog;
+#endif
 
 static void destroy_foreign (void)
 {
@@ -79,6 +87,18 @@ static int set_foreign_lang (const char *lang, PRN *prn)
 	foreign_lang = LANG_STATA;
     } else if (g_ascii_strcasecmp(lang, "python") == 0) {
 	foreign_lang = LANG_PYTHON;
+    } else if (g_ascii_strcasecmp(lang, "mpi") == 0) {
+#ifdef HAVE_MPI
+	if (gretl_mpi_initialized()) {
+	    gretl_errmsg_set(_("MPI is already initialized"));
+	    err = E_EXTERNAL;
+	} else {
+	    foreign_lang = LANG_MPI;
+	}
+#else
+	gretl_errmsg_set(_("MPI is not supported in this gretl build"));
+	err = E_NOTIMP;
+#endif
     } else {
 	pprintf(prn, "%s: unknown language\n", lang);
 	err = E_DATA;
@@ -87,15 +107,100 @@ static int set_foreign_lang (const char *lang, PRN *prn)
     return err;
 }
 
-static const gchar *gretl_ox_filename (void)
-{
-    if (gretl_Oxprog == NULL) {
-	const char *dotdir = gretl_dotdir();
+#ifdef HAVE_MPI
 
-	gretl_Oxprog = g_strdup_printf("%sgretltmp.ox", dotdir);
+enum {
+    MPI_OPENMPI,
+    MPI_MPICH,
+    MPI_MSMPI
+};
+
+# ifdef G_OS_WIN32
+static int mpi_variant = MPI_MSMPI;
+# else
+static int mpi_variant = MPI_OPENMPI;
+# endif
+
+void set_mpi_variant (const char *pref)
+{
+    if (!strcmp(pref, "OpenMPI")) {
+	mpi_variant = MPI_OPENMPI;
+    } else if (!strcmp(pref, "MPICH")) {
+	mpi_variant = MPI_MPICH;
+    } else if (strcmp(pref, "MS-MPI")) {
+	mpi_variant = MPI_MSMPI;
+    }
+}
+
+int gretl_max_mpi_processes (void)
+{
+    const char *hostfile = gretl_mpi_hosts();
+    int procmax = gretl_n_processors();
+
+    if (hostfile != NULL && *hostfile != '\0') {
+	FILE *fp = gretl_fopen(hostfile, "r");
+
+	if (fp != NULL) {
+	    const char *fmt;
+	    char line[256], host[128];
+	    int nf, slots, allslots = 0;
+	    int err = 0;
+
+	    if (mpi_variant == MPI_MSMPI) {
+		fmt = "%127s %d";
+	    } else if (mpi_variant == MPI_MPICH) {
+		fmt = "%127[^:]:%d";
+	    } else {
+		fmt = "%127s slots=%d";
+	    }
+
+	    while (fgets(line, sizeof line, fp) && !err) {
+		if (*line != '#' && !string_is_blank(line)) {
+		    nf = sscanf(line, fmt, host, &slots);
+		    if (nf == 2) {
+			allslots += slots;
+		    } else {
+			err = E_DATA;
+		    }
+		}
+	    }
+
+	    if (!err && allslots > 0) {
+		procmax = allslots;
+	    }
+
+	    fclose(fp);
+	}
     }
 
-    return gretl_Oxprog;
+    return procmax;
+}
+
+int check_for_mpiexec (void)
+{
+    const char *prog = gretl_mpiexec();
+
+    return check_for_program(prog);
+}
+
+#else
+
+int gretl_max_mpi_processes (void)
+{
+    return 0;
+}
+
+#endif /* HAVE_MPI */
+
+static const gchar *gretl_ox_filename (void)
+{
+    if (gretl_ox_prog == NULL) {
+	const char *dotdir = gretl_dotdir();
+
+	gretl_ox_prog = g_strdup_printf("%sgretltmp.ox", dotdir);
+    }
+
+    return gretl_ox_prog;
 }
 
 static const gchar *gretl_octave_filename (void)
@@ -131,6 +236,21 @@ static const gchar *gretl_python_filename (void)
     return gretl_python_prog;
 }
 
+#ifdef HAVE_MPI
+
+static const gchar *gretl_mpi_filename (void)
+{
+    if (gretl_mpi_prog == NULL) {
+	const char *dotdir = gretl_dotdir();
+
+	gretl_mpi_prog = g_strdup_printf("%sgretltmp-mpi.inp", dotdir);
+    }
+
+    return gretl_mpi_prog;
+}
+
+#endif
+
 static void make_gretl_R_names (void)
 {
     static int done;
@@ -229,6 +349,87 @@ static int lib_run_other_sync (gretlopt opt, PRN *prn)
     return err;
 }
 
+#ifdef HAVE_MPI
+
+/* Windows: for now we'll not attempt to support anything
+   other than "native" MS-MPI
+*/
+
+static int lib_run_mpi_sync (gretlopt opt, PRN *prn)
+{
+    const char *mpiexec = gretl_mpiexec();
+    const char *hostfile = gretl_mpi_hosts();
+    int np = 0;
+    int err = 0;
+
+    if (*hostfile == '\0') {
+	hostfile = getenv("GRETL_MPI_HOSTS");
+    }
+
+    if (opt & OPT_N) {
+	/* handle the number-of-processes option */
+	np = get_optval_int(MPI, OPT_N, &err);
+	if (!err && (np <= 0 || np > 9999999)) {
+	    err = E_DATA;
+	}
+    }
+
+    if (!err) {
+	gchar *hostbit, *npbit, *rngbit, *qopt;
+	gchar *cmd, *sout = NULL;
+
+	if (!(opt & OPT_L) && hostfile != NULL && *hostfile != '\0') {
+	    /* note: OPT_L corresponds to --local, meaning that we
+	       should not use a hosts file even if one is present
+	    */
+	    hostbit = g_strdup_printf(" /machinefile \"%s\"", hostfile);
+	} else {
+	    hostbit = g_strdup("");
+	    if (np == 0) {
+		/* no hosts file: supply a default np value */
+		np = gretl_n_processors();
+	    }
+	}
+
+	if (np > 0) {
+	    npbit = g_strdup_printf(" /np %d", np);
+	} else {
+	    npbit = g_strdup("");
+	}
+
+	if (opt & OPT_S) {
+	    rngbit = g_strdup(" --single-rng");
+	} else {
+	    rngbit = g_strdup("");
+	}
+
+	if (opt & OPT_Q) {
+	    qopt = g_strdup(" --quiet");
+	} else {
+	    qopt = g_strdup("");
+	}
+	
+	cmd = g_strdup_printf("%s%s%s \"%sgretlmpi\"%s%s \"%s\"",
+			      mpiexec, hostbit, npbit, gretl_home(), rngbit,
+			      qopt, gretl_mpi_filename());
+
+	err = gretl_win32_grab_output(cmd, &sout);
+	if (sout != NULL && *sout != '\0') {
+	    pputs(prn, sout);
+	}
+
+	g_free(hostbit);
+	g_free(npbit);
+	g_free(rngbit);
+	g_free(sout);
+	g_free(cmd);
+    }
+
+    return err;
+}
+
+#endif
+
 static char *win32_dotpath (void)
 {
     gchar *dotpath = g_strdup(gretl_dotdir());
@@ -273,10 +474,12 @@ static int lib_run_prog_sync (char **argv, gretlopt opt, PRN *prn)
     } else if (status != 0) {
 	pprintf(prn, "%s exited with status %d", argv[0], status);
 	if (sout != NULL && *sout != '\0') {
+	    pputs(prn, "stdout:\n");
 	    pputs(prn, sout);
 	    pputc(prn, '\n');
 	} 
 	if (errout != NULL && *errout != '\0') {
+	    pputs(prn, "\nstderr:\n");
 	    pputs(prn, errout);
 	    pputc(prn, '\n');
 	}	
@@ -291,6 +494,14 @@ static int lib_run_prog_sync (char **argv, gretlopt opt, PRN *prn)
 		pputc(prn, '\n');
 	    }
 	}
+	if (opt & OPT_V) {
+	    /* also print stderr output, if any */
+	    if (errout != NULL && *errout != '\0') {
+		pputs(prn, "\nstderr:\n");
+		pputs(prn, errout);
+		pputc(prn, '\n');
+	    }
+	}
     } else {
 	pprintf(prn, "%s: %s\n", argv[0], "Got no output");
 	err = 1;
@@ -345,13 +556,121 @@ static int lib_run_other_sync (gretlopt opt, PRN *prn)
 	   of the stata output (gretltmp.log)
 	*/
 	gretl_chdir(gretl_dotdir());
-    }	
+    }
 
     err = lib_run_prog_sync(argv, opt, prn);
 
     return err;
 }
 
+#ifdef HAVE_MPI
+
+#define MPI_DEBUG 0
+
+#if MPI_DEBUG
+static void print_mpi_command (char **argv)
+{
+    int i;
+
+    fputs("gretl/MPI argv array:\n ", stderr);
+    for (i=0; argv[i] != NULL; i++) {
+	fprintf(stderr, "%s ", argv[i]);
+    }
+    fputc('\n', stderr);
+}
+#endif
+
+/* The following should probably be redundant on Linux but may
+   be needed for the OS X package, where the gretl bin
+   directory may not be in PATH.
+*/
+
+static gchar *gretl_mpi_binary (void)
+{
+    gchar *tmp = g_strdup(gretl_home());
+    gchar *p = strstr(tmp, "/share/gretl");
+    gchar *ret;
+
+    if (p != NULL) {
+	*p = '\0';
+	ret = g_strdup_printf("%s/bin/gretlmpi", tmp);
+    } else {
+	ret = g_strdup("gretlmpi");
+    }
+
+    g_free(tmp);
+
+    return ret;
+}
+
+static int lib_run_mpi_sync (gretlopt opt, PRN *prn)
+{
+    const char *hostfile = gretl_mpi_hosts();
+    char npnum[8] = {0};
+    int err = 0;
+
+    if (*hostfile == '\0') {
+	hostfile = getenv("GRETL_MPI_HOSTS");
+    }
+
+    if (opt & OPT_N) {
+	/* handle the number-of-processes option */
+	int np = get_optval_int(MPI, OPT_N, &err);
+
+	if (!err && (np <= 0 || np > 9999999)) {
+	    err = E_DATA;
+	}
+	if (!err) {
+	    sprintf(npnum, "%d", np);
+	}
+    }
+
+    if (!err) {
+	const char *mpiexec = gretl_mpiexec();
+	gchar *mpiprog = gretl_mpi_binary();
+	const char *hostsopt = NULL;
+	char *argv[10];
+	int i = 0;
+
+	if (!(opt & OPT_L) && hostfile != NULL && *hostfile != '\0') {
+	    hostsopt = (mpi_variant == MPI_MPICH)? "-machinefile" :
+		"--hostfile";
+	} else if (*npnum == '\0') {
+	    /* no hosts file: supply a default np value */
+	    sprintf(npnum, "%d", gretl_n_processors());
+	}
+
+	argv[i++] = (char *) mpiexec;
+	if (hostsopt != NULL) {
+	    argv[i++] = (char *) hostsopt;
+	    argv[i++] = (char *) hostfile;
+	}
+	if (*npnum != '\0') {
+	    argv[i++] = "-np";
+	    argv[i++] = npnum;
+	}
+	argv[i++] = mpiprog;
+	if (opt & OPT_S) {
+	    argv[i++] = "--single-rng";
+	}
+	if (opt & OPT_Q) {
+	    argv[i++] = "--quiet";
+	}
+	argv[i++] = (char *) gretl_mpi_filename();
+	argv[i] = NULL;
+
+#if MPI_DEBUG
+	print_mpi_command(argv);
+#endif
+	err = lib_run_prog_sync(argv, opt, prn);
+	g_free(mpiprog);
+    }
+
+    return err;
+}
+
+#endif /* HAVE_MPI */
+
 #endif /* switch on MS Windows or not */
 
 static int write_ox_io_file (void)
@@ -741,6 +1060,75 @@ int write_gretl_python_file (const char *buf, gretlopt opt, const char **pfname)
     return 0;
 }
 
+#ifdef HAVE_MPI
+
+static int mpi_send_funcs_setup (FILE *fp)
+{
+    const char *dotdir = gretl_dotdir();
+    gchar *fname;
+    int err;
+
+    fname = g_strdup_printf("%smpi-funcs-tmp.xml", dotdir);
+    err = write_session_functions_file(fname);
+
+    if (!err) {
+	fprintf(fp, "include \"%s\"\n", fname);
+    }
+
+    g_free(fname);
+
+    return err;
+}
+
+static int write_gretl_mpi_file (gretlopt opt)
+{
+    const gchar *fname = gretl_mpi_filename();
+    FILE *fp = gretl_fopen(fname, "w");
+    int err = 0;
+
+    if (fp == NULL) {
+	return E_FOPEN;
+    }
+
+    if (opt & OPT_F) {
+	/* honor the --send-functions option */
+	if (n_user_functions() > 0) {
+	    err = mpi_send_funcs_setup(fp);
+	}
+    }
+
+    if (!err) {
+	if (opt & OPT_T) {
+	    /* respect the --omp-threads option */
+	    int nt = get_optval_int(MPI, OPT_T, &err);
+
+	    if (nt == -1) {
+		; /* unlimited/auto */
+	    } else {
+		if (!err && (nt <= 0 || nt > 9999999)) {
+		    err = E_DATA;
+		}
+		if (!err) {
+		    fprintf(fp, "set omp_num_threads %d\n", nt);
+		}
+	    }
+	} else {
+	    /* by default, don't use OMP threading */
+	    fputs("set omp_num_threads 1\n", fp);
+	}
+    }
+
+    if (!err) {
+	/* put out the stored 'foreign' lines */
+	put_foreign_lines(fp);
+	fclose(fp);
+    }
+
+    return err;
+}
+
+#endif /* HAVE_MPI */
+
 static int write_data_for_stata (const DATASET *dset,
 				 FILE *fp)
 {
@@ -1174,8 +1562,8 @@ void delete_gretl_R_files (void)
 
 static void delete_gretl_ox_file (void)
 {
-    if (gretl_Oxprog != NULL) {
-	gretl_remove(gretl_Oxprog);
+    if (gretl_ox_prog != NULL) {
+	gretl_remove(gretl_ox_prog);
     }
 }
 
@@ -1200,6 +1588,17 @@ static void delete_gretl_python_file (void)
     }
 }
 
+#ifdef HAVE_MPI
+
+static void delete_gretl_mpi_file (void)
+{
+    if (gretl_mpi_prog != NULL) {
+	gretl_remove(gretl_mpi_prog);
+    }
+}
+
+#endif
+
 /* The following code block is used if we're implementing
    gretl's R support by dlopening the R shared library
    (as opposed to executing the R binary).
@@ -1377,17 +1776,22 @@ void gretl_R_reset_error (void)
 
 static void set_path_for_Rlib (const char *Rhome)
 {
+#ifdef _WIN64
+    const char *arch = "x64";
+#else
+    const char *arch = "i386";
+#endif
     char *path = getenv("PATH");
     gchar *Rpath;
 
-    Rpath = g_strdup_printf("%s\\bin\\i386", Rhome);
+    Rpath = g_strdup_printf("%s\\bin\\%s", Rhome, arch);
     fprintf(stderr, "Rpath = '%s'\n", Rpath);
 
     if (path != NULL && strstr(path, Rpath) != NULL) {
 	; /* nothing to be done */
     } else {
 	g_free(Rpath);
-	Rpath = g_strdup_printf("%s;%s\\bin\\i386", path, Rhome);
+	Rpath = g_strdup_printf("%s;%s\\bin\\%s", path, Rhome, arch);
 	gretl_setenv("PATH", Rpath);
 	g_free(Rpath);
 	Rpath = NULL;
@@ -1398,7 +1802,7 @@ static void set_path_for_Rlib (const char *Rhome)
     }
 }
 
-#endif
+#endif /* WIN32 */
 
 /* Initialize the R library for use with gretl.  Note that we only
    need do this once per gretl session.  We need to check that the
@@ -1806,6 +2210,8 @@ static int foreign_block_init (const char *line, gretlopt opt, PRN *prn)
 	} else {
 	    err = E_PARSE;
 	}
+    } else if (!strncmp(line, "mpi", 3)) {
+	err = set_foreign_lang("mpi", prn);
     } else {
 	/* we'll default to R for now */
 	foreign_lang = LANG_R;
@@ -1907,6 +2313,19 @@ int foreign_execute (const DATASET *dset,
 
     foreign_opt |= opt;
 
+#ifdef HAVE_MPI
+    if (foreign_lang == LANG_MPI) {
+	err = write_gretl_mpi_file(foreign_opt);
+	if (err) {
+	    delete_gretl_mpi_file();
+	} else {
+	    err = lib_run_mpi_sync(foreign_opt, prn);
+	}
+	destroy_foreign();
+	return err;
+    }
+#endif
+
     if (foreign_lang == LANG_R) {
 #ifdef USE_RLIB
 	if (gretl_use_Rlib()) {
@@ -1957,4 +2376,3 @@ int foreign_execute (const DATASET *dset,
 
     return err;
 }
-
