@@ -1995,51 +1995,37 @@ int gretl_list_insert_list_minus (int **targ, const int *src, int pos)
 
 /**
  * list_members_replaced:
- * @list: an array of integer variable ID numbers, the first element
- * of which holds a count of the number of elements following.
+ * @pmod: the model whose list is to be tested.
  * @dset: dataset information.
- * @ref_id: ID number of reference #MODEL.
  *
- * Checks whether any variable in @list has been redefined via
- * gretl's %genr command since a previous model (identified by
- * @ref_id) was estimated.
+ * Checks whether any variable used in @pmod has been redefined
+ * since the model in question was estimated.
  *
- * Returns: 1 if any variables have been replaced, 0 otherwise.
+ * Returns: non-zero if any variables have been replaced, 0 otherwise.
  */
 
-int list_members_replaced (const int *list, const DATASET *dset,
-			   int ref_id)
+int list_members_replaced (const MODEL *pmod, const DATASET *dset)
 {
     const char *errmsg = N_("Can't do this: some vars in original "
 			    "model have been redefined");
-    const char *label;
-    char rword[16];
-    int j, mc, repl;
+    int i, vi;
 
-    if (ref_id == 0) {
-	mc = get_model_count();
-    } else {
-	mc = ref_id;
+    if (pmod->list == NULL) {
+	return 0;
     }
 
-    for (j=1; j<=list[0]; j++) {
-	if (list[j] == LISTSEP) {
+    for (i=1; i<=pmod->list[0]; i++) {
+	vi = pmod->list[i];
+	if (vi == LISTSEP) {
 	    continue;
 	}
-	if (list[j] >= dset->v) {
+	if (vi >= dset->v) {
 	    gretl_errmsg_set(_(errmsg));
 	    return E_DATA;
 	}
-	label = series_get_label(dset, list[j]);
-	*rword = '\0';
-	sscanf(label, "%15s", rword);
-	if (!strcmp(rword, _("Replaced"))) {
-	    repl = 0;
-	    sscanf(label, "%*s %*s %*s %d", &repl);
-	    if (repl >= mc) {
-		gretl_errmsg_set(_(errmsg));
-		return E_DATA;
-	    }
+	if (series_get_mtime(dset, vi) > pmod->esttime) {
+	    gretl_errmsg_set(_(errmsg));
+	    return E_DATA;
 	}
     }
 
@@ -2130,14 +2116,16 @@ int gretl_list_has_separator (const int *list)
 /**
  * gretl_list_split_on_separator:
  * @list: source list.
- * @plist1: pointer to accept first sub-list.
- * @plist2: pointer to accept second sub-list.
+ * @plist1: pointer to accept first sub-list, or NULL.
+ * @plist2: pointer to accept second sub-list, or NULL.
  *
  * If @list contains the list separator, #LISTSEP, creates two
  * sub-lists, one containing the elements of @list preceding
  * the separator and one containing the elements following
  * the separator.  The sub-lists are newly allocated, and assigned
- * as the content of @plist1 and @plist2 respectively.
+ * as the content of @plist1 and @plist2 respectively. Note, however,
+ * that one or other of the sublists can be discarded by passing
+ * NULL as the second or third argument.
  *
  * Returns: 0 on success, %E_ALLOC is memory allocation fails,
  * or %E_DATA if @list does not contain a separator.
@@ -2147,7 +2135,6 @@ int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2)
 {
     int *list1 = NULL, *list2 = NULL;
     int i, n = 0;
-    int err = 0;
 
     for (i=1; i<=list[0]; i++) {
 	if (list[i] == LISTSEP) {
@@ -2157,35 +2144,37 @@ int gretl_list_split_on_separator (const int *list, int **plist1, int **plist2)
     }
 
     if (n == 0) {
-	err = E_PARSE;
+	return E_PARSE;
     }
 
-    if (n > 1) {
-	list1 = gretl_list_new(n - 1);
-	if (list1 == NULL) {
-	    return E_ALLOC;
-	}
-	for (i=1; i<n; i++) {
-	    list1[i] = list[i];
+    if (plist1 != NULL) {
+	if (n > 1) {
+	    list1 = gretl_list_new(n - 1);
+	    if (list1 == NULL) {
+		return E_ALLOC;
+	    }
+	    for (i=1; i<n; i++) {
+		list1[i] = list[i];
+	    }
 	}
+	*plist1 = list1;
     }
 
-    if (n < list[0]) {
-	list2 = gretl_list_new(list[0] - n);
-	if (list2 == NULL) {
-	    free(list1);
-	    return E_ALLOC;
-	}
-
-	for (i=1; i<=list2[0]; i++) {
-	    list2[i] = list[i + n];
+    if (plist2 != NULL) {
+	if (n < list[0]) {
+	    list2 = gretl_list_new(list[0] - n);
+	    if (list2 == NULL) {
+		free(list1);
+		return E_ALLOC;
+	    }
+	    for (i=1; i<=list2[0]; i++) {
+		list2[i] = list[i + n];
+	    }
 	}
+	*plist2 = list2;
     }
-
-    *plist1 = list1;
-    *plist2 = list2;
     
-    return err;
+    return 0;
 }
 
 /**
@@ -2572,24 +2561,20 @@ int *gretl_list_build (const char *s, const DATASET *dset, int *err)
 
 /**
  * gretl_list_print:
- * @lname: name of list.
+ * @list: list to print.
  * @dset: dataset information.
  * @prn: gretl printing struct.
  * 
  * Prints to @prn the given @list of variables, by name.
  */
 
-void gretl_list_print (const char *lname, const DATASET *dset,
+void gretl_list_print (const int *list, const DATASET *dset,
 		       PRN *prn)
 {
-    const int *list = get_list_by_name(lname);
     int testlen = 62;
     int i, li, len = 0;
 
-    if (list == NULL) {
-	pprintf(prn, _("Unknown variable '%s'"), lname);
-	pputc(prn, '\n');
-    } else if (list[0] == 0) {
+    if (list == NULL || list[0] == 0) {
 	pputs(prn, "null\n");
     } else {
 	for (i=1; i<=list[0]; i++) {
