@@ -82,12 +82,16 @@ void gretl_print_destroy (PRN *prn)
 
     fpdup = (prn->fp == prn->fpaux);
 
-    if (prn->fp != NULL && prn->fp != stdout && prn->fp != stderr) {
+    if (prn->fp != NULL) {
+	if (prn->fp == stdout) {
+	    fflush(stdout);
+	} else if (prn->fp != stderr) {
 #if PRN_DEBUG
-  	fprintf(stderr, "gretl_print_destroy: prn=%p, closing fp at %p\n", 
-		(void *) prn, (void *) prn->fp); 
+	    fprintf(stderr, "gretl_print_destroy: prn=%p, closing fp at %p\n", 
+		    (void *) prn, (void *) prn->fp); 
 #endif
-	fclose(prn->fp);
+	    fclose(prn->fp);
+	}
     }
 
     if (prn->fname != NULL) {
@@ -170,7 +174,7 @@ static PRN *real_gretl_print_new (PrnType ptype,
     if (ptype == GRETL_PRINT_STREAM) {
 	prn->fp = fp;
     } else if (ptype == GRETL_PRINT_FILE) {
-	prn->fp = gretl_fopen(fname, "w");
+	prn->fp = gretl_fopen(fname, "wb");
 	if (prn->fp == NULL) {
 	    err = E_FOPEN;
 	    free(prn);
@@ -670,10 +674,10 @@ int gretl_print_set_save_position (PRN *prn)
 {
     if (prn == NULL || prn->buf == NULL) {
 	return E_DATA;
+    } else {
+	prn->savepos = prn->blen;
+	return 0;
     }
-
-    prn->savepos = strlen(prn->buf);
-    return 0;
 }
 
 /**
@@ -704,10 +708,11 @@ void gretl_print_unset_save_position (PRN *prn)
 
 char *gretl_print_get_chunk (PRN *prn)
 {
+    int maxpos = prn->blen; /* was strlen(prn->buf) */
     char *buf;
 
     if (prn == NULL || prn->buf == NULL ||
-	prn->savepos < 0 || prn->savepos > strlen(prn->buf)) {
+	prn->savepos < 0 || prn->savepos > maxpos) {
 	return NULL;
     }
 
@@ -717,6 +722,19 @@ char *gretl_print_get_chunk (PRN *prn)
     return buf;
 }
 
+static void set_default_csv_delim (PRN *prn)
+{
+    char test[4];
+
+    sprintf(test, "%.1f", 1.0);
+
+    if (test[1] == ',') {
+	prn->delim = ';';
+    } else {
+	prn->delim = ',';
+    }
+}
+
 /**
  * gretl_print_set_format:
  * @prn: printing struct.
@@ -731,6 +749,9 @@ void gretl_print_set_format (PRN *prn, PrnFormat format)
 	if (format == GRETL_FORMAT_RTF) {
 	    format = GRETL_FORMAT_RTF | GRETL_FORMAT_DOC;
 	}
+	if (format == GRETL_FORMAT_CSV) {
+	    set_default_csv_delim(prn);
+	}
 	prn->format = format;
     }
 }
@@ -854,7 +875,6 @@ static int pprintf_init (PRN *prn)
     return ret;
 }
 
-
 /**
  * pprintf:
  * @prn: gretl printing struct.
