@@ -29,17 +29,24 @@
 #include "gretl_xml.h"
 #include "forecast.h"
 #include "kalman.h"
-#include "gretl_www.h"
+#ifdef USE_CURL
+# include "gretl_www.h"
+#endif
+
+#ifdef _OPENMP
+# include <omp.h>
+#endif
+
+#ifdef HAVE_MPI
+# include "gretl_mpi.h"
+#endif
 
 #ifdef USE_RLIB
 # include "gretl_foreign.h"
 #endif
 
 #include <errno.h>
-
-#ifndef WIN32
-# include <glib.h>
-#endif
+#include <gmp.h>
 
 static void gretl_tests_cleanup (void);
 
@@ -1147,7 +1154,7 @@ int set_obs (const char *line, DATASET *dset, gretlopt opt)
 	} else {
 	    err = switch_panel_orientation(dset);
 	}
-    }	
+    }
 
     return err;
 }
@@ -1721,57 +1728,46 @@ int gretl_delete_var_by_name (const char *s, PRN *prn)
     return err;
 }
 
-/* internal execution timer: on OpenMP use omp_get_wtime(); else
-   on Windows use use GetTickCount; else use times() if available,
-   otherwise fall back on clock()
-*/ 
+/* internal execution timer (we do Windows separately) */
 
-#if defined(_OPENMP) && defined(USE_OPENMP)
-
-#include <omp.h>
-
-static double tim0;
+#ifdef WIN32
 
 static void gretl_stopwatch_init (void)
 {
-    tim0 = omp_get_wtime();
+    win32_stopwatch_init();
 }
 
 double gretl_stopwatch (void)
 {
-    double tim1 = omp_get_wtime();
-    double x = tim1 - tim0;
-
-    tim0 = tim1;
-
-    return x;
-} 
-
-#elif defined(WIN32)
-
-#include <windows.h>
-
-static DWORD tim0;
-
-static void gretl_stopwatch_init (void)
-{
-    tim0 = GetTickCount();
+    return win32_stopwatch();
 }
 
-double gretl_stopwatch (void)
-{
-    DWORD tim1 = GetTickCount();
-    double x;
+#elif defined(_OPENMP)
 
-    x = (double) (tim1 - tim0) / 1000.0;
-    tim0 = tim1;
+/* note: on Windows omp_get_wtime() has only
+   millisecond resolution; fancy footwork is
+   required to get anything better */
+
+static double omp_dt0;
+
+static void gretl_omp_stopwatch_init (void)
+{
+    omp_dt0 = omp_get_wtime();
+}
+
+static double gretl_omp_stopwatch (void)
+{
+    double dt1 = omp_get_wtime();
+    double x = dt1 - omp_dt0;
+
+    omp_dt0 = dt1;
 
     return x;
-} 
+}
 
-#else /* !OPENMP, !WIN32 */
+#else /* non-Windows, without OMP */
 
-static clock_t tim0;
+static clock_t ut0;
 
 #ifdef HAVE_SYS_TIMES_H
 # include <sys/times.h>
@@ -1781,40 +1777,73 @@ static unsigned ticks_per_sec;
 # include <time.h>
 #endif
 
-static void gretl_stopwatch_init (void)
+static void gretl_unix_stopwatch_init (void)
 {
 #ifdef HAVE_SYS_TIMES_H
     struct tms timebuf;
 
     ticks_per_sec = sysconf(_SC_CLK_TCK);
-    tim0 = times(&timebuf);
+    ut0 = times(&timebuf);
 #else
-    tim0 = clock();
+    ut0 = clock();
 #endif
 }
 
-double gretl_stopwatch (void)
+static double gretl_unix_stopwatch (void)
 {
-    clock_t tim1;
+    clock_t ut1;
     double x;
 
 #ifdef HAVE_SYS_TIMES_H
     struct tms timebuf;
 
-    tim1 = times(&timebuf);
-    x = (double) (tim1 - tim0) / ticks_per_sec;
+    ut1 = times(&timebuf);
+    x = (double) (ut1 - ut0) / ticks_per_sec;
 #else
-    tim1 = clock();
-    x = (double) (tim1 - tim0) / CLOCKS_PER_SEC;
+    ut1 = clock();
+    x = (double) (ut1 - ut0) / CLOCKS_PER_SEC;
 #endif
 
-    tim0 = tim1;
+    ut0 = ut1;
 
     return x;
-} 
+}
 
 #endif /* stopwatch variants */
 
+#ifndef WIN32
+
+static void gretl_stopwatch_init (void)
+{
+#if defined(HAVE_MPI)
+    if (gretl_mpi_initialized()) {
+	gretl_mpi_stopwatch_init();
+	return;
+    }
+#endif
+#if defined(_OPENMP)
+    gretl_omp_stopwatch_init();
+#else
+    gretl_unix_stopwatch_init();
+#endif    
+}
+
+double gretl_stopwatch (void)
+{
+#if defined(HAVE_MPI)
+    if (gretl_mpi_initialized()) {
+	return gretl_mpi_stopwatch();
+    }
+#endif
+#if defined(_OPENMP)
+    return gretl_omp_stopwatch();
+#else
+    return gretl_unix_stopwatch();
+#endif 
+}
+
+#endif /* !Windows */
+
 /* library init and cleanup functions */
 
 /**
@@ -1822,25 +1851,75 @@ double gretl_stopwatch (void)
  *
  * In a program that uses libgretl, this function should be 
  * called once, before any other libgretl functions are
- * used. See also libgretl_cleanup().
+ * used. See also libgretl_cleanup(), and libgretl_mpi_init().
  **/
 
 void libgretl_init (void)
 {
     libset_init();
-
-#ifdef WIN32
-    fprintf(stderr, "libgretl_init: initializing RNG\n");
-#endif
     gretl_rand_init();
-#ifdef WIN32
-    fprintf(stderr, " RNG ready\n");
-#endif
+    gretl_xml_init();
+    gretl_stopwatch_init();
+    mpf_set_default_prec(get_mp_bits());
+}
+
+#ifdef HAVE_MPI
+
+/**
+ * libgretl_mpi_init:
+ * @self: the MPI rank of the calling process.
+ * @np: the number of MPI processes.
+ * @dcmt: if non-zero, set up per-process RNG using DCMT.
+ *
+ * This function provides an alternative to libgretl_init()
+ * which should be used when a libgretl program is to be run in 
+ * MPI mode.
+ **/
+
+int libgretl_mpi_init (int self, int np, int dcmt)
+{
+    int err;
+
+    libset_init();
+
+    /* let geneval know */
+    set_mpi_rank_and_size(self, np);
+
+    /* load MPI library functions */
+    err = gretl_MPI_init();
+    if (err) {
+	return err;
+    }
+
+    if (dcmt && np > 1) {
+	/* use DCMT for multiple RNGs */
+	gretl_dcmt_init(np, self, 4172);
+    } else {
+	/* just use one RNG */
+	gretl_rand_init();
+    }
 
     gretl_xml_init();
     gretl_stopwatch_init();
+    mpf_set_default_prec(get_mp_bits());
+
+    /* be relatively quiet by default */
+    set_gretl_echo(0);
+    set_gretl_messages(0);
+
+    return 0;
 }
 
+#else
+
+int gretl_mpi_initialized (void)
+{
+    /* stub for non-MPI build */
+    return 0;
+}
+
+#endif
+
 void libgretl_session_cleanup (int mode)
 {
     gretl_saved_objects_cleanup();
@@ -1850,6 +1929,7 @@ void libgretl_session_cleanup (int mode)
     gretl_lists_cleanup();
     gretl_tests_cleanup();
     gretl_plotx(NULL, OPT_NONE);
+    setopt_cleanup();
 
     if (mode != SESSION_CLEAR_DATASET) {
 	destroy_user_vars();
@@ -1878,11 +1958,16 @@ void libgretl_cleanup (void)
     gretl_function_hash_cleanup();
     lapack_mem_free();
     forecast_matrix_cleanup();
-    option_flags_cleanup();
+    option_params_cleanup();
+    option_printing_cleanup();
     kalman_cleanup();
     gnuplot_cleanup();
     bufgets_cleanup();
+    plugins_cleanup();
+    gretl_bundle_cleanup();
+#ifdef USE_CURL
     gretl_www_cleanup();
+#endif
     builtin_strings_cleanup();
 
 #ifdef USE_RLIB
@@ -2113,3 +2198,301 @@ void gretl_aligned_free (void *mem)
 	free(ORIG_PTR(mem));
     }
 }
+
+#ifdef WIN32
+
+int check_for_program (const char *prog)
+{
+    int ret = 0;
+
+    if (prog == NULL || *prog == '\0') {
+	return 0;
+    }
+
+    if (has_suffix(prog, ".exe")) {
+	ret = win32_check_for_program(prog);
+    } else {
+	gchar *test = g_strdup_printf("%s.exe", prog);
+
+	ret = win32_check_for_program(test);
+	g_free(test);
+    }
+
+    return ret;
+}
+
+#else /* !WIN32 */
+
+#include <sys/stat.h>
+
+static int is_executable (const char *s, uid_t myid, gid_t mygrp)
+{
+    struct stat buf;
+    int ok = 0;
+
+    if (gretl_stat(s, &buf) != 0) {
+	return 0;
+    }
+
+    if (buf.st_mode & (S_IFREG|S_IFLNK)) {
+	if (buf.st_uid == myid && (buf.st_mode & S_IXUSR)) {
+	    ok = 1;
+	} else if (buf.st_gid == mygrp && (buf.st_mode & S_IXGRP)) {
+	    ok = 1;
+	} else if (buf.st_uid != myid && buf.st_gid != mygrp &&
+		   (buf.st_mode & S_IXOTH)) {
+	    ok = 1;
+	}
+    }
+
+    return ok;
+}
+
+/**
+ * check_for_program:
+ * @prog: name of program.
+ * 
+ * Returns: 1 if @prog is found in the PATH, otherwise 0.
+ */
+
+int check_for_program (const char *prog)
+{
+    uid_t myid = getuid();
+    gid_t mygrp = getgid();
+    char *path;
+    char *pathcpy;
+    char **dirs;
+    char *fullpath;
+    char *p;
+    int max_dlen = 0;
+    int found = 0;
+    int i, ndirs;
+
+    if (prog == NULL || *prog == '\0') {
+	return 0;
+    }
+
+    if (*prog == '/') {
+	return is_executable(prog, myid, mygrp);
+    }
+
+    path = getenv("PATH");
+    if (path == NULL || *path == '\0') {
+	return 0;
+    }
+
+    pathcpy = gretl_strdup(path);
+    if (pathcpy == NULL) {
+	return 0;
+    }
+
+    ndirs = 1;
+    p = pathcpy;
+    while (*p) {
+	if (*p == ':') ndirs++;
+	p++;
+    }
+
+    dirs = malloc(ndirs * sizeof *dirs);
+    if (dirs == NULL) {
+	free(pathcpy);
+	return 0;
+    }
+
+    if (ndirs == 1) {
+	dirs[0] = pathcpy;
+	max_dlen = strlen(pathcpy);
+    } else {
+	for (i=0; i<ndirs; i++) {
+	    int dlen;
+
+	    dirs[i] = strtok((i == 0)? pathcpy : NULL, ":");
+	    if (dirs[i] == NULL) {
+		ndirs = i;
+		break;
+	    }
+	    dlen = strlen(dirs[i]);
+	    if (dlen > max_dlen) {
+		max_dlen = dlen;
+	    }
+	}
+    }
+
+    if (ndirs == 0 || 
+	(fullpath = malloc(max_dlen + strlen(prog) + 2)) == NULL) {
+	free(dirs);
+	free(pathcpy);
+	return 0;
+    }
+
+    for (i=0; i<ndirs && !found; i++) { 
+	sprintf(fullpath, "%s/%s", dirs[i], prog);
+	found = is_executable(fullpath, myid, mygrp);
+    }
+
+    free(dirs);
+    free(pathcpy);
+    free(fullpath);
+
+    return found;
+}
+
+#endif /* WIN32 or not */
+
+enum {
+    BLAS_UNKNOWN,
+    BLAS_NETLIB,
+    BLAS_ATLAS,
+    BLAS_OPENBLAS,
+    BLAS_MKL,
+    BLAS_VECLIB
+};
+
+#if !defined(WIN32) && !defined(OS_OSX)
+
+static int real_detect_blas (const char *s)
+{
+    char found[5] = "nnnn";
+    char line[512];
+    int i = 0;
+    int ret = BLAS_UNKNOWN;
+
+    *line = '\0';
+
+    while (*s) {
+	if (*s == '\n') {
+	    line[i] = '\0';
+	    if (strstr(line, "libmkl")) {
+		found[3] = 'y';
+	    } else if (strstr(line, "openblas") ||
+		strstr(line, "OpenBLAS")) {
+		found[2] = 'y';
+	    } else if (strstr(line, "atlas")) {
+		found[1] = 'y';
+	    } else if (strstr(line, "libblas")) {
+		found[0] = 'y';
+	    }
+	    *line = '\0';
+	    i = 0;
+	} else {
+	    line[i++] = *s;
+	}
+	s++;
+    }
+
+    fputs("detect blas: ", stderr);
+
+    if (strcmp(found, "nnny") == 0) {
+	fputs("MKL\n", stderr);
+	ret = BLAS_MKL;
+    } else if (strcmp(found, "nnyn") == 0) {
+	fputs("OpenBLAS\n", stderr);
+	ret = BLAS_OPENBLAS;
+    } else if (strcmp(found, "nynn") == 0) {
+	fputs("ATLAS\n", stderr);
+	ret = BLAS_ATLAS;
+    } else if (strcmp(found, "ynnn") == 0) {
+	fputs("Netlib\n", stderr);
+	ret = BLAS_NETLIB;
+    } else if (strcmp(found, "nnnn") == 0) {
+	fputs("found no relevant libs!\n", stderr);
+    } else {
+	fputs("confused, found too many blas libs!\n", stderr);
+    }
+
+    return ret;
+}
+
+static int detect_blas_variant (void)
+{
+    gchar *argv[3];
+    gchar *prog;
+    gchar *sout = NULL;
+    gchar *errout = NULL;
+    gint status = 0;
+    GError *gerr = NULL;
+    int variant = 0;
+
+    prog = g_strdup(GRETL_PREFIX "/bin/gretlcli");
+
+    argv[0] = "ldd";
+    argv[1] = prog;
+    argv[2] = NULL;
+
+    g_spawn_sync(NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
+		 NULL, NULL, &sout, &errout,
+		 &status, &gerr);
+
+    if (gerr != NULL) {
+	fprintf(stderr, "%s\n", gerr->message); 
+	g_error_free(gerr);
+    } else if (status != 0) {
+	fprintf(stderr, "%s exited with status %d\n", argv[0], status);
+    } else if (sout != NULL) {
+	variant = real_detect_blas(sout);
+    } else {
+	fprintf(stderr, "%s: %s\n", argv[0], "Got no output");
+    }
+
+    g_free(sout);
+    g_free(errout);
+    g_free(prog);
+
+    return variant;
+}
+
+#endif /* not Windows or Mac */
+
+static int blas_variant_code (void)
+{
+#if defined(WIN32)
+# ifdef OPENBLAS_BUILD
+    return BLAS_OPENBLAS;
+# else
+    return BLAS_NETLIB;
+# endif
+#elif defined(OS_OSX)
+    return BLAS_VECLIB;
+#else
+    /* not a packaged build: we have to check */
+    static int bver = -1;
+
+    if (bver < 0) {
+	bver = detect_blas_variant();
+    }
+
+    return bver;
+#endif
+}
+
+const char *blas_variant_string (void)
+{
+    int bver = blas_variant_code();
+
+    if (bver == BLAS_NETLIB) {
+	return "netlib";
+    } else if (bver == BLAS_ATLAS) {
+	return "atlas";
+    } else if (bver == BLAS_OPENBLAS) {
+	return "openblas";
+    } else if (bver == BLAS_MKL) {
+	return "mkl";
+    } else if (bver == BLAS_VECLIB) {
+	return "veclib";
+    } else {
+	return "unknown";
+    }
+}
+
+#if GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 28 && defined(OS_OSX)
+# include <mach/mach_time.h>
+#endif
+
+gint64 gretl_monotonic_time (void)
+{
+#if GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 28 && defined(OS_OSX)
+    return (gint64) mach_absolute_time();
+#else
+    return g_get_monotonic_time();
+#endif
+}
