@@ -40,6 +40,7 @@
 #define SUBST_DEBUG 0
 
 #include <gmp.h>
+
 typedef mpf_t bigval;
 
 enum loop_types {
@@ -1225,6 +1226,7 @@ static int parse_first_loopline (char *s, LOOPSET *loop,
  * @s: loop specification line.
  * @inloop: current loop struct pointer, or %NULL.
  * @dset: dataset struct.
+ * @opt: options associated with new loop.
  * @nested: location to receive info on whether a new
  * loop was created, nested within the input loop.
  * @err: location to receive error code.
@@ -1237,7 +1239,9 @@ static int parse_first_loopline (char *s, LOOPSET *loop,
 
 static LOOPSET *start_new_loop (char *s, LOOPSET *inloop, 
 				DATASET *dset,
-				int *nested, int *err)
+				gretlopt opt,
+				int *nested,
+				int *err)
 {
     LOOPSET *loop = NULL;
 
@@ -1248,6 +1252,13 @@ static LOOPSET *start_new_loop (char *s, LOOPSET *inloop,
 	    (void *) inloop, s);
 #endif
 
+#if 0
+    if (inloop != NULL && (opt & OPT_P)) {
+	/* don't allow nesting of "progressive" loops? */
+	fprintf(stderr, "inner loop with OPT_P\n");
+    }
+#endif
+
     if (inloop == NULL || compile_level <= inloop->level) {
 	loop = gretl_loop_new(NULL);
     } else {
@@ -1284,7 +1295,7 @@ static LOOPSET *start_new_loop (char *s, LOOPSET *inloop,
 #if LOOP_DEBUG
 # define MAX_FOR_TIMES  10
 #else
-# define MAX_FOR_TIMES  500000
+# define MAX_FOR_TIMES  50000000
 #endif
 
 static int loop_count_too_high (LOOPSET *loop)
@@ -1300,9 +1311,11 @@ static int loop_count_too_high (LOOPSET *loop)
     } else {
 	int maxit = libset_get_int(LOOP_MAXITER);
 
-	if ( (maxit > 0) && (nt > maxit) ) {
+	if (maxit > 0 && nt > maxit) {
 	    gretl_errmsg_sprintf(_("Reached maximum iterations, %d"),
 				 maxit);
+	    gretl_errmsg_append(_("You can use \"set loop_maxiter\" "
+				  "to increase the limit"));
 	    loop->err = 1;
 	}
     }
@@ -1457,7 +1470,7 @@ static void loop_model_init (LOOP_MODEL *lmod, int lno)
     lmod->cdiff = NULL;
 }
 
-/* Start up a LOOP_MODEL struct: copy pmod into place and
+/* Start up a LOOP_MODEL struct: copy @pmod into place and
    allocate storage */
 
 static int loop_model_start (LOOP_MODEL *lmod, const MODEL *pmod)
@@ -1916,6 +1929,9 @@ static int loop_model_update (LOOP_MODEL *lmod, MODEL *pmod)
 	if (err) {
 	    return err;
 	}
+    } else if (pmod->ncoeff != lmod->nc) {
+	gretl_errmsg_set(_("progressive loop: model must be of constant size"));
+	return E_DATA;
     }
 
     mpf_init(m);
@@ -1951,8 +1967,8 @@ static int loop_model_update (LOOP_MODEL *lmod, MODEL *pmod)
     return err;
 }
 
-/* Update the LOOP_PRINT struct lprn using the current values from the
-   variables in list.  If this is the first use we need to do some
+/* Update the LOOP_PRINT struct @lprn using the current values of the
+   specified variables. If this is the first use we need to do some
    allocation first.
 */
 
@@ -2117,6 +2133,7 @@ int gretl_loop_append_line (ExecState *s, DATASET *dset)
     LOOPSET *newloop = currloop;
     int err = 0;
 
+    warnmsg(s->prn); /* catch "end loop" if present */
     gretl_error_clear();
 
 #if LOOP_DEBUG > 1
@@ -2138,7 +2155,7 @@ int gretl_loop_append_line (ExecState *s, DATASET *dset)
 
 	if (!err) {
 	    newloop = start_new_loop(s->line, loop, dset, 
-				     &nested, &err);
+				     opt, &nested, &err);
 #if LOOP_DEBUG
 	    fprintf(stderr, "got LOOP: newloop at %p (err = %d)\n", 
 		    (void *) newloop, err);
@@ -2906,10 +2923,6 @@ static int conditional_line (LOOPSET *loop, int j)
     return loop->cmds[j].ci == IF || loop->cmds[j].ci == ELIF;
 }
 
-#define COMPILE_IF 1 /* should be OK now? */
-
-#if COMPILE_IF 
-
 static int do_compile_conditional (LOOPSET *loop, int j)
 {
     if ((loop->cmds[j].ci == IF || loop->cmds[j].ci == ELIF) &&
@@ -2920,8 +2933,6 @@ static int do_compile_conditional (LOOPSET *loop, int j)
     }
 }
 
-#endif	
-
 static int block_model (CMD *cmd)
 {
     return cmd->ci == END && 
@@ -3038,7 +3049,6 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 	       for "if" or "elif" conditions that may be already
 	       compiled, or that should now be compiled
 	    */
-#if COMPILE_IF
 	    if (conditional_compiled(loop, j)) {
 		err = parse_command_line(line, cmd, dset, &loop->cmds[j].genr);
 	    } else if (do_compile_conditional(loop, j)) {
@@ -3052,9 +3062,6 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 	    } else {
 		err = parse_command_line(line, cmd, dset, NULL);
 	    }
-#else
-	    err = parse_command_line(line, cmd, dset, NULL);
-#endif
 
 #if LOOP_DEBUG
 	    fprintf(stderr, "    after: '%s'\n", line);
@@ -3153,10 +3160,11 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 				     cmd->word);
 		err = 1;
 	    } else if (cmd->ci == GENR) {
-		if (subst || (cmd->opt & OPT_U)) {
+		if (subst || (cmd->opt & OPT_O)) {
 		    /* can't use a "compiled" genr if string substitution
 		       has been done, since the genr expression will not
-		       be constant */
+		       be constant 
+		    */
 		    if (!loop_is_verbose(loop)) {
 			cmd->opt |= OPT_Q;
 		    }
