@@ -19,6 +19,7 @@
 
 #include "libgretl.h"
 #include "uservar.h"
+#include "gretl_func.h"
 
 #include <errno.h>
 
@@ -98,6 +99,7 @@ struct gretl_option gretl_opts[] = {
     { AR1,      OPT_B, "no-corc", 0 },
     { AR1,      OPT_H, "hilu", 0 },
     { AR1,      OPT_P, "pwe", 0 },
+    { AR1,      OPT_L, "loose", 0 },
     { APPEND,   OPT_A, "all-cols", 0 },
     { APPEND,   OPT_T, "time-series", 0 },
     { APPEND,   OPT_R, "rowoffset", 2 },
@@ -106,6 +108,7 @@ struct gretl_option gretl_opts[] = {
     { APPEND,   OPT_M, "rowmask", 2 },
     { APPEND,   OPT_L, "cols", 2 },
     { APPEND,   OPT_S, "sheet", 2 },
+    { APPEND,   OPT_V, "verbose", 0 },
     { ARBOND,   OPT_A, "asymptotic", 0 },
     { ARBOND,   OPT_D, "time-dummies", 1 },
     { ARBOND,   OPT_H, "orthdev", 0 },
@@ -132,12 +135,13 @@ struct gretl_option gretl_opts[] = {
     { BXPLOT,   OPT_Z, "factorized", 0 },
     { CHOW,     OPT_D, "dummy", 0 },
     { CLEAR,    OPT_D, "dataset", 0 },
-    { COINT,    OPT_E, "test-down", 0 },
+    { COINT,    OPT_E, "test-down", 1 },
     { COINT,    OPT_N, "nc", 0 },
     { COINT,    OPT_R, "ctt", 0 },     
     { COINT,    OPT_S, "skip-df", 0 },
     { COINT,    OPT_T, "ct", 0 },
-    { COINT,    OPT_V, "verbose", 0 },    
+    { COINT,    OPT_V, "verbose", 0 },
+    { COINT,    OPT_I, "silent", 0 },
     { COINT2,   OPT_A, "crt", 0 },
     { COINT2,   OPT_D, "seasonals", 0 },
     { COINT2,   OPT_N, "nc", 0 },
@@ -185,6 +189,7 @@ struct gretl_option gretl_opts[] = {
     { EQNPRINT, OPT_O, "complete", 0 },
     { EQNPRINT, OPT_T, "t-ratios", 0 },
     { TABPRINT, OPT_O, "complete", 0 },
+    { TABPRINT, OPT_C, "csv", 0 },
     { TABPRINT, OPT_R, "rtf", 0 },
     { TABPRINT, OPT_T, "format", 2 },
     { EQUATION, OPT_M, "multi", 0 },
@@ -298,6 +303,14 @@ struct gretl_option gretl_opts[] = {
     { MODTEST,  OPT_R, "robust", 0 },
     { MODTEST,  OPT_W, "white", 0 },
     { MODTEST,  OPT_X, "white-nocross", 0 },
+    { MODTEST,  OPT_I, "silent", 0 },
+    { MPI,      OPT_F, "send-functions", 0 },
+    { MPI,      OPT_L, "local", 0 },
+    { MPI,      OPT_N, "np", 2},
+    { MPI,      OPT_T, "omp-threads", 2},
+    { MPI,      OPT_Q, "quiet", 0},
+    { MPI,      OPT_V, "verbose", 0},
+    { MPI,      OPT_S, "single-rng", 0},
     { LABELS,   OPT_D, "delete", 0 },
     { LABELS,   OPT_F, "from-file", 2 },
     { LABELS,   OPT_T, "to-file", 2 },
@@ -372,6 +385,7 @@ struct gretl_option gretl_opts[] = {
     { OPEN,     OPT_W, "www", 0 },
     { OPEN,     OPT_L, "cols", 2 },
     { OPEN,     OPT_M, "rowmask", 2 },
+    { OPEN,     OPT_V, "verbose", 0 },
     { OUTFILE,  OPT_A, "append", 0 },
     { OUTFILE,  OPT_C, "close", 0 },
     { OUTFILE,  OPT_W, "write", 0 },
@@ -454,6 +468,7 @@ struct gretl_option gretl_opts[] = {
     { SETOBS,   OPT_N, "special-time-series", 0 },
     { SETOBS,   OPT_G, "panel-groups", 0 },
     { SETOBS,   OPT_I, "panel-time", 0 },
+    { SMPL,     OPT_A, "no-all-missing", 0 },
     { SMPL,     OPT_B, "balanced", 0 },
     { SMPL,     OPT_C, "contiguous", 0 },
     { SMPL,     OPT_F, "full", 0 },
@@ -462,6 +477,7 @@ struct gretl_option gretl_opts[] = {
     { SMPL,     OPT_N, "random", 0 },
     { SMPL,     OPT_P, "replace", 0 }, 
     { SMPL,     OPT_R, "restrict", 0 },
+    { SMPL,     OPT_T, "permanent", 0 },
     { SPEARMAN, OPT_V, "verbose", 0 },
     { SQUARE,   OPT_O, "cross", 0 },
     { STORE,    OPT_C, "csv", 0 },
@@ -472,13 +488,13 @@ struct gretl_option gretl_opts[] = {
     { STORE,    OPT_M, "gnu-octave", 0 },
     { STORE,    OPT_N, "no-header", 0 },
     { STORE,    OPT_R, "gnu-R", 0 },
-    { STORE,    OPT_T, "traditional", 0 },
-    { STORE,    OPT_Z, "gzipped", 0 },
+    { STORE,    OPT_Z, "gzipped", 1 },
     { STORE,    OPT_X, "omit-obs", 0 },
     { STORE,    OPT_E, "comment", 2 },
     { STORE,    OPT_I, "decimal-comma", 0 },
     { SUMMARY,  OPT_B, "by", 2 },
     { SUMMARY,  OPT_S, "simple", 0 },
+    { SUMMARY,  OPT_W, "weights", 2 },
     { SUMMARY,  OPT_X, "matrix", 2 },
     { SYSTEM,   OPT_I, "iterate", 0 },
     { SYSTEM,   OPT_V, "verbose", 0 },
@@ -518,9 +534,27 @@ struct gretl_option gretl_opts[] = {
     { 0,        0L,    NULL, 0 }
 };
 
+static const char *get_longopt (int ci, gretlopt opt)
+{
+    int i, got_ci = 0;
+
+    for (i=0; gretl_opts[i].ci; i++) {
+	if (gretl_opts[i].ci == ci) {
+	    if (gretl_opts[i].o == opt) {
+		return gretl_opts[i].longopt;
+	    }
+	    got_ci = 1;
+	} else if (got_ci) {
+	    break;
+	}
+    }
+
+    return "??";
+}
+
 int cluster_option_ok (int ci)
 {
-    int i, found = 0;
+    int i, got_ci = 0;
 
     for (i=0; gretl_opts[i].ci; i++) {
 	if (gretl_opts[i].ci == ci) {
@@ -528,8 +562,8 @@ int cluster_option_ok (int ci)
 		!strcmp(gretl_opts[i].longopt, "cluster")) {
 		return 1;
 	    }
-	    found = 1;
-	} else if (found) {
+	    got_ci = 1;
+	} else if (got_ci) {
 	    break;
 	}
     }
@@ -540,7 +574,7 @@ int cluster_option_ok (int ci)
 int matrix_data_option (int ci, gretlopt opt)
 {
     if (opt & OPT_X) {
-	int i, found = 0;
+	int i, got_ci = 0;
 
 	for (i=0; gretl_opts[i].ci; i++) {
 	    if (gretl_opts[i].ci == ci) {
@@ -548,8 +582,8 @@ int matrix_data_option (int ci, gretlopt opt)
 		    !strcmp(gretl_opts[i].longopt, "matrix")) {
 		    return 1;
 		}
-		found = 1;
-	    } else if (found) {
+		got_ci = 1;
+	    } else if (got_ci) {
 		break;
 	    }
 	}
@@ -593,7 +627,7 @@ char **get_all_option_strings (int *pn)
 
 const char **get_opts_for_command (int ci, int *nopt)
 {
-    int i, j, n = 0;
+    int i, j, got_ci, n = 0;
     const char **ret = NULL;
 
     if (ci != OLS) {
@@ -614,10 +648,13 @@ const char **get_opts_for_command (int ci, int *nopt)
     ret = malloc(n * sizeof *ret);
     if (ret == NULL) return NULL;
 
-    j = 0;
+    j = got_ci = 0;
     for (i=0; gretl_opts[i].ci != 0; i++) {
 	if (gretl_opts[i].ci == ci) {
 	    ret[j++] = gretl_opts[i].longopt;
+	    got_ci = 1;
+	} else if (got_ci) {
+	    break;
 	}
     }
 
@@ -692,19 +729,22 @@ gretlopt opt_from_flag (unsigned char c)
 
 static int opt_is_valid (gretlopt opt, int ci, char c)
 {
-    int i;
+    int i, got_ci = 0;
 
     if (opt == OPT_O && vcv_opt_ok(ci)) {
 	return 1;
-    }
-
-    if (opt == OPT_Q && quiet_opt_ok(ci)) {
+    } else if (opt == OPT_Q && quiet_opt_ok(ci)) {
 	return 1;
     }    
 
     for (i=0; gretl_opts[i].ci != 0; i++) {
-	if (ci == gretl_opts[i].ci && opt == gretl_opts[i].o) {
-	    return 1;
+	if (ci == gretl_opts[i].ci) {
+	    if (opt == gretl_opts[i].o) {
+		return 1;
+	    }
+	    got_ci = 1;
+	} else if (got_ci) {
+	    break;
 	}
     }
 
@@ -777,116 +817,249 @@ static gretlopt get_short_opts (char *line, int ci, int *err)
     return ret;
 }
 
-/* Apparatus for setting and retrieving parameters associated
-   with command options, as in --opt=val.  
-*/
-
-typedef struct optparm_ optparm;
-
-struct optparm_ {
-    int ci;
-    gretlopt opt;
-    char *val;
+enum {
+    OPT_SETOPT  = 1 << 0,
+    OPT_PERSIST = 1 << 1
 };
 
-static optparm *optparms;
-static int n_parms;
+/* The following apparatus is used for 
 
-/* Note: the following is called at the start of parse_command_line(),
-   via gretl_cmd_clear().  Hopefully this ensures that we won't get an
-   accumulation of stale data in the record of option parameters.
+   (a) setting and retrieving parameters associated with
+       command options, as in --opt=val, and 
+
+   (b) storing options for a specified command via the
+       "setopt" command (with or without parameters).
 */
 
+typedef struct stored_opt_ stored_opt;
+
+struct stored_opt_ {
+    int ci;       /* index of the associated command */
+    gretlopt opt; /* option flag */
+    char *val;    /* option parameter value, or NULL */
+    int flags;    /* may include OPT_SETOPT, OPT_PERSIST */
+    int fd;       /* "function depth" at which registered */
+};
+
+static stored_opt *optinfo;
+static int n_stored_opts;
+
+static void clear_one_option (stored_opt *so)
+{
+    free(so->val);
+    so->val = NULL;
+    so->ci = 0;
+    so->opt = 0;
+    so->flags = 0;
+}
+
 /**
  * clear_option_params:
  *
- * Clears any ancillary parameter values currently associated 
- * with gretl command/option pairs.
+ * Clears any non-persistent ancillary parameter values
+ * currently associated with gretl command/option pairs.
  */
 
 void clear_option_params (void)
 {
-    int i;
+    int i, fd = gretl_function_depth();
 
 #if OPTDEBUG > 1
-    fprintf(stderr, "clearing option params\n");
+    fprintf(stderr, "clear option params\n");
 #endif
 
-    for (i=0; i<n_parms; i++) {
-	free(optparms[i].val);
+    for (i=0; i<n_stored_opts; i++) {
+	if (optinfo[i].fd == fd && !(optinfo[i].flags & OPT_PERSIST)) {
+	    clear_one_option(&optinfo[i]);
+	}
     }
-    free(optparms);
-    optparms = NULL;
-    n_parms = 0;
 }
 
-static optparm *matching_optparm (int ci, gretlopt opt)
+void destroy_option_params_at_level (int level)
+{
+    int i, n = n_stored_opts;
+
+#if OPTDEBUG
+    fprintf(stderr, "destroy_option_params_at_level: %d\n", level);
+#endif
+
+    for (i=0; i<n_stored_opts; i++) {
+	if (optinfo[i].fd == level) {
+	    clear_one_option(&optinfo[i]);
+	    n--;
+	}
+    }
+
+    if (n == 0) {
+	free(optinfo);
+	optinfo = NULL;
+    }
+
+    n_stored_opts = n;
+}
+
+/* clean up any options set via "setopt" */
+
+void setopt_cleanup (void)
+{
+    int i, n = n_stored_opts;
+
+#if OPTDEBUG
+    fprintf(stderr, "setopt_cleanup\n");
+#endif
+
+    for (i=0; i<n_stored_opts; i++) {
+	if (optinfo[i].flags & OPT_SETOPT) {
+	    clear_one_option(&optinfo[i]);
+	    n--;
+	}
+    }
+
+    if (n == 0) {
+	free(optinfo);
+	optinfo = NULL;
+    }
+
+    n_stored_opts = n;    
+}
+
+/* unconditionally clean up the entire optinfo stack */
+
+void option_params_cleanup (void)
 {
     int i;
 
-    for (i=0; i<n_parms; i++) {
-	if (optparms[i].ci == ci && optparms[i].opt == opt) {
-	    return &optparms[i];
+#if OPTDEBUG
+    fprintf(stderr, "option_params_cleanup\n");
+#endif
+
+    for (i=0; i<n_stored_opts; i++) {
+	free(optinfo[i].val);
+    }
+
+    free(optinfo);
+    optinfo = NULL;
+    n_stored_opts = 0;
+}
+
+static stored_opt *matching_stored_opt (int ci, gretlopt opt)
+{
+    int i, fd = gretl_function_depth();
+
+#if OPTDEBUG
+    fprintf(stderr, "matching_stored_opt: ci=%d, fd=%d, opt=%d, n_stored=%d\n",
+	    ci, fd, opt, n_stored_opts);
+#endif
+
+    for (i=0; i<n_stored_opts; i++) {
+	if (optinfo[i].ci == ci && 
+	    optinfo[i].opt == opt &&
+	    optinfo[i].fd == fd) {
+	    return &optinfo[i];
 	}
     }
 
     return NULL;
 }
 
-/* for a given @ci, @opt pair, determine its status with regard
-   to a parameter value: 0 = not allowed, 1 = allowed, 2 = required
+static stored_opt *empty_stored_opt_slot (void)
+{
+    int i;
+
+    for (i=0; i<n_stored_opts; i++) {
+	if (optinfo[i].ci == 0) {
+	    return &optinfo[i];
+	}
+    }
+
+    return NULL;
+}
+
+/* for a given (@ci, @opt) pair, determine its status with regard
+   to a parameter value: 0 = not allowed, 1 = optional, 2 = required
 */
 
 static int option_parm_status (int ci, gretlopt opt)
 {
-    int i;
+    int i, got_ci = 0;
 
     for (i=0; gretl_opts[i].ci != 0; i++) {
-	if (gretl_opts[i].ci == ci && gretl_opts[i].o == opt) {
-	    return gretl_opts[i].parminfo;
+	if (gretl_opts[i].ci == ci) {
+	    if (gretl_opts[i].o == opt) {
+		return gretl_opts[i].parminfo;
+	    }
+	    got_ci = 1;
+	} else if (got_ci) {
+	    break;
 	}
     }
 
     return 0;
 }
 
-static int real_push_option_param (int ci, gretlopt opt, char *val,
-				   int checked)
+static int real_push_option (int ci, gretlopt opt, char *val,
+			     int checked, int flags)
 {
-    optparm *op;
+    int fd = gretl_function_depth();
     int n, err = 0;
+    stored_opt *so;
 
     if (!checked && option_parm_status(ci, opt) == OPT_NO_PARM)  {
 	return E_DATA;
     }
 
-    n = n_parms + 1;
+#if OPTDEBUG 
+    fprintf(stderr, "push_option_param: ci=%d, fd=%d, opt=%d,"
+	    " val='%s'\n", ci, fd, opt, val);
+#endif
 
-    op = matching_optparm(ci, opt);
-    if (op != NULL) {
-	/* got a match for the ci, opt pair already */
-	free(op->val);
-	op->val = val;
+    so = matching_stored_opt(ci, opt);
+    if (so != NULL) {
+	/* got a match for the (ci, opt) pair already */
+#if OPTDEBUG 
+	fprintf(stderr, " push_option_param: replacing\n");
+#endif
+	if (!(flags & OPT_SETOPT)) {
+	    free(so->val);
+	    so->val = val;
+	}
+	so->flags = flags;
 	return 0;
     }
 
-    op = realloc(optparms, n * sizeof *op);
+    so = empty_stored_opt_slot();
+    if (so != NULL) {
+	/* re-use a vacant slot */
+#if OPTDEBUG 
+	fprintf(stderr, " push_option_param: reusing empty\n");
+#endif
+	so->ci = ci;
+	so->opt = opt;
+	so->val = val;
+	so->flags = flags;
+	so->fd = fd;
+	return 0;
+    }    
 
-    if (op == NULL) {
+    /* so we have to extend the array */
+    n = n_stored_opts + 1;
+    so = realloc(optinfo, n * sizeof *so);
+
+    if (so == NULL) {
 	err = E_ALLOC;
     } else {
-	optparms = op;
-	op = &optparms[n-1];
-	op->ci = ci;
-	op->opt = opt;
-	op->val = val;
-	n_parms = n;
-    }
-
 #if OPTDEBUG 
-    fprintf(stderr, "push_option_param: val='%s', n_parms=%d, "
-	    "err = %d\n", op->val, n_parms, err);
+	fprintf(stderr, " push_option_param: appending\n");
 #endif
+	optinfo = so;
+	so = &optinfo[n-1];
+	so->ci = ci;
+	so->opt = opt;
+	so->val = val;
+	so->flags = flags;
+	so->fd = fd;
+	n_stored_opts = n;
+    }
 
     return err;
 }
@@ -908,7 +1081,7 @@ static int real_push_option_param (int ci, gretlopt opt, char *val,
 
 int push_option_param (int ci, gretlopt opt, char *val)
 {
-    return real_push_option_param(ci, opt, val, 0);
+    return real_push_option(ci, opt, val, 0, 0);
 }
 
 /**
@@ -923,9 +1096,9 @@ int push_option_param (int ci, gretlopt opt, char *val)
 
 const char *get_optval_string (int ci, gretlopt opt)
 {
-    optparm *op = matching_optparm(ci, opt);
+    stored_opt *so = matching_stored_opt(ci, opt);
 
-    return (op != NULL)? op->val : NULL;
+    return (so != NULL)? so->val : NULL;
 }
 
 /**
@@ -940,14 +1113,14 @@ const char *get_optval_string (int ci, gretlopt opt)
 
 double get_optval_double (int ci, gretlopt opt)
 {
-    optparm *op = matching_optparm(ci, opt);
+    stored_opt *so = matching_stored_opt(ci, opt);
     double ret = NADBL;
 
-    if (op != NULL && op->val != NULL) {
-	if (numeric_string(op->val)) {
-	    ret = dot_atof(op->val);
+    if (so != NULL && so->val != NULL) {
+	if (numeric_string(so->val)) {
+	    ret = dot_atof(so->val);
 	} else {
-	    ret = gretl_scalar_get_value(op->val, NULL);
+	    ret = gretl_scalar_get_value(so->val, NULL);
 	}
     }
 
@@ -969,28 +1142,149 @@ double get_optval_double (int ci, gretlopt opt)
 
 int get_optval_int (int ci, gretlopt opt, int *err)
 {
-    optparm *op = matching_optparm(ci, opt);
+    stored_opt *so = matching_stored_opt(ci, opt);
     int status = option_parm_status(ci, opt);
     int ret = 0;
 
-    if (op != NULL && op->val != NULL) {
-	if (integer_string(op->val)) {
-	    ret = atoi(op->val);
+    if (so != NULL && so->val != NULL) {
+	if (integer_string(so->val)) {
+	    ret = atoi(so->val);
 	} else {
-	    double x = gretl_scalar_get_value(op->val, err);
+	    double x = gretl_scalar_get_value(so->val, err);
 
 	    if (!*err) {
 		ret = (int) x;
 	    }
 	}
     } else if (status == 2 && err != NULL) {
-	/* parameter is required */
+	const char *longopt = get_longopt(ci, opt);
+
+	gretl_errmsg_sprintf(_("The option '--%s' requires a parameter"), 
+			     longopt);
 	*err = E_DATA;
     }
 
     return ret;
 }
 
+static void clear_options_for_command (int ci)
+{
+    int i, fd = gretl_function_depth();
+
+    for (i=0; i<n_stored_opts; i++) {
+	if (optinfo[i].fd == fd && optinfo[i].ci == ci) {
+	    clear_one_option(&optinfo[i]);
+	}
+    }
+}
+
+static void set_stored_options (int ci, gretlopt opt, int flags)
+{
+    int i, got_ci = 0;
+
+#if OPTDEBUG
+    fprintf(stderr, "setting stored options: ci = %d\n", ci);
+#endif
+
+    for (i=0; gretl_opts[i].o != 0; i++) {
+	if (ci == gretl_opts[i].ci) {
+	    if (opt & gretl_opts[i].o) {
+		real_push_option(ci, gretl_opts[i].o, NULL, 1, flags);
+	    }
+	    got_ci = 1;
+	} else if (got_ci) {
+	    break;
+	}
+    }
+}
+
+/* Apparatus for pre-selecting options for a specified command,
+   using "setopt" */
+
+int set_options_for_command (gretlopt opt, const char *cmdword,
+			     const char *param)
+{
+    int ci = gretl_command_number(cmdword);
+    int flags = OPT_SETOPT;
+    int clear = 0;
+    int err = 0;
+
+    if (ci == 0 || ci == SETOPT) {
+	gretl_errmsg_sprintf(_("field '%s' in command is invalid"),
+			     cmdword);
+	return E_DATA;
+    }
+
+    if (param != NULL && *param != '\0') {
+	if (!strcmp(param, "persist")) {
+	    flags |= OPT_PERSIST;
+	} else if (!strcmp(param, "clear")) {
+	    clear = 1;
+	} else {
+	    gretl_errmsg_sprintf(_("field '%s' in command is invalid"),
+				 param);
+	    return E_DATA;
+	}
+    }
+
+    if (clear) {
+	clear_options_for_command(ci);
+    } else if (opt == 0) {
+	err = E_ARGS;
+    } else {
+	set_stored_options(ci, opt, flags);
+    }
+
+    return err;
+}
+
+static void maybe_get_stored_options (int ci, gretlopt *popt)
+{
+    int i, fd = gretl_function_depth();
+
+    for (i=0; i<n_stored_opts; i++) {
+	if (optinfo[i].fd == fd && optinfo[i].ci == ci) {
+#if OPTDEBUG
+	    fprintf(stderr, "ci %d: got stored opt %d\n", ci, optinfo[i].opt);
+#endif
+	    *popt |= optinfo[i].opt;
+	}
+    }
+}
+
+/* When writing data as gdt this is called conditionally on OPT_Z
+   being given, but when writing gdtb it is called unconditionally
+   (since the format of the latter is a zipfile in all cases).
+   The effects are then:
+
+   gdt: no compression applied by default; if --gzipped given
+   with no parameter, zlib level 1 used; if --gzipped given
+   with parameter, param value respected within range 0 to 9.
+
+   gdtb: zlib level 1 used by default, levels 0 to 9 can be
+   specified via --gzipped + param.
+*/
+
+int get_compression_option (int ci)
+{
+    stored_opt *so = matching_stored_opt(ci, OPT_Z);
+    int level = 0;
+
+    if (so == NULL || so->val == NULL) {
+	/* use zlib level 1 by default */
+	level = 1;
+    } else {
+	level = atoi(so->val);
+	if (level < 0) {
+	    level = 0;
+	} else if (level > 9) {
+	    level = 9;
+	}
+    }
+
+    return level;
+}
+
 /* below: called via GUI */
 
 /**
@@ -1013,7 +1307,7 @@ int set_optval_double (int ci, gretlopt opt, double x)
     sprintf(s, "%g", x);
     gretl_pop_c_numeric_locale();
 
-    return real_push_option_param(ci, opt, gretl_strdup(s), 1);
+    return real_push_option(ci, opt, gretl_strdup(s), 1, 0);
 }
 
 /**
@@ -1032,7 +1326,7 @@ int set_optval_int (int ci, gretlopt opt, int k)
 {
     char *s = gretl_strdup_printf("%d", k);
 
-    return real_push_option_param(ci, opt, s, 1);
+    return real_push_option(ci, opt, s, 1, 0);
 }
 
 /**
@@ -1049,7 +1343,7 @@ int set_optval_int (int ci, gretlopt opt, int k)
 
 int set_optval_string (int ci, gretlopt opt, const char *s)
 {
-    return real_push_option_param(ci, opt, gretl_strdup(s), 1);
+    return real_push_option(ci, opt, gretl_strdup(s), 1, 0);
 }
 
 /* valid_long_opt: this is (semi-) public because we have need of
@@ -1059,7 +1353,7 @@ int set_optval_string (int ci, gretlopt opt, const char *s)
 gretlopt valid_long_opt (int ci, const char *s, OptStatus *status)
 {
     gretlopt opt = OPT_NONE;
-    int i;
+    int i, got_ci = 0;
 
     *status = 0;
 
@@ -1077,10 +1371,14 @@ gretlopt valid_long_opt (int ci, const char *s, OptStatus *status)
 
     /* start by looking for an exact match */
     for (i=0; gretl_opts[i].o != 0; i++) {
-	if (ci == gretl_opts[i].ci && 
-	    !strcmp(s, gretl_opts[i].longopt)) {
-	    opt = gretl_opts[i].o;
-	    *status = gretl_opts[i].parminfo;
+	if (ci == gretl_opts[i].ci) {
+	    if (!strcmp(s, gretl_opts[i].longopt)) {
+		opt = gretl_opts[i].o;
+		*status = gretl_opts[i].parminfo;
+		break;
+	    }
+	    got_ci = 1;
+	} else if (got_ci) {
 	    break;
 	}
     }
@@ -1090,6 +1388,7 @@ gretlopt valid_long_opt (int ci, const char *s, OptStatus *status)
 	int optlen, slen = strlen(s);
 	int nmatch = 0;
 
+	got_ci = 0;
 	for (i=0; gretl_opts[i].o != 0; i++) {
 	    if (ci == gretl_opts[i].ci) {
 		optlen = strlen(gretl_opts[i].longopt);
@@ -1098,6 +1397,9 @@ gretlopt valid_long_opt (int ci, const char *s, OptStatus *status)
 		    *status = gretl_opts[i].parminfo;
 		    nmatch++;
 		}
+		got_ci = 1;
+	    } else if (got_ci) {
+		break;
 	    }
 	}
 	if (nmatch > 1) {
@@ -1251,7 +1553,7 @@ static int handle_optval (char *s, int ci, gretlopt opt, int status)
 
     if (!err) {
 	if (status == OPT_NEEDS_PARM || status == OPT_ACCEPTS_PARM) {
-	    err = real_push_option_param(ci, opt, val, 1);
+	    err = real_push_option(ci, opt, val, 1, 0);
 	} else {
 	    err = E_DATA;
 	}
@@ -1282,7 +1584,7 @@ static gretlopt get_long_opts (char *line, int ci, int *err)
 	match = 0;
 	*longopt = '\0';
 
-#if OPTDEBUG
+#if OPTDEBUG > 1
 	fprintf(stderr, "get_long_opts: s = '%s'\n", s);
 #endif
 
@@ -1351,7 +1653,7 @@ static void get_cmdword (const char *line, char *word)
 	    line += 2;
 	}
 	sscanf(line, "%8s", word);
-    } else if (!sscanf(line, "%*s <- %8s", word)) {
+    } else if (sscanf(line, "%*s <- %8s", word) != 1) {
 	sscanf(line, "%8s", word);
     }
 }
@@ -1388,22 +1690,14 @@ static int end_block_ci (const char *s)
 	    return FOREIGN;
 	} else if (!strcmp(word, "system")) {
 	    return SYSTEM;
+	} else if (!strcmp(word, "mpi")) {
+	    return MPI;
 	}
     }
 
     return END;
 }
 
-#define ar1_alias(s) (!strcmp(s, "corc") || \
-		      !strcmp(s, "hilu") || \
-		      !strcmp(s, "pwe"))
-
-#define ols_alias(s) (!strcmp(s, "hccm"))
-
-#define smpl_alias(s) (!strcmp(s, "sample"))
-
-#define modtest_alias(s) (!strcmp(s, "lmtest"))
-
 /**
  * get_gretl_options:
  * @line: command line to parse.
@@ -1421,17 +1715,27 @@ gretlopt get_gretl_options (char *line, int *err)
     gretlopt oflags = 0;
     gretlopt opt;
     char cmdword[9] = {0};
+    char *dashp = NULL;
     int endblock = 0;
+    int storing = 0;
     int ci, myerr = 0;
 
     if (err != NULL) {
 	*err = 0;
     }
-
-    if (strlen(line) < 2 || *line == '#' || strchr(line, '-') == NULL) {
+    
+    if (strlen(line) < 2 || *line == '#') {
 	return 0;
     }
 
+    dashp = strchr(line, '-');
+    if (dashp == NULL) {
+	/* no options in this line itself */
+	if (n_stored_opts == 0) {
+	    return 0;
+	}
+    }
+
 #if OPTDEBUG > 1
     fprintf(stderr, "get_gretl_options, starting: '%s'\n", line);
 #endif
@@ -1441,32 +1745,29 @@ gretlopt get_gretl_options (char *line, int *err)
     if (!strcmp(cmdword, "catch")) {
 	*cmdword = '\0';
 	get_cmdword(line + 6, cmdword);
-    }
+    } else if (!strcmp(cmdword, "setopt")) {
+	storing = 1;
+	*cmdword = '\0';
+	get_cmdword(line + 7, cmdword);
+    }	
 
     if (!strcmp(cmdword, "end")) {
 	endblock = 1;
 	ci = end_block_ci(line);
-    } else if (ar1_alias(cmdword)) {
-	ci = AR1;
-    } else if (smpl_alias(cmdword)) {
-	ci = SMPL;
-    } else if (ols_alias(cmdword)) {
-	ci = OLS;
-    } else if (modtest_alias(cmdword)) {
-	ci = MODTEST;
     } else {
 	ci = gretl_command_number(cmdword);
     }
 
 #if OPTDEBUG
-    fprintf(stderr, "get gretl_options: '%s' -> ci = %d\n", cmdword, ci);
+    fprintf(stderr, "get gretl_options: '%s' -> ci=%d (storing = %d)\n", 
+	    cmdword, ci, storing);
 #endif
 
     /* some commands do not take a "flag", and "-%c" may have
        some other meaning */
-    if (ci == 0 || ci == GENR || ci == PRINTF) {
+    if (ci == 0 || ci == GENR || ci == PRINTF || ci == SETOPT) {
 	return oflags;
-    } else if (!endblock && (ci == GMM || ci == MLE || ci == NLS)) {
+    } else if (!endblock && !storing && (ci == GMM || ci == MLE || ci == NLS)) {
 	return oflags;
     }
 
@@ -1475,6 +1776,15 @@ gretlopt get_gretl_options (char *line, int *err)
 	return oflags;
     }
 
+    if (!storing) {
+	maybe_get_stored_options(ci, &oflags);
+    }
+
+    if (dashp == NULL) {
+	/* no point in continuing */
+	return oflags;
+    }
+
     if (ci != SET && ci != SETINFO && ci != SMPL &&
 	ci != TABPRINT && ci != EQNPRINT) {
 	/* try for short-form options (e.g. "-o"): but note that
@@ -1562,7 +1872,7 @@ const char *print_flags (gretlopt oflags, int ci)
 {
     const char *parm;
     gretlopt opt;
-    int i;
+    int i, got_ci;
 
     if (ci == FUNDEBUG) {
 	/* options are for internal use only */
@@ -1597,23 +1907,29 @@ const char *print_flags (gretlopt oflags, int ci)
 	oflags &= ~OPT_Q; /* handled */
     }
 
+    got_ci = 0;
     for (i=0; gretl_opts[i].ci != 0; i++) {
-	opt = gretl_opts[i].o;
-	if (ci == gretl_opts[i].ci && (oflags & opt)) {
-	    pprintf(flagprn, " --%s", gretl_opts[i].longopt);
-	    if (gretl_opts[i].parminfo) {
-		parm = get_optval_string(ci, opt);
-		if (parm != NULL && *parm != '\0') {
-		    print_option_param(parm, flagprn);
+	if (ci == gretl_opts[i].ci) {
+	    opt = gretl_opts[i].o;
+	    if (oflags & opt) {
+		pprintf(flagprn, " --%s", gretl_opts[i].longopt);
+		if (gretl_opts[i].parminfo) {
+		    parm = get_optval_string(ci, opt);
+		    if (parm != NULL && *parm != '\0') {
+			print_option_param(parm, flagprn);
+		    }
 		}
 	    }
+	    got_ci = 1;
+	} else if (got_ci) {
+	    break;
 	}
     }
 
     return gretl_print_get_buffer(flagprn);
 }
 
-void option_flags_cleanup (void)
+void option_printing_cleanup (void)
 {
     gretl_print_destroy(flagprn);
     flagprn = NULL;
@@ -1750,3 +2066,20 @@ int inapplicable_option_error (int ci, gretlopt opt)
     gretl_errmsg_sprintf("%s: inapplicable option", s);
     return E_BADOPT;
 }
+
+void debug_print_option_flags (const char *msg, gretlopt opt)
+{
+    fprintf(stderr, "%s: opt = %d\n", msg, opt);
+
+    if (opt) {
+	char c = 'A';
+	int i;
+
+	for (i=OPT_A; i<=OPT_Y; i*=2) {
+	    if (opt & i) {
+		fprintf(stderr, " includes OPT_%c (%d)\n", c, i);
+	    }
+	    c++;
+	}
+    }    
+}
