@@ -24,12 +24,9 @@
 #include "libset.h"
 #include "forecast.h"
 #include "gretl_func.h"
-#include "matrix_extra.h"
 #include "uservar.h"
-#include "gretl_string_table.h"
 
 #include <time.h>
-#include <glib.h>
 
 #define PDEBUG 0
 
@@ -160,6 +157,7 @@ void gui_script_logo (PRN *prn)
     pprintf(prn, _("gretl version %s\n"), GRETL_VERSION);
     print_time(timestr);
     pprintf(prn, "%s: %s\n", _("Current session"), timestr);
+    pputc(prn, '\n');
 }
 
 /* ----------------------------------------------------- */
@@ -360,7 +358,7 @@ void print_freq (const FreqDist *freq, int varno, const DATASET *dset,
     int total, valid, missing;
     char word[64];
     double f, cumf = 0;
-    const char **labels = NULL;
+    int strvals = 0;
 
     if (freq == NULL) {
 	return;
@@ -381,23 +379,26 @@ void print_freq (const FreqDist *freq, int varno, const DATASET *dset,
     } 
 
     if (varno > 0 && dset != NULL && is_string_valued(dset, varno)) {
-	int n_labels;
-
-	labels = series_get_string_vals(dset, varno, &n_labels);
-	if (n_labels != freq->numbins) {
-	    labels = NULL;
-	}
+	strvals = 1;
     }
 
     if (freq->discrete) {
+	const char *s;
+	double dval;
+
 	pputs(prn, _("\n          frequency    rel.     cum.\n\n"));
 
 	for (k=0; k<=K; k++) {
-	    if (labels != NULL) {
-		*word = '\0';
-		gretl_utf8_strncat(word, labels[k], 8);
-	    } else {
-		sprintf(word, "%4g", freq->midpt[k]);
+	    dval = freq->midpt[k];
+	    *word = '\0';
+	    if (strvals) {
+		s = series_get_string_for_value(dset, varno, dval);
+		if (s != NULL) {
+		    gretl_utf8_strncat(word, s, 8);
+		}
+	    }
+	    if (*word == '\0') {
+		sprintf(word, "%4g", dval);
 	    }
 	    pputs(prn, word);
 	    nlw = 10 - strlen(word);
@@ -531,16 +532,16 @@ void print_freq (const FreqDist *freq, int varno, const DATASET *dset,
 
 void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn)
 {
+    int collabels = (tab->clabels != NULL);
+    int rowlabels = (tab->rlabels != NULL);
     int r = tab->rows;
     int c = tab->cols;
     double x, y;
     int n5 = 0;
     double ymin = 1.0e-7;
     double pearson = 0.0;
-    int i, j;
     char lbl[64];
-    int has_clabels = (tab->clabels!=NULL);
-    int has_rlabels = (tab->rlabels!=NULL);
+    int i, j;
 
     if (*tab->rvarname != '\0' && *tab->cvarname != '\0') {
 	pputc(prn, '\n');
@@ -551,12 +552,12 @@ void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn)
 	pputs(prn, "\n       ");
     }
 
-    if(has_rlabels){
+    if (rowlabels) {
 	pputs(prn, "    ");
     }
 
     for (j=0; j<c; j++) {
-	if (!has_clabels) {
+	if (!collabels) {
 	    pprintf(prn, "[%4g]", tab->cval[j]);
 	} else {
 	    *lbl = '\0';
@@ -568,21 +569,18 @@ void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn)
     pprintf(prn,"  %s\n  \n", _("TOT."));
 
     for (i=0; i<r; i++) {
-
 	if (tab->rtotal[i] > 0) {
-	    if (!has_rlabels) {
+	    if (!rowlabels) {
 		pprintf(prn, "[%4g] ", tab->rval[i]);
 	    } else {
 		*lbl = '\0';
 		gretl_utf8_strncat(lbl, tab->rlabels[i], 8);
 		pprintf(prn, "[%8s] ", lbl);
 	    }
-
 	    for (j=0; j<c; j++) {
-		if (has_clabels) {
+		if (collabels) {
 		    pputs(prn, "    ");
 		}
-
 		if (tab->ctotal[j]) {
 		    if (tab->f[i][j] || (opt & OPT_Z)) {
 			if (opt & (OPT_C | OPT_R)) {
@@ -599,19 +597,19 @@ void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn)
 			pputs(prn, "      ");
 		    }
 		} 
-
 		if (!na(pearson)) {
-		    y = (double) (tab->rtotal[i] * tab->ctotal[j]) / tab->n;
-		    x = (double) tab->f[i][j] - y;
+		    y = ((double) tab->rtotal[i] * tab->ctotal[j]) / tab->n;
 		    if (y < ymin) {
 			pearson = NADBL;
 		    } else {
+			x = (double) tab->f[i][j] - y;
 			pearson += x * x / y;
-			if (y >= 5) n5++;
+			if (y >= 5) {
+			    n5++;
+			}
 		    }
 		}
 	    }
-
 	    if (opt & OPT_C) {
 		x = 100.0 * tab->rtotal[i] / tab->n;
 		pprintf(prn, "%5.1f%%\n", x);
@@ -623,12 +621,12 @@ void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn)
 
     pputc(prn, '\n');
     pputs(prn, _("TOTAL  "));
-    if(has_rlabels){
+    if (rowlabels) {
 	pputs(prn, "    ");
     }
 
     for (j=0; j<c; j++) {
-	if (has_clabels) {
+	if (collabels) {
 	    pputs(prn, "    ");
 	}
 	if (opt & OPT_R) {
@@ -648,6 +646,7 @@ void print_xtab (const Xtab *tab, gretlopt opt, PRN *prn)
     }
 
     if (na(pearson)) {
+	pputc(prn, '\n');
 	pprintf(prn, _("Pearson chi-square test not computed: some "
 		       "expected frequencies were less\n"
 		       "than %g\n"), ymin);
@@ -1795,57 +1794,15 @@ void maybe_list_vars (const DATASET *dset, PRN *prn)
     }
 }
 
-static void print_varlist (const char *name, const int *list, 
-			   const DATASET *dset, PRN *prn)
-{
-    int i, v, len = 0;
-
-    if (list[0] == 0) {
-	pprintf(prn, " %s\n", _("list is empty"));
-    } else {
-	len += pprintf(prn, " %s: ", name);
-	for (i=1; i<=list[0]; i++) {
-	    v = list[i];
-	    if (v == LISTSEP) {
-		len += pputs(prn, "; ");
-	    } else if (v >= 0 && v < dset->v) {
-		len += pprintf(prn, "%s ", dset->varname[v]);
-	    } else {
-		len += pprintf(prn, "%d ", v);
-	    }
-	    if (i < list[0] && len > 68) {
-		pputs(prn, " \\\n ");
-		len = 0;
-	    }
-	}
-	pputc(prn, '\n');
-    }
-}
-
 static void print_listed_objects (const char *s, 
 				  const DATASET *dset, 
 				  PRN *prn)
 {
-    const gretl_matrix *m;
-    const int *list;
-    const char *p;
-    gretl_bundle *b;
     char *name;
     int err = 0;
 
     while ((name = gretl_word_strdup(s, &s, OPT_NONE, &err)) != NULL) {
-	if (gretl_is_scalar(name)) {
-	    print_scalar_by_name(name, prn);
-	} else if ((m = get_matrix_by_name(name)) != NULL) {
-	    gretl_matrix_print_to_prn(m, name, prn);
-	} else if ((list = get_list_by_name(name)) != NULL) {
-	    print_varlist(name, list, dset, prn);
-	} else if ((b = get_bundle_by_name(name)) != NULL) {
-	    gretl_bundle_print(b, prn);
-	} else if ((p = get_string_by_name(name)) != NULL) {
-	    pputs(prn, p);
-	    pputc(prn, '\n');
-	} 
+	print_user_var_by_name(name, dset, prn);
 	free(name);
     }
 }
