@@ -23,10 +23,12 @@
 #include "libset.h"
 #include "gretl_func.h"
 #include "matrix_extra.h"
+#include "gretl_string_table.h"
 #include "uservar.h"
 
 #include <errno.h>
 
+#define SPRINTF_TARGLEN 64
 #define PSDEBUG 0
 
 /* For the point of "x *= (1 + 0x1p-52)" below, see
@@ -274,7 +276,7 @@ get_printf_format_chunk (const char *s, int *fc,
 			 int *len, int *wstar, int *pstar,
 			 int *err)
 {
-    const char *cnvchars = "eEfgGduxs";
+    const char *cnvchars = "eEfgGduxsv";
     const char *numchars = "0123456789";
     char *chunk = NULL;
     const char *p = s;
@@ -469,6 +471,11 @@ static int print_arg (const char **pfmt, const char **pargs,
 
     free(arg);
 
+    if (!err && fc == 'v' && m == NULL) {
+	/* 'v' for variant is for matrices only */
+	err = E_PARSE;
+    }
+
     if (err) {
 	goto bailout;
     } 
@@ -476,7 +483,7 @@ static int print_arg (const char **pfmt, const char **pargs,
     if (got_scalar && na(x)) {
 	fc = fmt[flen - 1] = 's';
 	str = gretl_strdup("NA");
-    }	
+    }
 
     /* do the actual printing */
 
@@ -538,43 +545,32 @@ static int print_arg (const char **pfmt, const char **pargs,
     } else if (free_v) {
 	dataset_drop_last_variables(dset, 1);
     }
+
+#if PSDEBUG
+    fprintf(stderr, "print_arg: returning %d\n", err);
+#endif	
     
     return err;
 }
 
-static user_var *get_stringvar_target (const char *targ, 
-				       DATASET *dset,
-				       PRN *prn,
-				       int *err)
+static int handle_sprintf_output (const char *targ,
+				  const char *buf,
+				  PRN *prn)
 {
-    user_var *uvar = NULL;
+    char *tmp = malloc(strlen(targ) + 8);
+    int err = 0;
 
-    if (*targ == '\0') {
-	*err = E_PARSE;
+    if (tmp == NULL) {
+	err = E_ALLOC;
     } else {
-	uvar = get_user_var_by_name(targ);
-	if (uvar != NULL && 
-	    user_var_get_type(uvar) != GRETL_TYPE_STRING) {
-	    *err = E_TYPES;
-	    return NULL;
-	}
-	if (uvar == NULL) {
-	    char genline[64];
+	gretl_insert_builtin_string("pstmp", buf);
+	sprintf(tmp, "%s=$pstmp", targ);
+	err = generate(tmp, NULL, OPT_NONE, prn);
+	free(tmp);
+	gretl_insert_builtin_string("pstmp", NULL);
+    }    
 
-	    sprintf(genline, "string %s = \"\"", targ);
-	    *err = generate(genline, dset, OPT_NONE, prn);
-	    if (!*err) {
-		uvar = get_user_var_of_type_by_name(targ, GRETL_TYPE_STRING);
-		
-	    }
-	}
-	if (uvar == NULL) {
-	    gretl_errmsg_sprintf(_("%s: not a string variable"), targ);
-	    *err = E_DATA;
-	}
-    }
-
-    return uvar;
+    return err;
 }
 
 /* supports both printf and sprintf */
@@ -583,31 +579,22 @@ static int real_do_printf (const char *targ, const char *format,
 			   const char *args, DATASET *dset, 
 			   PRN *inprn, int *nchars, int t)
 {
+    int ci = (targ != NULL) ? SPRINTF : PRINTF;
     PRN *prn = NULL;
-    user_var *uvar = NULL;
-    int sp, err = 0;
+    int err = 0;
 
     gretl_error_clear();
 
 #if PSDEBUG
-    fprintf(stderr, "do_printf:\n");
+    fprintf(stderr, "real_do_printf:\n");
     fprintf(stderr, " targ = '%s'\n", targ);
     fprintf(stderr, " format = '%s'\n", format);
     fprintf(stderr, " args = '%s'\n", args);
 #endif
 
-    sp = targ != NULL;
-
-    if (sp) {
-	/* sprintf: we need a target string variable */
-	uvar = get_stringvar_target(targ, dset, inprn, &err);
-	if (err) {
-	    return err;
-	}
-    }
-
-    /* Even for printf we'll buffer the output locally in 
-       case there's an error part way through the printing.
+    /* Even when doing printf, we'll buffer the output
+       locally in case there's an error part-way through
+       the printing.
     */
     prn = gretl_print_new(GRETL_PRINT_BUFFER, &err);
     if (err) {
@@ -633,8 +620,10 @@ static int real_do_printf (const char *targ, const char *format,
 	    }
 	}
 
-	if (q != NULL && *q != '\0') {
-	    pprintf(prn, "\nunmatched argument '%s'", q);
+	if (!err && q != NULL && *q != '\0') {
+	    pprintf(inprn, "\n%s: ", gretl_command_word(ci));
+	    pprintf(inprn, _("unprocessed argument(s): '%s'"), q);
+	    pputc(prn, '\n');
 	    err = E_PARSE;
 	}
     }
@@ -646,18 +635,14 @@ static int real_do_printf (const char *targ, const char *format,
 	    *nchars = strlen(buf);
 	}
 
-	if (sp) {
-	    char *tmp = gretl_strdup(buf);
-
-	    if (tmp == NULL) {
-		err = E_ALLOC;
-	    } else {
-		user_var_replace_value(uvar, tmp);
-	    }
+	if (ci == SPRINTF) {
+	    /* sprintf: output to a string variable */
+	    handle_sprintf_output(targ, buf, prn);
 	} else {
+	    /* plain printf: output to @inprn */
 	    pputs(inprn, buf);
 	}
-    } else if (!sp) {
+    } else if (ci == PRINTF) {
 	pputc(inprn, '\n');
     }
 
@@ -670,7 +655,7 @@ static int real_do_printf (const char *targ, const char *format,
  * do_printf:
  * @targ: target string (for sprintf); should be NULL for printf.
  * @format: format string.
- * @args: list of argument as string.
+ * @args: list of arguments as string.
  * @dset: dataset struct.
  * @prn: printing struct.
  * @nchars: location to receive number of characters printed.
@@ -892,6 +877,7 @@ static int scan_scalar (char *targ, const char **psrc,
 			int fc, int width, DATASET *dset, 
 			int *ns)
 {
+    int intscan = (fc == 'd');
     char *endp = NULL;
     long k = 0;
     double x = 0;
@@ -899,7 +885,7 @@ static int scan_scalar (char *targ, const char **psrc,
 
     errno = 0;
 
-    if (fc == 'd') {
+    if (intscan) {
 	k = strtol(*psrc, &endp, 10);
     } else {
 	x = strtod(*psrc, &endp);
@@ -924,10 +910,13 @@ static int scan_scalar (char *targ, const char **psrc,
 	}
     }
 
+    /* "genr" operates in the C locale regardless */
+    gretl_push_c_numeric_locale();
+
     if (!errno && endp != *psrc) {
 	char *genline;
 
-	if (fc == 'd') {
+	if (intscan) {
 	    genline = gretl_strdup_printf("%s=%ld", targ, k);
 	} else {
 	    genline = gretl_strdup_printf("%s=%.16g", targ, x);
@@ -942,12 +931,16 @@ static int scan_scalar (char *targ, const char **psrc,
 	free(genline);
     }
 
+    /* match the above call */
+    gretl_pop_c_numeric_locale();
+
     *psrc = endp;
 
     return err;
 }
 
-static int scan_matrix (char *targ, const char **psrc, int rows, int *ns)
+static int scan_matrix (char *targ, const char **psrc, 
+			int rows, int *ns)
 {
     const char *src = *psrc;
     char *endp = NULL;
@@ -1012,7 +1005,8 @@ static int scan_matrix (char *targ, const char **psrc, int rows, int *ns)
 	if (err) {
 	    gretl_matrix_free(m);
 	} else {
-	    *ns += 1;
+	    /* number of elements scanned */
+	    *ns += m->rows * m->cols;
 	}
     }
 
@@ -1093,12 +1087,12 @@ static int scan_arg (const char **psrc, const char **pfmt, const char **pargs,
  * do_sscanf:
  * @src: the source string.
  * @format: the format string.
- * @src: the arguments, as composite string.
+ * @args: the arguments, as composite string.
  * @dset: dataset struct.
  * @n_items: location to receive the number of scanned items.
  *
  * Implements a somewhat simplified version of C's sscanf()
- * for use in gretl scripts.
+ * for use in hansl scripts.
  *
  * Returns: 0 on success, non-zero on error.
  */
@@ -1150,118 +1144,94 @@ int do_sscanf (const char *src, const char *format, const char *args,
     return err;
 }
 
-static char *formulate_sprintf_call (const char *s, int *err)
+static int get_array_index (const char *s, char *targ)
 {
-    char *call = NULL;
-    char vname[VNAMELEN] = {0};
-    int n = gretl_namechar_spn(s);
-
-    /* @s should look something like 'myvar "%g", x' */
+    const char *p = strchr(s, ' ');
     
-    if (n == 0 || n >= VNAMELEN) {
-	*err = E_PARSE;
-    } else {
-	/* m: allow for "sprintf()" and terminating NUL */
-	int m = strlen(s) + 11;
+    if (p != NULL && *(p-1) == ']') {
+	size_t n = strlen(targ) + (p - s) + 1;
 
-	strncat(vname, s, n);
-	s += n;
-	call = calloc(m, 1);
-	if (call == NULL) {
-	    *err = E_ALLOC;
-	} else {
-	    s += strspn(s, " ");
-	    if (*s == ',') {
-		sprintf(call, "sprintf(%s%s)", vname, s);
-	    } else {
-		sprintf(call, "sprintf(%s,%s)", vname, s);
-	    }
+	if (n < SPRINTF_TARGLEN) {
+	    /* let's not overflow @targ */
+	    strncat(targ, s, p - s);
+	    return 1;
 	}
     }
 
-    if (!*err && *vname != '\0') {
-	get_stringvar_target(vname, NULL, NULL, err);
-	if (*err) {
-	    free(call);
-	    call = NULL;
-	}
-    }
-	
-
-    return call;
+    return 0;
 }
 
-/* If USE_COMMAND_FORM is defined we don't pass printf and sprintf,
-   given in command form, through "genr" but just process them here.
-   This is faster, and although it gives a little less flexibility
-   this was the expected behaviour in gretl <= 1.9.13.
-*/
-
-#define USE_COMMAND_FORM 1
-
-#if USE_COMMAND_FORM
-
-/* split line into format and args, copying both parts */
-
-static int split_printf_line (const char *s, char *targ, int *sp,
-			      char **format, char **args)
+static int get_printf_format (const char **ps, char **format)
 {
-    const char *p;
+    const char *s = *ps;
     int n, err = 0;
 
-    *sp = 0;
-
-    if (!strncmp(s, "printf ", 7)) {
-	s += 7;
-    } else if (!strncmp(s, "sprintf ", 8)) {
-	s += 8;
-	*sp = 1;
-    }
-
-#if PSDEBUG
-    fprintf(stderr, "split_printf_line: s='%s', sp=%d\n", s, *sp);
-#endif
-
-    if (*sp) {
-	/* need a target name */
-	s += strspn(s, " ");
-	err = extract_varname(targ, s, &n);
-	if (!err && n == 0) {
-	    err = E_PARSE;
-	}
-	if (err) {
-	    return err;
-	}
-	s += n;
-	/* allow comma after target */
-	s += strspn(s, " ");
-	if (*s == ',') {
-	    s++;
-	}
-    }
-
-    s += strspn(s, " ");
     if (*s != '"') {
+	/* At present we insist that the format is a string literal:
+	   FIXME accept a string variable/expression?
+	*/
 	return E_PARSE;
     }
 
     s++;
-    p = s;
     n = double_quote_position(s);
 
     if (n < 0) {
 	/* malformed format string */
-	return E_PARSE;
-    } else if (n == 0) {
-	/* empty format string */
-	return 0;
+	err = E_PARSE;
+    } else if (n > 0) {
+	/* non-empty format */
+	*format = gretl_strndup(s, n);
+	if (*format == NULL) {
+	    err = E_ALLOC;
+	}
+	s += n;
     }
 
-    s += n;
+    /* advance pointer if needed */
+    *ps = s;
 
-    *format = gretl_strndup(p, n);
-    if (*format == NULL) {
-	return E_ALLOC;
+    return err;
+}
+
+/* split @s into format and args, copying both parts */
+
+static int split_printf_line (const char *s, char *targ, int ci,
+			      char **format, char **args)
+{
+    int n, err = 0;
+
+    if (ci == SPRINTF) {
+	/* sprintf: we need a "target": this should be the name
+	   of an existing string variable, an unused name that
+	   can be given to a new string variable, or perhaps a
+	   string indicating an element of an array of strings,
+	   such as "S[4]" or a more complex variant thereof.
+	   We start by looking for a legit variable name, and
+	   then allow appending an [...] index expression.
+	*/
+	err = extract_varname(targ, s, &n);
+	if (err || n == 0) {
+	    return E_PARSE;
+	}
+	s += n;
+	/* allow for attached [...] */
+	if (*s == '[' && get_array_index(s, targ)) {
+	    s += strcspn(s, " ");
+	}	
+	/* allow (not required) comma after target,
+	   and skip white space */
+	s += strspn(s, " ");
+	if (*s == ',') {
+	    s++;
+	    s += strspn(s, " ");
+	}
+    }
+
+    /* We should now find a format string */
+    err = get_printf_format(&s, format);
+    if (err) {
+	return err;
     }
 
     s++;
@@ -1270,31 +1240,30 @@ static int split_printf_line (const char *s, char *targ, int *sp,
     if (*s != ',') {
 	/* empty args */
 	*args = NULL;
-	return 0;
-    }
-
-    s++;
-    s += strspn(s, " ");
-
-    *args = gretl_strdup(s);
-    if (*args == NULL) {
-	return E_ALLOC;
+    } else {
+	s++;
+	s += strspn(s, " ");
+	*args = gretl_strdup(s);
+	if (*args == NULL) {
+	    return E_ALLOC;
+	}
     }
 
     return 0;
 }
 
-static int printf_driver (const char *line, DATASET *dset, PRN *prn)
+static int printf_driver (const char *line, int ci, DATASET *dset, 
+			  PRN *prn)
 {
+    char targ[SPRINTF_TARGLEN];
     char *format = NULL;
     char *args = NULL;
-    char targ[VNAMELEN];
-    int err, sp = 0;
+    int err;
 
-    err = split_printf_line(line, targ, &sp, &format, &args);
+    err = split_printf_line(line, targ, ci, &format, &args);
 
     if (!err) {
-	char *vname = sp ? targ : NULL;
+	char *vname = (ci == SPRINTF)? targ : NULL;
 
 	err = do_printf(vname, format, args, dset, prn, NULL);
     }
@@ -1305,67 +1274,49 @@ static int printf_driver (const char *line, DATASET *dset, PRN *prn)
     return err;
 }
 
-#endif
-
-/* apparatus to support the command-forms of printf, sprintf 
-   and sscanf */
-
-int do_printscan_command (const char *line, DATASET *dset, PRN *prn)
+static int sscanf_driver (const char *line, DATASET *dset, PRN *prn)
 {
     static int warned;
     char *tmp;
-    int ci, err = 0;
-
-#if USE_COMMAND_FORM
-    if (strncmp(line, "ss", 2)) {
-	/* not sscanf() */
-	return printf_driver(line, dset, prn);
-    }
-#endif
-
-    if (!strncmp(line, "printf ", 7)) {
-	ci = PRINTF;
-	line += 7;
-    } else if (!strncmp(line, "sprintf ", 8)) {
-	ci = SPRINTF;
-	line += 8;
-    } else if (!strncmp(line, "sscanf ", 7)) {
-	ci = SSCANF;
-	line += 7;
-    } else {
-	return E_DATA;
-    }
-
-    if (ci == SSCANF && !warned) {
+    int err;
+    
+    if (!warned) {
 	pputs(prn, "*** \"sscanf\": obsolete command, please use the "
 	      "function of the same name\n");
 	warned = 1;
     }
 
-    line += strspn(line, " ");
+    tmp = malloc(strlen(line) + 9);
 
-    if (ci == PRINTF) {
-	tmp = gretl_strdup_printf("printf(%s)", line);
-    } else if (ci == SSCANF) {
-	tmp = gretl_strdup_printf("sscanf(%s)", line);
-    } else {
-	tmp = formulate_sprintf_call(line, &err);
-    }
-
-    if (!err && tmp == NULL) {
+    if (tmp == NULL) {
 	err = E_ALLOC;
+    } else {
+	sprintf(tmp, "sscanf(%s)", line);
+	err = generate(tmp, dset, OPT_O, prn);
+	free(tmp);
     }
 
-#if PSDEBUG
-    fprintf(stderr, "do_printscan_command: revised='%s'\n", tmp);
-#endif
+    return err;
+}
 
-    if (!err) {
-	err = generate(tmp, dset, OPT_U, prn);
+/* apparatus to support the command-forms of printf, sprintf 
+   and sscanf */
+
+int do_printscan_command (int ci, const char *line, 
+			  DATASET *dset, PRN *prn)
+{
+    int err;
+
+    /* skip the command word and following space */
+    line += strcspn(line, " ");
+    line += strspn(line, " ");
+    
+    if (ci == PRINTF || ci == SPRINTF) {
+	err = printf_driver(line, ci, dset, prn);
+    } else {
+	err = sscanf_driver(line, dset, prn);
     }
 
-    free(tmp);
-
     return err;
 }
 
