@@ -28,7 +28,14 @@ double hac_weight (int kern, int h, int i);
 
 double qs_hac_weight (double bt, int i);
 
-int newey_west_bandwidth (const gretl_matrix *f, int kern, int *h, double *bt);
+int maybe_limit_VAR_coeffs (gretl_matrix *A,
+			    gretl_matrix *Y,
+			    gretl_matrix *X,
+			    gretl_matrix *E);
+
+int newey_west_bandwidth (const gretl_matrix *f, 
+			  const gretl_matrix *w,
+			  int kern, int *h, double *bt);
 
 gretl_matrix *HAC_XOX (const gretl_matrix *uhat, const gretl_matrix *X,
 		       VCVInfo *vi, int *err);
