@@ -33,6 +33,17 @@
 #define SUBDEBUG 0
 #define FULLDEBUG 0
 
+typedef enum {
+    SUBSAMPLE_NONE,
+    SUBSAMPLE_DROP_MISSING,
+    SUBSAMPLE_DROP_EMPTY,
+    SUBSAMPLE_DROP_WKENDS,
+    SUBSAMPLE_USE_DUMMY,
+    SUBSAMPLE_BOOLEAN,
+    SUBSAMPLE_RANDOM,
+    SUBSAMPLE_UNKNOWN
+} SubsampleMode;
+
 /*
   The purpose of the static pointers below: When the user subsamples
   the current dataset in a non-trivial way -- i.e., by selecting cases
@@ -388,7 +399,11 @@ static int update_case_markers (const DATASET *dset)
 {
     int err = 0;
 
-    if (dset->markers && !fullset->markers) {
+    if (fullset->markers == DAILY_DATE_STRINGS) {
+	; /* don't mess with them */
+    } else if (dated_daily_data(fullset)) {
+	; /* again, don't mess! */
+    } else if (dset->markers && !fullset->markers) {
 	dataset_allocate_obs_markers(fullset);
 	if (fullset->S == NULL) {
 	    err = 1;
@@ -631,7 +646,7 @@ int restore_full_sample (DATASET *dset, ExecState *state)
 	*/
 	if (state->submask != NULL) {
 	    err = restrict_sample_from_mask(state->submask, dset,
-					    OPT_NONE);
+					    OPT_NONE, NULL);
 	} else {
 	    int t1min, t2max;
 
@@ -664,6 +679,75 @@ static int overlay_masks (char *targ, const char *src, int n)
     return sn;
 }
 
+static int make_weekday_mask (const DATASET *dset, char *mask)
+{
+    if (!dated_daily_data(dset) || 
+	dset->markers != 0 || dset->pd < 6) {
+	return E_PDWRONG;
+    } else {
+	char datestr[OBSLEN];
+	int t, wd;
+
+	for (t=0; t<dset->n; t++) {
+	    ntodate(datestr, t, dset);
+	    wd = weekday_from_date(datestr);
+	    mask[t] = (wd >= 1 && wd <= 5);
+	}
+	
+	return 0;
+    }
+}
+
+/* write into @mask: 0 for observations at which _all_ variables
+   (or all variables in @list, if @list is non-NULL) have missing 
+   values, 1 for all other observations (that have at least one 
+   non-NA). (Refinement: if no list is given, we ignore the generic
+   variables "time" and "index".)
+*/
+
+static int 
+make_empty_mask (const int *list, const DATASET *dset, char *mask)
+{
+    int i, vi, t, vt;
+
+    if (list != NULL && list[0] > 0) {
+	/* check specified list of variables */
+	for (t=0; t<dset->n; t++) {
+	    mask[t] = 0;
+	    for (i=1; i<=list[0]; i++) {
+		vi = list[i];
+		if (!na(dset->Z[vi][t])) {
+		    mask[t] = 1;
+		    break;
+		}
+	    }
+	}
+    } else {
+	/* check (almost) all variables */
+	vt = current_series_index(dset, "time");
+	vi = current_series_index(dset, "index");
+
+	for (t=0; t<dset->n; t++) {
+	    mask[t] = 0;
+	    for (i=1; i<dset->v; i++) {
+		if (!series_is_hidden(dset, i) &&
+		    i != vt && i != vi &&
+		    !na(dset->Z[i][t])) {
+		    mask[t] = 1;
+		    break;
+		}
+	    }
+	}
+    }
+
+    return 0;
+}
+
+/* write into @mask: 0 for observations at which _any_ variable
+   (or any variable in @list, if @list is non-NULL) has a missing 
+   value, 1 for all other observations.
+*/
+
 static int 
 make_missing_mask (const int *list, const DATASET *dset, char *mask)
 {
@@ -695,7 +779,7 @@ make_missing_mask (const int *list, const DATASET *dset, char *mask)
     }
 
     return 0;
-} 
+}
 
 static int copy_dummy_to_mask (char *mask, const double *x, int n)
 {
@@ -914,6 +998,32 @@ int backup_full_dataset (DATASET *dset)
     return 0;
 }
 
+static void destroy_full_dataset (DATASET *dset)
+{
+    if (fullset != NULL) {
+	if (fullset->varname == dset->varname) {
+	    fullset->varname = NULL;
+	}
+	if (fullset->varinfo == dset->varinfo) {
+	    fullset->varinfo = NULL;
+	}
+	if (fullset->descrip == dset->descrip) {
+	    fullset->descrip = NULL;
+	}
+	destroy_dataset(fullset);
+	fullset = NULL;
+    }
+
+    dset->varname = NULL;
+    dset->varinfo = NULL;
+    dset->descrip = NULL;
+
+    free_Z(dset);
+    clear_datainfo(dset, CLEAR_SUBSAMPLE); 
+
+    peerset = NULL;
+}
+
 int complex_subsampled (void)
 {
     return (fullset != NULL && fullset->Z != NULL);
@@ -1025,8 +1135,11 @@ copy_data_to_subsample (DATASET *subset, const DATASET *dset,
 	}
     }
 
-    strcpy(subset->stobs, "1");
-    sprintf(subset->endobs, "%d", subset->n);
+    if (subset->stobs[0] == '\0' || subset->endobs[0] == '\0') {
+	/* impose simple default if not already handled */
+	strcpy(subset->stobs, "1");
+	sprintf(subset->endobs, "%d", subset->n);
+    }
 }
 
 int get_restriction_mode (gretlopt opt)
@@ -1041,7 +1154,11 @@ int get_restriction_mode (gretlopt opt)
 	mode = SUBSAMPLE_RANDOM;
     } else if (opt & OPT_O) {
 	mode = SUBSAMPLE_USE_DUMMY;
-    } 
+    } else if (opt & OPT_A) {
+	mode = SUBSAMPLE_DROP_EMPTY;
+    } else if (opt & OPT_W) {
+	mode = SUBSAMPLE_DROP_WKENDS;
+    }
 
     return mode;
 }
@@ -1161,6 +1278,126 @@ make_panel_submask (char *mask, const DATASET *dset, int *err)
     return np;
 }
 
+static int add_daily_date_strings (char *selected, 
+				   const DATASET *dset,
+				   DATASET *subset)
+{
+    int err, t, i = 0;
+    
+    err = dataset_allocate_obs_markers(subset);
+
+    if (!err) {
+	subset->markers = DAILY_DATE_STRINGS;
+	for (t=0; t<dset->n; t++) {
+	    if (selected[t]) {
+		ntodate(subset->S[i++], t, dset);
+	    }
+	}
+    }
+
+    return err;
+}
+
+/* See if sub-sampling dated daily data leaves a useable daily dataset
+   (as in dropping weekends and/or holidays).
+*/
+
+static int try_for_daily_subset (char *selected, 
+				 const DATASET *dset,
+				 DATASET *subset,
+				 gretlopt *optp)
+{
+    char datestr[OBSLEN];
+    long ed, ed0 = 0, edbak = 0;
+    int t, wd, ngaps = 0;
+    int delta, mon_delta;
+    int delta_max = 0;
+    int started = 0;
+    int any_sat = 0;
+    int newpd = 5;
+    int err = 0;
+
+    if (dset->pd > 5) {
+	/* first pass: look at weekend status of included obs */
+	for (t=0; t<dset->n; t++) {
+	    if (selected[t]) {
+		wd = weekday_from_date(datestr);
+		if (wd == 0) {
+		    /* got a Sunday: result must be 7-day */
+		    newpd = 7;
+		    break;
+		} else if (wd == 6) {
+		    any_sat = 1;
+		}
+	    }
+	}
+	if (newpd == 5 && any_sat) {
+	    /* no Sundays, but got a Saurday: result
+	       must be 6-day data */
+	    newpd = 6;
+	}
+    }
+
+    /* determine expected "delta days" for Mondays */
+    mon_delta = (newpd == 7) ? 1 : (newpd == 6)? 2 : 3;
+
+    /* second pass: count the calendar gaps */
+    for (t=0; t<dset->n; t++) {
+	if (selected[t]) {
+	    ntodate(datestr, t, dset);
+	    wd = weekday_from_date(datestr);
+	    ed = get_epoch_day(datestr);
+	    if (started) {
+		delta = ed - edbak;
+		ngaps += (wd == 1)? (delta > mon_delta) : (delta > 1);
+		if (delta > delta_max) {
+		    delta_max = delta;
+		}
+	    } else {
+		ed0 = ed;
+		started = 1;
+	    }
+	    edbak = ed;
+	}
+    }
+
+    /* Now, we should probably restrict the proportion of
+       missing days for this treatment: for exclusion of
+       non-trading days 10 percent should be generous. 
+       But we'll also require that the maximum "daily
+       delta" be less than 10.
+    */
+
+    if (delta_max < 10) {
+	double keepfrac = subset->n / (double) dset->n;
+	double keepmin = 0.9 * newpd / (double) dset->pd;
+
+#if SUBDEBUG
+	fprintf(stderr, "*** daily: delta_max=%d, keepfrac=%.4f\n"
+		"keepmin=%.4f, ngaps=%d, newpd=%d\n", delta_max, keepfrac,
+		keepmin, ngaps, newpd);
+#endif
+
+	if (keepfrac >= keepmin) {
+	    if (dset->S == NULL && ngaps > 0) {
+		err = add_daily_date_strings(selected, dset, subset);
+		/* flag not to destroy subset date strings */
+		*optp |= OPT_P;
+	    }
+	    if (!err) {
+		subset->structure = TIME_SERIES;
+		subset->pd = newpd;
+		subset->sd0 = (double) ed0;
+		subset->t2 = subset->n - 1;
+		ntodate(subset->stobs, 0, subset);
+		ntodate(subset->endobs, subset->t2, subset);
+	    }
+	}
+    }
+
+    return err;
+}
+
 #define needs_string_arg(m) (m == SUBSAMPLE_RANDOM || \
 	                     m == SUBSAMPLE_USE_DUMMY || \
                              m == SUBSAMPLE_BOOLEAN)
@@ -1189,8 +1426,12 @@ make_restriction_mask (int mode, const char *s,
 
     /* construct subsample mask in one of several possible ways */
 
-    if (mode == SUBSAMPLE_DROP_MISSING) {   
+    if (mode == SUBSAMPLE_DROP_MISSING) { 
 	err = make_missing_mask(list, dset, mask);
+    } else if (mode == SUBSAMPLE_DROP_EMPTY) {
+	err = make_empty_mask(list, dset, mask);
+    } else if (mode == SUBSAMPLE_DROP_WKENDS) {
+	err = make_weekday_mask(dset, mask);
     } else if (mode == SUBSAMPLE_RANDOM) {
 	err = make_random_mask(s, oldmask, dset, mask);
     } else if (mode == SUBSAMPLE_USE_DUMMY) {
@@ -1265,9 +1506,11 @@ static void finalize_panel_subset (DATASET *subset)
 */
 
 int 
-restrict_sample_from_mask (char *mask, DATASET *dset, gretlopt opt)
+restrict_sample_from_mask (char *mask, DATASET *dset, gretlopt opt,
+			   int *n_dropped)
 {
     DATASET *subset;
+    gretlopt zopt = OPT_R;
     int err = 0;
 
     if (mask == RESAMPLED) {
@@ -1323,15 +1566,25 @@ restrict_sample_from_mask (char *mask, DATASET *dset, gretlopt opt)
 	    /* time series for single panel unit */
 	    subset->structure = SPECIAL_TIME_SERIES;
 	}
+    } else if (dated_daily_data(dset)) {
+	/* see if we can preserve daily time series */
+	err = try_for_daily_subset(mask, dset, subset, &zopt);
+    }
+
+    if (!err) {
+	/* set up the sub-sampled dataset */
+	err = start_new_Z(subset, zopt);
     }
 
-    /* set up the sub-sampled datainfo */
-    err = start_new_Z(subset, OPT_R);
     if (err) { 
 	free(subset);
 	return err;
     }
 
+    if (n_dropped != NULL) {
+	*n_dropped = dset->n - subset->n;
+    }
+
 #if SUBDEBUG
     fprintf(stderr, "started new Z for subset (v=%d, n=%d, Z=%p)\n", 
 	    subset->v, subset->n, (void *) subset->Z);
@@ -1356,9 +1609,13 @@ restrict_sample_from_mask (char *mask, DATASET *dset, gretlopt opt)
     /* copy across data (and case markers, if any) */
     copy_data_to_subsample(subset, dset, dset->v, mask);
 
-    err = backup_full_dataset(dset);
-
-    subset->submask = copy_subsample_mask(mask, &err);
+    if (opt & OPT_T) {
+	/* --permanent */
+	destroy_full_dataset(dset);
+    } else {
+	err = backup_full_dataset(dset);
+	subset->submask = copy_subsample_mask(mask, &err);
+    }
 
     /* switch pointers */
     *dset = *subset;
@@ -1477,6 +1734,14 @@ static int restriction_uses_obs (const char *s)
 
 #endif
 
+static void destroy_restriction_string (DATASET *dset)
+{
+    if (dset->restriction != NULL) {
+	free(dset->restriction);
+	dset->restriction = NULL;
+    }
+}
+
 /* Make a string representing the sample restriction. This is
    for reporting purposes. Note that in some cases the
    incoming @restr string may be NULL, for instance if the
@@ -1489,10 +1754,7 @@ static int make_restriction_string (DATASET *dset, char *old,
     char *s = NULL;
     int n = 0, err = 0;
 
-    if (dset->restriction != NULL) {
-	free(dset->restriction);
-	dset->restriction = NULL;
-    }
+    destroy_restriction_string(dset);
 
     if (old != NULL) {
 	n = strlen(old);
@@ -1529,10 +1791,6 @@ static int make_restriction_string (DATASET *dset, char *old,
 	dset->restriction = s;
     }
 
-    if (old != NULL) {
-	free(old);
-    }
-
     return err;
 }
 
@@ -1543,6 +1801,8 @@ static int make_restriction_string (DATASET *dset, char *old,
  * @list: list of variables in case of OPT_M (or %NULL).  
  * @opt: option flags.
  * @prn: printing apparatus.
+ * @n_dropped: location to receive count of dropped
+ * observations, or NULL.
  *
  * Sub-sample the data set, based on the criterion of skipping all
  * observations with missing data values (OPT_M); or using as a mask a
@@ -1566,18 +1826,48 @@ static int make_restriction_string (DATASET *dset, char *old,
 
 int restrict_sample (const char *line, const int *list,
 		     DATASET *dset, ExecState *state, 
-		     gretlopt opt, PRN *prn)
+		     gretlopt opt, PRN *prn,
+		     int *n_dropped)
 {
     char *oldrestr = NULL;
     char *oldmask = NULL;
     char *mask = NULL;
     int free_oldmask = 0;
+    int permanent = 0;
     int mode, err = 0;
     
     if (dset == NULL || dset->Z == NULL) {
 	return E_NODATA;
     }
 
+    /* We'll accept the redundant combination of the options --dummy
+       (OPT_O) and --restrict (OPT_R), but other than that the options
+       --dummy, 
+       --restrict, 
+       --no-missing (OPT_M), 
+       --no-all-missing (OPT_A) and
+       --random (OPT_N) 
+       are all mutually incompatible.
+    */
+    if (incompatible_options(opt, OPT_O | OPT_M | OPT_A | OPT_N) ||
+	incompatible_options(opt, OPT_R | OPT_M | OPT_A | OPT_N)) {
+	return E_BADOPT;
+    }
+
+    if (opt & OPT_T) {
+	/* permanent */
+	if (gretl_function_depth() > 0) {
+	    /* can't permanently shrink the dataset within a function */
+	    gretl_errmsg_set(_("The dataset cannot be modified at present"));
+	    return E_DATA;
+	} else if (!(opt & (OPT_O | OPT_M | OPT_A | OPT_N | OPT_R))) {
+	    /* we need some kind of restriction flag */
+	    return E_BADOPT;
+	} else {
+	    permanent = 1;
+	}
+    }
+
     gretl_error_clear();
 
 #if FULLDEBUG || SUBDEBUG
@@ -1641,14 +1931,14 @@ int restrict_sample (const char *line, const int *list,
     }
 
     if (!err && mask != NULL) {
+	int contiguous = 0;
 	int t1 = 0, t2 = 0;
-	int contig = 0;
 
-	if (mode != SUBSAMPLE_RANDOM) {
-	    contig = mask_contiguous(mask, dset, &t1, &t2);
+	if (!permanent && mode != SUBSAMPLE_RANDOM) {
+	    contiguous = mask_contiguous(mask, dset, &t1, &t2);
 	}
 
-	if (contig) {
+	if (contiguous) {
 	    /* The specified subsample consists of contiguous
 	       observations, so we'll just adjust the range, avoiding
 	       the overhead of creating a parallel dataset.
@@ -1662,7 +1952,8 @@ int restrict_sample (const char *line, const int *list,
 #if SUBDEBUG
 	    fprintf(stderr, "restrict sample: using mask\n");
 #endif
-	    err = restrict_sample_from_mask(mask, dset, opt);
+	    err = restrict_sample_from_mask(mask, dset, opt, 
+					    n_dropped);
 	}
     }
 
@@ -1672,7 +1963,13 @@ int restrict_sample (const char *line, const int *list,
 	free(oldmask);
     }
 
-    make_restriction_string(dset, oldrestr, line, mode);
+    if (permanent) {
+	destroy_restriction_string(dset);
+    } else {
+	make_restriction_string(dset, oldrestr, line, mode);
+    }
+
+    free(oldrestr);
 
     return err;
 }
@@ -2186,9 +2483,11 @@ static void pd_string (char *str, const DATASET *dset)
 	case 52:
 	    strcpy(str, _("weekly")); break;
 	case 5:
+	    strcpy(str, _("daily (5 days)")); break;
 	case 6:
+	    strcpy(str, _("daily (6 days)")); break;
 	case 7:
-	    strcpy(str, _("daily")); break;
+	    strcpy(str, _("daily (7 days)")); break;
 	case 10:
 	    strcpy(str, _("decennial")); break;
 	default:
@@ -2246,10 +2545,15 @@ void print_sample_status (const DATASET *dset, PRN *prn)
 
 	pprintf(prn, "%s: %d\n", _("Number of cross-sectional units"), nu);
 	pprintf(prn, "%s: %d\n", _("Number of time periods"), dset->pd);
-    }	
-    pprintf(prn, "%s: %s - %s (n = %d)\n", _("Full range"), 
-	    dset->stobs, dset->endobs, dset->n);
-    print_sample_obs(dset, prn); 
+    }
+    if (dset->t1 == 0 && dset->t2 == dset->n - 1) {
+	pprintf(prn, "%s: %s - %s (n = %d)\n", _("Range"), 
+		dset->stobs, dset->endobs, dset->n);
+    } else {
+	pprintf(prn, "%s: %s - %s (n = %d)\n", _("Range"), 
+		dset->stobs, dset->endobs, dset->n);
+	print_sample_obs(dset, prn);
+    } 
 }
 
 /**
