@@ -478,6 +478,9 @@ static int is_date_format (int fmt)
 	ret = 1;
     } else if (fmt >= 50 && fmt <= 58) {
 	ret = 1;
+    } else if (fmt == 164) {
+	/* FRED uses this */
+	ret = 1;
     }
 
     return ret;
@@ -555,6 +558,31 @@ static void check_for_date_formula (BiffQuery *q, wbook *book)
     }
 }
 
+/* Excel's NA() formula stores a "result" of 0.0, so if
+   we get a formula result of zero we need to check for
+   NA() and react accordingly. Use of NA() can be found
+   in XLS files downloaded from FRED.
+*/
+
+static int is_na_formula (unsigned char *ptr, wbook *book)
+{
+    int version = book->version;
+    int offset = (version < MS_BIFF_V5)? 16 : 20;
+    guint16 sz;
+
+    ptr += offset;
+    sz = MS_OLE_GET_GUINT16(ptr);
+
+    /* 0x42: indicates a built-in function
+       10 is the index of the BIFF NA() function
+    */
+    if (sz == 4 && ptr[2] == 0x42 && ptr[4] == 10) {
+	return 1;
+    } else {
+	return 0;
+    }
+}
+
 #undef FORMAT_INFO
 
 static int process_item (BiffQuery *q, wbook *book, xls_info *xi,
@@ -751,9 +779,11 @@ static int process_item (BiffQuery *q, wbook *book, xls_info *xi,
 	if (allocate_row_col(i, j, book, xi)) {
 	    return E_ALLOC;
 	} else {
+	    /* the result of the formula should be at offset 6 */
 	    ptr = q->data + 6;
 	    prow = xi->rows + i;
 	    if (ptr[6] == 0xff && ptr[7] == 0xff) {
+		/* string, boolean or error result */
 		unsigned char fcode = ptr[0];
 
 		if (fcode == 0x0) {
@@ -772,13 +802,16 @@ static int process_item (BiffQuery *q, wbook *book, xls_info *xi,
 		    prow->cells[j] = g_strdup("-999");
 		}
 	    } else {
-		/* floating-point */
+		/* should have a floating-point result */
 		val = get_le_double(ptr);
-		dbprintf(" floating-point value = %g\n", val);
-		if (isnan(val)) {
-		    fprintf(stderr, "Got a NaN\n");
+		if (val == 0.0 && is_na_formula(q->data, book)) {
+		    dbprintf("floating-point value = na()\n");
 		    prow->cells[j] = g_strdup("-999");
-		} else {
+		} else if (isnan(val)) {
+		    dbprintf("floating-point value is NaN\n");
+		    prow->cells[j] = g_strdup("-999");
+		} else {		    
+		    dbprintf(" floating-point value = %g\n", val);
 		    prow->cells[j] = g_strdup_printf("%.15g", val);
 		    if (i == book->row_offset + 1 && j == book->col_offset) {
 			/* could be a date formula? */
@@ -905,16 +938,11 @@ static int process_sheet (const char *filename, wbook *book, xls_info *xi,
 	return 1;
     }
 
-    result = ms_ole_stream_open(&stream, file, "/", "workbook");
+    result = ms_ole_stream_open_workbook(&stream, file);
 
     if (result != MS_OLE_ERR_OK) {
-	ms_ole_stream_close(&stream);
-	result = ms_ole_stream_open(&stream, file, "/", "book");
-	if (result != MS_OLE_ERR_OK) {
-	    ms_ole_stream_close(&stream);
-	    fputs("No book or workbook streams found\n", stderr);
-	    return 1;
-	}
+	ms_ole_destroy(&file);
+	return 1;
     }
 
     fputs("Reading file...\n", stderr);
@@ -1186,7 +1214,7 @@ static int check_all_varnames (wbook *book, xls_info *xi, PRN *prn)
 		/* pass along */
 		;
 	    } else {
-		int verr = check_imported_varname(test, i, j, prn);
+		int verr = check_imported_varname(test, 0, i, j, prn);
 
 		if (verr) {
 		    return verr;
@@ -1527,6 +1555,7 @@ int xls_get_data (const char *fname, int *list, char *sheetname,
 		  DATASET *dset, gretlopt opt, PRN *prn)
 {
     int gui = (opt & OPT_G);
+    gretlopt merge_opt = 0;
     wbook xbook;
     wbook *book = &xbook;
     xls_info xlsi;
@@ -1730,7 +1759,11 @@ int xls_get_data (const char *fname, int *list, char *sheetname,
 	reverse_data(newset, prn);
     }
 
-    err = merge_or_replace_data(dset, &newset, opt, prn);
+    if (merge && (opt & OPT_T)) {
+	merge_opt = OPT_T;
+    }
+
+    err = merge_or_replace_data(dset, &newset, merge_opt, prn);
 
     if (!err && !merge) {
 	dataset_add_import_info(dset, fname, GRETL_XLS);
