@@ -332,7 +332,7 @@ static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
 			    ; /* keep going */
 			} else {
 			    err = check_imported_varname(sheet->varname[i],
-							 r, c, prn);
+							 i, r, c, prn);
 			}
 		    } else if (i == 0) {
 			/* first column, not first row */
@@ -664,6 +664,7 @@ int gnumeric_get_data (const char *fname, int *list, char *sheetname,
 	int i, j, t;
 	int ts_markers = 0;
 	int merge = (dset->Z != NULL);
+	gretlopt merge_opt = 0;
 	char **ts_S = NULL;
 	int blank_cols = 0;
 	int missvals = 0;
@@ -793,7 +794,11 @@ int gnumeric_get_data (const char *fname, int *list, char *sheetname,
 	    import_ts_check(newset);
 	}
 
-	err = merge_or_replace_data(dset, &newset, opt, prn);
+	if (merge && (opt & OPT_T)) {
+	    merge_opt = OPT_T;
+	}
+
+	err = merge_or_replace_data(dset, &newset, merge_opt, prn);
 
 	if (!err && !merge) {
 	    dataset_add_import_info(dset, fname, GRETL_GNUMERIC);
