@@ -150,7 +150,7 @@ struct spss_data_ {
     gretlopt opt;              /* option flags */
     int nvars;                 /* number of variables (really, 'elements') */
     int nobs;                  /* number of observations ('cases') */
-    int swapends;              /* reversing endianness? (1/0) */
+    int swapends;              /* reversing endianness? (0/1) */
     int encoding;              /* encoding for strings */
     int max_sv;                /* index of highest-numbered string variable */
     spss_var *vars;            /* info on individual variables */
@@ -230,7 +230,8 @@ static int sav_read_int32 (spss_data *sdat, int *err)
     int32_t ret = 0;
 
     *err = fread(&ret, sizeof ret, 1, sdat->fp) != 1;
-    if (*err && sdat->swapends) {
+
+    if (!*err && sdat->swapends) {
 	reverse_int(ret);
     }
 
@@ -1941,6 +1942,7 @@ int sav_get_data (const char *fname, DATASET *dset,
 	destroy_dataset(newset);
     } else {
 	int merge = (dset->Z != NULL);
+	gretlopt merge_opt = 0;
 
 	if (fix_varname_duplicates(newset)) {
 	    pputs(prn, _("warning: some variable names were duplicated\n"));
@@ -1958,7 +1960,11 @@ int sav_get_data (const char *fname, DATASET *dset,
 	    sdat.descrip = NULL;
 	}
 
-	err = merge_or_replace_data(dset, &newset, opt, prn);
+	if (merge && (opt & OPT_T)) {
+	    merge_opt = OPT_T;
+	}
+
+	err = merge_or_replace_data(dset, &newset, merge_opt, prn);
 
 	if (!err && !merge) {
 	    dataset_add_import_info(dset, fname, GRETL_SAV);
