@@ -4,7 +4,7 @@
    Based on stataread.c from the GNU R "foreign" package with the 
    following original info:
 
-     * $Id: stata_import.c,v 1.55 2013/04/21 13:26:02 jacklucchetti Exp $
+     * $Id: stata_import.c,v 1.57 2014/06/02 14:13:00 allin Exp $
   
      (c) 1999, 2000, 2001, 2002 Thomas Lumley. 
      2000 Saikat DebRoy
@@ -601,9 +601,11 @@ static int read_dta_data (FILE *fp, DATASET *dset,
 	*pst = dta_make_string_table(types, nvar, nsv);
     }
 
-    /* names */
+    /* variable names */
     for (i=0; i<nvar && !err; i++) {
         stata_read_string(fp, namelen + 1, aname, &err);
+	/* try to fix possible bad encoding */
+	iso_to_ascii(aname);
 	pprintf(vprn, "variable %d: name = '%s'\n", i+1, aname);
 	if (check_varname(aname) && try_fix_varname(aname)) {
 	    err = 1;
@@ -952,6 +954,7 @@ int dta_get_data (const char *fname, DATASET *dset,
 	}	
     } else {
 	int merge = (dset->Z != NULL);
+	gretlopt merge_opt = 0;
 	int nvtarg = newset->v - 1;
 
 	if (nvread < nvtarg) {
@@ -967,7 +970,11 @@ int dta_get_data (const char *fname, DATASET *dset,
 	    gretl_string_table_destroy(st);
 	}
 
-	err = merge_or_replace_data(dset, &newset, opt, prn);
+	if (merge && (opt & OPT_T)) {
+	    merge_opt = OPT_T;
+	}
+
+	err = merge_or_replace_data(dset, &newset, merge_opt, prn);
     
 	if (!err && !merge) {
 	    dataset_add_import_info(dset, fname, GRETL_DTA);
