@@ -313,9 +313,11 @@ static int xlsx_set_varname (xlsx_info *xinfo, int i, const char *s,
 	err = E_DATA;
     } else {
 	*xinfo->dset->varname[i] = '\0';
-	strncat(xinfo->dset->varname[i], s, VNAMELEN - 1);
+	if (s != NULL) {
+	    strncat(xinfo->dset->varname[i], s, VNAMELEN - 1);
+	}
 	err = check_imported_varname(xinfo->dset->varname[i], 
-				     row, col, prn);
+				     i, row, col, prn);
     }
 
     return err;
@@ -682,6 +684,8 @@ static int xlsx_read_row (xmlNodePtr cur, xlsx_info *xinfo, PRN *prn)
 		    err = xlsx_set_value(xinfo, i, t, xval);
 		} else if (gotf) {
 		    xlsx_maybe_handle_formula(xinfo, formula, i, t);
+		} else if (row == xinfo->namerow) {
+		    err = xlsx_set_varname(xinfo, i, NULL, row, col, prn);
 		}
 	    } else if (stringcell == 2 && strval != NULL) {
 		free((char *) strval);
@@ -1358,9 +1362,9 @@ static int finalize_xlsx_import (DATASET *dset,
 				 gretlopt opt,
 				 PRN *prn)
 {
-    int merge, err;
+    int merge = (dset->Z != NULL);
+    int err;
 
-    merge = (dset->Z != NULL);
     err = import_prune_columns(xinfo->dset);
 
     if (!err) {
@@ -1387,7 +1391,12 @@ static int finalize_xlsx_import (DATASET *dset,
     }
 
     if (!err) {
-	err = merge_or_replace_data(dset, &xinfo->dset, opt, prn);
+	gretlopt merge_opt = 0;
+
+	if (merge && (opt & OPT_T)) {
+	    merge_opt = OPT_T;
+	}
+	err = merge_or_replace_data(dset, &xinfo->dset, merge_opt, prn);
     } 
 
     if (!err && !merge) {
